Type: Package
Package: RobinCar2
Title: ROBust INference for Covariate Adjustment in Randomized Clinical
        Trials
Version: 0.2.2
Date: 2026-01-09
Authors@R: 
    c(
      person("Liming", "Li", , "liming.li1@astrazeneca.com", role = c("aut", "cre"), comment = c(ORCID = "0009-0008-6870-0878")),
      person("Marlena", "Bannick", , "mnorwood@uw.edu", role="aut", comment=c(ORCID="0000-0001-6279-5978")),
      person("Daniel", "Sabanes Bove", , "daniel@rconis.com", role = "aut", comment = c(ORCID = "0000-0002-0176-9239")),
      person("Dong", "Xi", , "Dong.Xi1@gilead.com", role = "aut"),
      person("Ting", "Ye", , "tingye1@uw.edu", role="aut"),
      person("Yanyao", "Yi", , "yi_yanyao@lilly.com", role="aut"),
      person("Gregory", "Chen", , "xiangyi.gregory.chen@msd.com", role = "ctb"),
      person("Gilead Sciences, Inc.", role = c("cph", "fnd")),
      person("F. Hoffmann-La Roche AG", role = c("cph", "fnd")),
      person("Merck Sharp & Dohme, Inc.", role = c("cph", "fnd")),
      person("AstraZeneca plc", role = c("cph", "fnd")),
      person("Eli Lilly and Company", role = c("cph", "fnd")),
      person("Novartis Pharma AG", role = c("cph", "fnd")),
      person("The University of Washington", role = c("cph", "fnd"))
    )
Description: Performs robust estimation and inference when using covariate adjustment
  and/or covariate-adaptive randomization in randomized controlled trials.
  This package is trimmed to reduce the dependencies and validated to be used across industry.
  See "FDA's final guidance on covariate adjustment"<https://www.regulations.gov/docket/FDA-2019-D-0934>, Tsiatis (2008) <doi:10.1002/sim.3113>,
  Bugni et al. (2018) <doi:10.1080/01621459.2017.1375934>, Ye, Shao, Yi, and Zhao (2023)<doi:10.1080/01621459.2022.2049278>,
  Ye, Shao, and Yi (2022)<doi:10.1093/biomet/asab015>, Rosenblum and van der Laan (2010)<doi:10.2202/1557-4679.1138>,
  Wang et al. (2021)<doi:10.1080/01621459.2021.1981338>, Ye, Bannick, Yi, and Shao (2023)<doi:10.1080/24754269.2023.2205802>,
  and Bannick, Shao, Liu, Du, Yi, and Ye (2024)<doi:10.48550/arXiv.2306.10213>.
License: Apache License 2.0
URL: https://github.com/openpharma/RobinCar2/
BugReports: https://github.com/openpharma/RobinCar2/issues
Depends: R (>= 3.6)
Imports: checkmate, numDeriv, MASS, sandwich, stats, survival, utils
Suggests: knitr, rmarkdown, testthat (>= 3.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-01-09 02:59:08 UTC; root
Author: Liming Li [aut, cre] (ORCID: <https://orcid.org/0009-0008-6870-0878>),
  Marlena Bannick [aut] (ORCID: <https://orcid.org/0000-0001-6279-5978>),
  Daniel Sabanes Bove [aut] (ORCID:
    <https://orcid.org/0000-0002-0176-9239>),
  Dong Xi [aut],
  Ting Ye [aut],
  Yanyao Yi [aut],
  Gregory Chen [ctb],
  Gilead Sciences, Inc. [cph, fnd],
  F. Hoffmann-La Roche AG [cph, fnd],
  Merck Sharp & Dohme, Inc. [cph, fnd],
  AstraZeneca plc [cph, fnd],
  Eli Lilly and Company [cph, fnd],
  Novartis Pharma AG [cph, fnd],
  The University of Washington [cph, fnd]
Maintainer: Liming Li <liming.li1@astrazeneca.com>
Repository: CRAN
Date/Publication: 2026-01-09 06:50:02 UTC
Built: R 4.6.0; ; 2026-04-27 19:53:23 UTC; unix

Type: Package
Package: RobinCar2
Title: ROBust INference for Covariate Adjustment in Randomized Clinical
        Trials
Version: 0.2.4
Date: 2026-08-19
Authors@R: 
    c(
      person("Liming", "Li", , "liming.li1@astrazeneca.com", role = c("aut", "cre"), comment = c(ORCID = "0009-0008-6870-0878")),
      person("Marlena", "Bannick", , "mnorwood@uw.edu", role="aut", comment=c(ORCID="0000-0001-6279-5978")),
      person("Daniel", "Sabanes Bove", , "daniel@rconis.com", role = "aut", comment = c(ORCID = "0000-0002-0176-9239")),
      person("Dong", "Xi", , "Dong.Xi1@gilead.com", role = "aut"),
      person("Ting", "Ye", , "tingye1@uw.edu", role="aut"),
      person("Yanyao", "Yi", , "yi_yanyao@lilly.com", role="aut"),
      person("Gregory", "Chen", , "xiangyi.gregory.chen@msd.com", role = "ctb"),
      person("Gilead Sciences, Inc.", role = c("cph", "fnd")),
      person("F. Hoffmann-La Roche AG", role = c("cph", "fnd")),
      person("Merck Sharp & Dohme, Inc.", role = c("cph", "fnd")),
      person("AstraZeneca plc", role = c("cph", "fnd")),
      person("Eli Lilly and Company", role = c("cph", "fnd")),
      person("Novartis Pharma AG", role = c("cph", "fnd")),
      person("The University of Washington", role = c("cph", "fnd"))
    )
Description: Performs robust estimation and inference when using covariate adjustment
  and/or covariate-adaptive randomization in randomized controlled trials.
  This package is trimmed to reduce the dependencies and validated to be used across industry.
  See "FDA's final guidance on covariate adjustment"<https://www.regulations.gov/docket/FDA-2019-D-0934>, Tsiatis (2008) <doi:10.1002/sim.3113>,
  Bugni et al. (2018) <doi:10.1080/01621459.2017.1375934>, Ye, Shao, Yi, and Zhao (2023)<doi:10.1080/01621459.2022.2049278>,
  Ye, Shao, and Yi (2022)<doi:10.1093/biomet/asab015>, Rosenblum and van der Laan (2010)<doi:10.2202/1557-4679.1138>,
  Wang et al. (2021)<doi:10.1080/01621459.2021.1981338>, Ye, Bannick, Yi, and Shao (2023)<doi:10.1080/24754269.2023.2205802>,
  and Bannick, Shao, Liu, Du, Yi, and Ye (2024)<doi:10.48550/arXiv.2306.10213>.
License: Apache License 2.0
URL: https://github.com/openpharma/RobinCar2/
BugReports: https://github.com/openpharma/RobinCar2/issues
Depends: R (>= 3.6)
Imports: checkmate, numDeriv, MASS, sandwich, stats, survival, utils
Suggests: knitr, rmarkdown, speff2trial, testthat (>= 3.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
LazyData: true
Config/roxygen2/version: 8.1.0
NeedsCompilation: no
Packaged: 2026-08-21 01:57:33 UTC; root
Author: Liming Li [aut, cre] (ORCID: <https://orcid.org/0009-0008-6870-0878>),
  Marlena Bannick [aut] (ORCID: <https://orcid.org/0000-0001-6279-5978>),
  Daniel Sabanes Bove [aut] (ORCID:
    <https://orcid.org/0000-0002-0176-9239>),
  Dong Xi [aut],
  Ting Ye [aut],
  Yanyao Yi [aut],
  Gregory Chen [ctb],
  Gilead Sciences, Inc. [cph, fnd],
  F. Hoffmann-La Roche AG [cph, fnd],
  Merck Sharp & Dohme, Inc. [cph, fnd],
  AstraZeneca plc [cph, fnd],
  Eli Lilly and Company [cph, fnd],
  Novartis Pharma AG [cph, fnd],
  The University of Washington [cph, fnd]
Maintainer: Liming Li <liming.li1@astrazeneca.com>
Repository: CRAN
Date/Publication: 2026-08-22 17:30:02 UTC
Built: R 4.5.2; ; 2026-08-22 21:01:30 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(confint,prediction_cf)
S3method(confint,surv_effect)
S3method(confint,treatment_effect)
S3method(find_data,glm)
S3method(find_data,lm)
S3method(predict_counterfactual,glm)
S3method(predict_counterfactual,lm)
S3method(print,prediction_cf)
S3method(print,robin_output)
S3method(print,surv_effect)
S3method(print,treatment_effect)
S3method(table,default)
S3method(table,surv_effect)
S3method(treatment_effect,glm)
S3method(treatment_effect,lm)
S3method(treatment_effect,prediction_cf)
export(against_ref)
export(custom_contrast)
export(eff_jacob)
export(find_data)
export(h_diff)
export(h_jac_diff)
export(h_jac_log_odds_ratio)
export(h_jac_log_risk_ratio)
export(h_jac_odds_ratio)
export(h_jac_risk_ratio)
export(h_log_odds_ratio)
export(h_log_risk_ratio)
export(h_odds_ratio)
export(h_risk_ratio)
export(pairwise)
export(predict_counterfactual)
export(robin_glm)
export(robin_lm)
export(robin_surv)
export(surv_control)
export(table)
export(treatment_effect)
export(vcovG)
export(vcovHC)
import(checkmate)
importFrom(MASS,negative.binomial)
importFrom(numDeriv,grad)
importFrom(sandwich,vcovHC)
importFrom(stats,as.formula)
importFrom(stats,coefficients)
importFrom(stats,confint)
importFrom(stats,family)
importFrom(stats,fitted)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,residuals)
importFrom(stats,terms)
importFrom(stats,var)
importFrom(utils,combn)
importFrom(utils,tail)

#' Counterfactual Prediction
#' @description Obtain counterfactual prediction of a fit.
#'
#' @param fit fitted object.
#' @param treatment (`formula`) formula of form treatment ~ strata(s).
#' @param data (`data.frame`) raw dataset.
#' @param vcov (`function` or `character`) variance function or name.
#' @param vcov_args (`list`) additional arguments for variance function.
#' @param ... Additional arguments for methods.
#'
#' @return List of class `prediction_cf` containing following elements:
#' - `estimate`: predicted marginal mean.
#' - `residual`: residual of the bias-corrected prediction.
#' - `predictions`: all predicted values.
#' - `predictions_liner`: linear predictions.
#' - `schema`: randomization schema.
#' - `response`: response value.
#' - `fit`: fitted model.
#' - `model_matrix`: model matrix.
#' - `treatment_formula`: treatment assignment and randomization formula.
#' - `treatment`: treatment value.
#' - `group_idx`: group index based on the stratification.
#' - `variance`: estimated variance of the marginal mean.
#' - `variance_name`: name of the variance.
#'
#' @export
predict_counterfactual <- function(fit, treatment, data, vcov, vcov_args, ...) {
  UseMethod("predict_counterfactual")
}

#' @export
predict_counterfactual.lm <- function(fit, treatment, data = find_data(fit), vcov = "vcovG", vcov_args = list(), ...) {
  trt_vars <- h_get_vars(treatment)
  assert(
    test_string(vcov),
    test_function(vcov),
    test_null(vcov)
  )
  assert_data_frame(data)
  assert_subset(c(trt_vars$treatment, trt_vars$strata), colnames(data))
  formula <- formula(fit)
  assert_subset(trt_vars$treatment, all.vars(formula[[3]]))
  assert(
    test_character(data[[trt_vars$treatment]]),
    test_factor(data[[trt_vars$treatment]])
  )
  data[[trt_vars$treatment]] <- as.factor(data[[trt_vars$treatment]])

  trt_lvls <- levels(data[[trt_vars$treatment]])
  n_lvls <- length(trt_lvls)

  df <- lapply(
    data,
    function(i) {
      rep(i, times = n_lvls)
    }
  )

  df[[trt_vars$treatment]] <- gl(n = n_lvls, k = nrow(data), labels = trt_lvls)

  mm <- model.matrix(fit, data = df)
  pred_linear <- mm %*% coefficients(fit)
  preds <- family(fit)$linkinv(pred_linear)

  ret <- matrix(preds, ncol = n_lvls, dimnames = list(row.names(data), trt_lvls))
  y <- model.response(fit$model)
  residual_raw <- y - fitted(fit)

  strata <- data[trt_vars$strata]
  if (ncol(strata) == 0L) {
    strata <- integer(nrow(strata))
  }
  group_idx <- split(seq_len(nrow(data)), strata)

  if (identical(trt_vars$schema, "ps")) {
    ret <- ret + bias(residual_raw, data[[trt_vars$treatment]], group_idx)
  } else {
    ret <- ret + bias(residual_raw, data[[trt_vars$treatment]], list(seq_len(nrow(ret))))
  }

  # Create residual based on the
  # prediction-unbiased response
  trt_idx <- match(data[[trt_vars$treatment]], trt_lvls)
  residual <- y - ret[cbind(seq_len(nrow(ret)), trt_idx)]
  ret <- structure(
    list(
      estimate = colMeans(ret),
      residual = residual,
      predictions = ret,
      schema = trt_vars$schema,
      predictions_linear = pred_linear,
      response = y,
      fit = fit,
      model_matrix = mm,
      treatment_formula = treatment,
      treatment = data[[trt_vars$treatment]],
      group_idx = group_idx
    ),
    class = "prediction_cf"
  )
  if (test_string(vcov)) {
    variance_name <- vcov
    vcov <- match.fun(vcov)
    mm_variance <- do.call(vcov, c(list(ret), vcov_args))
  } else if (test_function(vcov)) {
    variance_name_full <- deparse(substitute(vcov))
    variance_name <- paste(variance_name_full[1], if (length(variance_name_full) > 1) "...")
    mm_variance <- do.call(vcov, c(list(ret), vcov_args))
  } else {
    variance_name <- NULL
    mm_variance <- matrix(NA_real_, nrow = n_lvls, ncol = n_lvls)
  }
  ret$variance <- mm_variance
  ret$variance_name <- variance_name
  ret
}

#' @export
predict_counterfactual.glm <- function(fit, treatment, data = find_data(fit), vcov = "vcovG", vcov_args = list(), ...) {
  # use eval and bquote to allow vcov names to be further passed into inner functions.
  eval(
    bquote(
      predict_counterfactual.lm(
        fit = fit, data = data, treatment = treatment,
        vcov = .(substitute(vcov)), vcov_args = vcov_args
      )
    )
  )
}

test_that("predict_counterfactual works for guassian", {
  expect_snapshot(predict_counterfactual(fit_glm, treatment ~ 1))
})

test_that("predict_counterfactual works for guassian with lm", {
  expect_snapshot(predict_counterfactual(fit_lm, treatment ~ 1, data = glm_data))
})

test_that("predict_counterfactual works for binomial", {
  expect_snapshot(predict_counterfactual(fit_binom, treatment ~ 1))
})

test_that("predict_counterfactual for negative binomial", {
  fit <- glm(y_b ~ treatment * covar, data = glm_data, family = negative.binomial(theta = 1))
  pc <- predict_counterfactual(fit, treatment ~ 1, glm_data)
  predictions <- pc$predictions

  # Check that the mean of the predicted outcomes within
  # each treatment group matches with the observed outcomes
  trt_levels <- levels(glm_data$treatment)
  for (i in seq_along(trt_levels)) {
    idx <- glm_data$treatment == trt_levels[i]
    # mean of the predicted outcomes within treatment group i
    pred_mean <- mean(predictions[idx, i])
    # mean of the y's within treatment group i
    obs_mean <- mean(glm_data[idx, ][["y_b"]])
    expect_equal(pred_mean, obs_mean, tolerance = 1e-15)
  }

  # check that the mean of the residuals is zero within a treatment group
  # this test exists because previously residual and predictions attributes
  # were not aligned properly when predictions were biased
  residuals <- pc$residual
  res_mean <- mean(residuals[idx])
  expect_equal(res_mean, 0, tolerance = 1e-15)
})

test_that("predict_counterfactual works if contrast are non-standard", {
  glm_data2 <- glm_data
  glm_data2$s1 <- as.ordered(glm_data2$s1)
  fit <- glm(y_b ~ treatment * s1, data = glm_data2, family = binomial())
  expect_silent(pc <- predict_counterfactual(fit, treatment ~ 1, data = find_data(fit)))
  expect_snapshot(pc)
  newdf <- rbind(glm_data2, glm_data2, glm_data2)
  newdf$treatment <- rep(factor(levels(glm_data2$treatment), levels(glm_data2$treatment)), each = nrow(glm_data2))
  expect_identical(
    pc$predictions_linear[, 1],
    predict(fit, newdata = newdf, type = "link")
  )
})

test_that("predict_counterfactual works with provided vcov function", {
  vcov_dummy <- function(x) {
    n <- length(x$estimate)
    matrix(0.1, nrow = n, ncol = n)
  }
  expect_snapshot(predict_counterfactual(fit_lm, treatment ~ 1, data = glm_data, vcov = vcov_dummy))
  expect_snapshot(predict_counterfactual(fit_lm, treatment ~ 1, data = glm_data, vcov = NULL))
})

test_that("confint method for prediction_cf works as expected", {
  expect_snapshot(
    confint(predict_counterfactual(fit_binom, treatment = treatment ~ s1, eff_measure = h_diff))
  )
})


test_that("predict_counterfactual works for treatment factor levels in non-alphabetical order", {
  relabel_dat <- glm_data
  levels(relabel_dat$treatment)[1] <- "trtpbo"

  fit_glm_relabel <- glm(y ~ treatment * s1 + covar, data = relabel_dat)
  expected_result <- predict_counterfactual(fit_glm, treatment ~ 1)

  result_relabel <- predict_counterfactual(fit_glm_relabel, treatment ~ 1, data = relabel_dat)

  expected <- expected_result$estimate
  names(expected)[1] <- "trtpbo"
  expect_equal(expected, result_relabel$estimate)
})

test_that("vcovHC works", {
  pc <- predict_counterfactual(fit_binom, treatment ~ s1)
  expect_snapshot(
    vcovHC(pc)
  )
  pc <- predict_counterfactual(fit_binom, treatment ~ pb(s1))
  expect_snapshot(
    vcovHC(pc)
  )
})

test_that("vcovG works", {
  pc <- predict_counterfactual(fit_binom, treatment ~ s1)
  expect_snapshot(
    vcovG(pc)
  )
  pc <- predict_counterfactual(fit_binom, treatment ~ 1)
  expect_snapshot(
    vcovG(pc)
  )
  pc <- predict_counterfactual(fit_binom, treatment ~ pb(s1))
  expect_snapshot(
    vcovG(pc)
  )
  expect_snapshot(
    vcovG(pc, decompose = FALSE)
  )
  pc <- predict_counterfactual(fit_binom, treatment ~ ps(s1))
  expect_snapshot(
    vcovG(pc)
  )
})

#' Covariate adjusted lm model
#' @param formula (`formula`) A formula of analysis.
#' @param data (`data.frame`) Input data frame.
#' @param treatment (`formula` or `character(1)`) A formula of treatment assignment or assignment by stratification,
#' or a string name of treatment assignment.
#' @param vcov (`function`) A function to calculate the variance-covariance matrix of the treatment effect,
#' including `vcovHC` and `vcovG`. The default is 'vcovG'.
#' @param vcov_args (`list`) Additional arguments passed to `vcov`.
#' @param pair Pairwise treatment comparison.
#' @param ... Additional arguments passed to `lm`.
#' @export
#' @return A robin_output object, with `marginal_mean` and `contrast` components.
#' @examples
#' robin_lm(
#'   y ~ treatment * s1,
#'   data = glm_data,
#'   treatment = treatment ~ s1
#' )
robin_lm <- function(
  formula,
  data,
  treatment,
  vcov = "vcovG",
  vcov_args = list(),
  pair,
  ...
) {
  attr(formula, ".Environment") <- environment()
  assert_subset(all.vars(formula), names(data))
  assert_subset(all.vars(treatment), names(data))
  has_interaction <- h_interaction(formula, treatment)
  use_vcovhc <- identical(vcov, "vcovHC") || identical(vcov, vcovHC)
  if (use_vcovhc && has_interaction) {
    stop(
      "Huber-White variance estimator is ONLY supported when using a linear model
      without treatment-covariate interactions; see the 2023 FDA guidance."
    )
  }
  fit <- lm(formula, data = data, ...)
  # same as robin_glm, to allow vcov as function to be used, also allow the function name to be captured.
  pc <- eval(bquote(predict_counterfactual(fit, treatment, data, vcov = .(substitute(vcov)), vcov_args = vcov_args)))
  if (missing(pair)) {
    pair <- pairwise(names(pc$estimate))
  }
  trt_eff <- difference(pc, pair = pair)
  structure(
    list(
      marginal_mean = pc,
      contrast = trt_eff
    ),
    class = "robin_output"
  )
}

#' Evaluate if Interaction Exists
#' @keywords internal
#' @param formula (`formula`) the formula for model fitting.
#' @param treatment (`formula`) the formula for treatment assignment.
h_interaction <- function(formula, treatment) {
  assert_formula(formula)
  treatment <- h_get_vars(treatment)
  assert_subset(treatment$treatment, all.vars(formula[[length(formula)]]))
  tms <- terms(formula)
  fct <- attr(tms, "factors")
  any(fct[treatment$treatment, ] %in% c(1, 2) & colSums(fct != 0) > 1)
}

# h_interaction ----

test_that("h_interaction works correctly", {
  expect_false(h_interaction(y ~ trt + z, treatment = trt ~ x))
  expect_true(h_interaction(y ~ trt:z, treatment = trt ~ x))
  expect_true(h_interaction(trt * y ~ trt:z, treatment = trt ~ x))
  expect_true(h_interaction(y ~ trt:z, treatment = trt ~ 1))
})

# robin_lm ----

test_that("robin_lm works correctly", {
  expect_silent(
    robin_lm(
      y ~ treatment * s1,
      data = glm_data,
      treatment = treatment ~ s1,
    )
  )
  expect_error(
    robin_lm(
      y ~ treatment * s1,
      data = glm_data,
      treatment = treatment ~ s1,
      vcov = "vcovHC"
    ),
    "Huber-White variance estimator is ONLY supported when using a linear model
      without treatment-covariate interactions; see the 2023 FDA guidance."
  )
  expect_snapshot(
    robin_lm(y_b ~ treatment + s1, data = glm_data, treatment = treatment ~ s1)
  )
})


test_that("robin_lm give same result as robin_glm", {
  expect_silent(
    f1 <- robin_lm(
      y ~ treatment * s1,
      data = glm_data,
      treatment = treatment ~ pb(s1),
    )
  )
  expect_silent(
    f2 <- robin_glm(
      y ~ treatment * s1,
      data = glm_data,
      treatment = treatment ~ pb(s1),
    )
  )
  expect_equal(
    f1$marginal_mean$estimate,
    f2$marginal_mean$estimate
  )
  expect_equal(
    f1$contrast$estimate,
    f2$contrast$estimate
  )

  expect_silent(
    f1 <- robin_lm(
      y ~ treatment + s1,
      data = glm_data,
      treatment = treatment ~ pb(s1),
      vcov = vcovHC
    )
  )
  expect_silent(
    f2 <- robin_glm(
      y ~ treatment + s1,
      data = glm_data,
      treatment = treatment ~ pb(s1),
      vcov = vcovHC
    )
  )
  expect_equal(
    f1$marginal_mean$estimate,
    f2$marginal_mean$estimate
  )
  expect_equal(
    f1$contrast$estimate,
    f2$contrast$estimate
  )
})

# glm fit on iris
fit_glm <- glm(y ~ treatment * s1 + covar, data = glm_data)

# lm fit on iris
fit_lm <- lm(y ~ treatment * s1 + covar, data = glm_data)

# glm fit on ToothGrowth
fit_binom <- glm(y_b ~ treatment * s1 + covar, data = glm_data, family = binomial())

# robin glm result with log risk ratio contrast
robin_res1 <- robin_glm(y_b ~ treatment * s1, data = glm_data, treatment = treatment ~ s1, contrast = "log_risk_ratio")
# robin glm result with difference contrast
robin_res2 <- robin_glm(y_b ~ treatment * s1, data = glm_data, treatment = treatment ~ s1, contrast = "diff")

#' Estimate Log Hazard Ratio via Score Function
#'
#' This function estimates the log hazard ratio by finding the root of the log-rank score function.
#'
#' @details This deactivates the ties factor correction in the score function by passing
#' `use_ties_factor = FALSE` to the `score_fun`. The root finding is done without calculating
#' the variance by passing `calculate_variance = FALSE`, which is only calculated at the solution.
#' This saves computation time, and avoids spurious warnings about negative variances during the root
#' finding process.
#'
#' @param score_fun (`function`) The log-rank score function to be used for estimation.
#' @param interval (`numeric`) A numeric vector of length 2 specifying the interval in which to search for the root.
#' @param control (`list`) Control parameters from [surv_control()] for the root-finding algorithm.
#' @param ... Additional arguments passed to `score_fun`.
#' @return A list containing:
#' - `theta`: The estimated log hazard ratio.
#' - `se`: The standard error of the estimated log hazard ratio.
#' - `sigma_L2`: The variance of the log-rank statistic.
#' - `n`: The number of observations used in the calculation.
#'
#' @keywords internal
h_log_hr_est_via_score <- function(score_fun, interval = c(-5, 5), control = surv_control(), ...) {
  assert_function(score_fun, args = c("theta", "use_ties_factor", "calculate_variance"))
  assert_numeric(interval, len = 2L, finite = TRUE)
  assert_true(interval[1] < interval[2])
  h_assert_surv_control(control)

  score_solution <- stats::uniroot(
    score_fun,
    interval = interval,
    extendInt = "yes", # If the root is not found in the interval, extend the interval.
    check.conv = TRUE, # If the root cannot be found, an error is thrown.
    use_ties_factor = FALSE,
    calculate_variance = FALSE, # We will only do this at the solution.
    tol = control$tol,
    maxiter = control$maxiter,
    trace = control$trace,
    ...
  )
  score_root <- score_solution$root
  # Now calculate the variance at the solution.
  score_result <- score_fun(
    theta = score_root,
    use_ties_factor = FALSE,
    calculate_variance = TRUE,
    ...
  )
  solution_attrs <- attributes(score_result)

  list(
    theta = score_root,
    se = solution_attrs$se_theta_l,
    sigma_l2 = solution_attrs$sigma_l2,
    n = solution_attrs$n
  )
}

#' Log-Rank Test via Score Function
#'
#' This function performs a log-rank test using the score function.
#'
#' @details This activates the ties factor correction in the score function by passing
#'   `use_ties_factor = TRUE` to the `score_fun`.
#'
#' @param score_fun (`function`) The log-rank score function to be used for testing.
#' @param ... Additional arguments passed to `score_fun`.
#' @return A list containing:
#' - `u_l`: The log-rank score statistic.
#' - `sigma_l2`: The variance of the log-rank statistic.
#' - `tau_l`: The log-rank test statistic.
#' - `pval`: The two-sided p-value of the log-rank test.
#' - `n`: The number of observations used in the calculation.
#'
#' @keywords internal
h_lr_test_via_score <- function(score_fun, ...) {
  assert_function(score_fun, args = c("theta", "use_ties_factor", "calculate_variance"))

  score_res <- score_fun(theta = 0, use_ties_factor = TRUE, calculate_variance = TRUE, ...)
  u_l <- as.numeric(score_res)
  score_attrs <- attributes(score_res)

  sigma_l2 <- score_attrs$sigma_l2
  n <- score_attrs$n
  tau_l <- sqrt(n) * u_l / sqrt(sigma_l2)
  pval <- 2 * stats::pnorm(-abs(tau_l))

  list(
    u_l = u_l,
    sigma_l2 = sigma_l2,
    tau_l = tau_l,
    pval = pval,
    n = n,
    give_rand_strat_warning = score_attrs$give_rand_strat_warning
  )
}

#' Log Hazard Ratio Estimation and Log-Rank Test via Score Function
#'
#' This function combines the estimation of the log hazard ratio and the log-rank test
#' using a score function. Only two treatment arms are being compared and the `data` is subset accordingly.
#'
#' @details If an unadjusted score function is provided in `unadj_score_fun`, then it is used to estimate the
#' log hazard ratio first. This unadjusted log hazard ratio estimate is then passed on to the adjusted
#' score function `score_fun` as `theta_hat`. This is required when the score function is adjusted for covariates.
#'
#' @param score_fun (`function`) The log-rank score function to be used for both estimation and testing.
#' @param vars (`list`) A list containing `levels`, `treatment`, and `covariates`.
#' @param data (`data.frame`) The data frame containing the survival data.
#' @param exp_level (`count`) Level of the experimental treatment arm.
#' @param control_level (`count`) Level of the control treatment arm.
#' @param unadj_score_fun (`function` or `NULL`) Optional unadjusted score function, see details.
#' @param ... Additional arguments passed to `score_fun`.
#' @return A list containing:
#' - `estimate`: The estimated log hazard ratio.
#' - `se`: The standard error of the estimated log hazard ratio.
#' - `hr_n`: The number of observations used in the estimation.
#' - `hr_sigma_l2`: The variance of the log-rank statistic used in the estimation.
#' - `test_stat`: The log-rank test statistic.
#' - `p_value`: The two-sided p-value of the log-rank test.
#' - `test_score`: The log-rank score statistic.
#' - `test_n`: The number of observations used in the log-rank test.
#' - `test_sigma_l2`: The variance of the log-rank statistic used in the log-rank test.
#'
#' @keywords internal
robin_surv_comparison <- function(
  score_fun,
  vars,
  data,
  exp_level,
  control_level,
  contrast,
  control = surv_control(),
  unadj_score_fun = NULL,
  ...
) {
  assert_list(vars)
  assert_names(names(vars), must.include = c("levels", "treatment", "covariates"))
  assert_character(vars$levels, min.len = 2L)
  assert_data_frame(data)
  assert_string(vars$treatment)
  assert_true(is.factor(data[[vars$treatment]]))
  assert_count(exp_level)
  assert_count(control_level)
  assert_true(exp_level != control_level)
  assert_string(contrast)
  h_assert_surv_control(control)

  # Subset data to the two treatment arms of interest.
  trt_levels <- vars$levels[c(control_level, exp_level)]
  in_trt_scope <- data[[vars$treatment]] %in% trt_levels
  data <- data[in_trt_scope, , drop = FALSE]
  data[[vars$treatment]] <- droplevels(data[[vars$treatment]])
  data[[vars$treatment]] <- stats::relevel(data[[vars$treatment]], ref = trt_levels[1L])

  # Prepare arguments for the test and estimation calls below.
  score_args <- list(
    score_fun = score_fun,
    df = data,
    ...
  )

  # Perform the log-rank test via the score function.
  test_result <- do.call(h_lr_test_via_score, score_args)

  # Estimate the log hazard ratio via the score function, if requested.
  hr_result <- if (contrast == "hazardratio") {
    hr_args <- c(score_args, list(control = control))
    # If an unadjusted score function is provided, use it to estimate the log hazard ratio first.
    if (!is.null(unadj_score_fun)) {
      assert_function(unadj_score_fun)
      assert_true(length(vars$covariates) > 0)
      args_to_drop <- c("model", "hr_se_plugin_adjusted", "check_rand_strat_warning")
      unadj_args <- hr_args[!(names(hr_args) %in% args_to_drop)]
      unadj_args$score_fun <- unadj_score_fun
      # Get theta_hat from the unadjusted score function.
      unadj_hr_result <- do.call(h_log_hr_est_via_score, unadj_args)
      # Add this to the arguments for the adjusted score function call below.
      hr_args$theta_hat <- unadj_hr_result$theta
    } else {
      # We enforce to have no covariates in this case.
      assert_true(length(vars$covariates) == 0L)
    }
    # Estimate the log hazard ratio via the score function.
    do.call(h_log_hr_est_via_score, hr_args)
  } else {
    list(
      theta = NA_real_,
      se = NA_real_,
      n = NA_integer_,
      sigma_l2 = NA_real_
    )
  }

  list(
    estimate = hr_result$theta,
    se = hr_result$se,
    hr_n = hr_result$n,
    hr_sigma_l2 = hr_result$sigma_l2,
    test_stat = test_result$tau_l,
    p_value = test_result$pval,
    test_score = test_result$u_l,
    test_n = test_result$n,
    test_sigma_l2 = test_result$sigma_l2,
    give_rand_strat_warning = test_result$give_rand_strat_warning
  )
}

#' Survival Comparison Functions
#'
#' These are simple wrappers around [robin_surv_comparison()] called with the corresponding log-rank score
#' functions.
#'
#' @inheritParams robin_surv_comparison
#' @return See [robin_surv_comparison()].
#'
#' @name survival_comparison_functions
#' @keywords internal
NULL

#' @describeIn survival_comparison_functions without strata and without covariates, based on
#'   [h_lr_score_no_strata_no_cov()].
robin_surv_no_strata_no_cov <- function(
  vars,
  data,
  exp_level,
  control_level,
  contrast,
  control = surv_control(),
  check_rand_strat_warning = FALSE
) {
  robin_surv_comparison(
    score_fun = h_lr_score_no_strata_no_cov,
    vars = vars,
    data = data,
    exp_level = exp_level,
    control_level = control_level,
    contrast = contrast,
    control = control,
    treatment = vars$treatment,
    time = vars$time,
    status = vars$status,
    randomization_strata = vars$randomization_strata,
    check_rand_strat_warning = check_rand_strat_warning
  )
}

#' @describeIn survival_comparison_functions without strata and without covariates, based on
#'   [h_lr_score_strat()].
robin_surv_strata <- function(
  vars,
  data,
  exp_level,
  control_level,
  contrast,
  control = surv_control(),
  check_rand_strat_warning = FALSE
) {
  robin_surv_comparison(
    score_fun = h_lr_score_strat,
    vars = vars,
    data = data,
    exp_level = exp_level,
    control_level = control_level,
    contrast = contrast,
    control = control,
    treatment = vars$treatment,
    time = vars$time,
    status = vars$status,
    strata = vars$strata,
    randomization_strata = vars$randomization_strata,
    check_rand_strat_warning = check_rand_strat_warning
  )
}

#' @describeIn survival_comparison_functions without strata and without covariates, based on
#'   [h_lr_score_cov()] and [h_lr_score_no_strata_no_cov()] (which is used to find the unadjusted
#'   log hazard ratio estimate).
robin_surv_cov <- function(vars, data, exp_level, control_level, contrast, control = surv_control(), ...) {
  robin_surv_comparison(
    score_fun = h_lr_score_cov,
    unadj_score_fun = h_lr_score_no_strata_no_cov,
    vars = vars,
    data = data,
    exp_level = exp_level,
    control_level = control_level,
    contrast = contrast,
    control = control,
    treatment = vars$treatment,
    time = vars$time,
    status = vars$status,
    model = vars$model,
    randomization_strata = vars$randomization_strata,
    ...
  )
}

#' @describeIn survival_comparison_functions with strata and covariates, based on
#'   [h_lr_score_strat_cov()] and [h_lr_score_strat()] (which is used to find the unadjusted
#'   log hazard ratio estimate).
robin_surv_strata_cov <- function(vars, data, exp_level, control_level, contrast, control = surv_control(), ...) {
  robin_surv_comparison(
    score_fun = h_lr_score_strat_cov,
    unadj_score_fun = h_lr_score_strat,
    vars = vars,
    data = data,
    exp_level = exp_level,
    control_level = control_level,
    contrast = contrast,
    control = control,
    treatment = vars$treatment,
    time = vars$time,
    status = vars$status,
    strata = vars$strata,
    model = vars$model,
    randomization_strata = vars$randomization_strata,
    ...
  )
}

#' Log Hazard Ratio Coefficient Matrix
#'
#' This function creates a coefficient matrix for the log hazard ratio estimates.
#'
#' @param x (`list`) A list containing the log hazard ratio estimates and their standard errors.
#' @return A matrix with columns for the log hazard ratio estimate, standard error, z-value,
#'   and p-value.
#'
#' @keywords internal
h_log_hr_coef_mat <- function(x) {
  assert_list(x, names = "unique")
  assert_names(names(x), must.include = c("estimate", "se", "pair"))
  assert_numeric(x$estimate, finite = TRUE)
  assert_numeric(x$se, finite = TRUE, len = length(x$estimate), lower = .Machine$double.eps)
  assert_list(x$pair, types = "integer", len = 2L)
  assert_character(attr(x$pair, "levels"), min.len = max(unlist(x$pair)))
  assert_true(length(x$pair[[1]]) == length(x$pair[[2]]))
  assert_true(length(x$pair[[1]]) == length(x$se))

  z_value <- x$estimate / x$se
  p_value <- 2 * pnorm(-abs(z_value))
  ret <- matrix(
    c(
      x$estimate,
      x$se,
      z_value,
      p_value
    ),
    nrow = length(x$estimate)
  )
  colnames(ret) <- c("Estimate", "Std.Err", "Z Value", "Pr(>|z|)")
  pair <- x$pair
  row.names(ret) <- sprintf("%s v.s. %s", attr(pair, "levels")[pair[[1]]], attr(pair, "levels")[pair[[2]]])
  ret
}

#' Log-Rank Test Results Matrix
#'
#' This function creates a matrix summarizing the results of the log-rank test.
#'
#' @param x (`list`) A list containing the log-rank test results.
#' @return A matrix with columns for the test statistic and p-value.
#'
#' @keywords internal
h_test_mat <- function(x) {
  assert_list(x, names = "unique")
  assert_names(names(x), must.include = c("test_stat", "p_value", "pair"))
  assert_numeric(x$test_stat, finite = TRUE)
  assert_numeric(x$p_value, finite = TRUE, len = length(x$test_stat))
  assert_list(x$pair, types = "integer", len = 2L)
  assert_character(attr(x$pair, "levels"), min.len = max(unlist(x$pair)))
  assert_true(length(x$pair[[1]]) == length(x$pair[[2]]))
  assert_true(length(x$pair[[1]]) == length(x$test_stat))

  ret <- matrix(
    c(
      x$test_stat,
      x$p_value
    ),
    nrow = length(x$test_stat)
  )
  colnames(ret) <- c("Test Stat.", "Pr(>|z|)")
  pair <- x$pair
  row.names(ret) <- sprintf("%s v.s. %s", attr(pair, "levels")[pair[[1]]], attr(pair, "levels")[pair[[2]]])
  ret
}

#' Prepare Events Table
#'
#' This function creates a data frame summarizing the number of patients and events
#' for each treatment arm and stratification factor.
#'
#' @param data (`data.frame`) The data frame containing the survival data.
#' @param vars (`list`) A list containing the treatment, time, status, and strata variables.
#' @return A data frame with columns for the treatment, strata, number of patients, and number of events.
#'
#' @keywords internal
h_events_table <- function(data, vars) {
  assert_data_frame(data, col.names = "unique")
  assert_subset(with(vars, c(treatment, time, status, strata)), names(data))

  agg_res <- stats::aggregate(
    by = data[c(vars$treatment, vars$strata)], # This order leads to the expected sorting.
    x = data[vars$status],
    FUN = function(x) (c(Patients = length(x), Events = as.integer(sum(x)))),
    drop = TRUE
  )
  cbind(
    agg_res[c(vars$strata, vars$treatment)],
    agg_res[[vars$status]]
  )
}

#' Control Survival Analysis Root Finding
#'
#' Control parameters for the root-finding algorithm used by [robin_surv()] to estimate
#' the log hazard ratio.
#'
#' @param tol (`number`) The desired accuracy, passed to [stats::uniroot()].
#' @param maxiter (`count`) The maximum number of iterations, passed to [stats::uniroot()].
#' @param trace (`count`) Tracing level, passed to [stats::uniroot()].
#'
#' @return A named list with elements `tol`, `maxiter`, and `trace`.
#'
#' @export
surv_control <- function(tol = .Machine$double.eps^0.25, maxiter = 1000, trace = 0) {
  control <- list(
    tol = tol,
    maxiter = maxiter,
    trace = trace
  )
  h_assert_surv_control(control)

  list(
    tol = tol,
    maxiter = as.integer(maxiter),
    trace = as.integer(trace)
  )
}

h_assert_surv_control <- function(control) {
  assert_list(control, len = 3L, names = "unique")
  assert_names(names(control), must.include = c("tol", "maxiter", "trace"))
  assert_number(control$tol, lower = 0, finite = TRUE)
  assert_integerish(control$maxiter, len = 1L, lower = 1L, any.missing = FALSE)
  assert_integerish(control$trace, len = 1L, lower = 0L, any.missing = FALSE)

  invisible(control)
}

#' Covariate Adjusted and Stratified Survival Analysis
#'
#' Calculate log-rank test as well as hazard ratio estimates for survival data, optionally adjusted
#' for covariates and a stratification factor.
#'
#' @param formula (`formula`) A formula of analysis, of the form
#'   `Surv(time, status) ~ covariates + strata(x, y, z)`.
#'   If no covariates should be adjusted for, use `1` instead on the right hand side. The intercept must not be removed.
#'   If no stratification factors should be used for the analysis, do not use `strata()` in the formula.
#' @param data (`data.frame`) Input data frame.
#' @param treatment (`formula`) A formula of treatment assignment or assignment by stratification, of the form
#'   `treatment ~ scheme(vars)`. Note that currently the randomization scheme is not used in the analysis. However,
#'   any variables that were used in the randomization scheme must be included in the model formula,
#'   either as covariates, or as `strata()`.
#' @param comparisons (`list`) An optional list of comparisons between treatment levels to be performed,
#'   see details. By default, all pairwise comparisons are performed automatically.
#' @param contrast (`character(1)`) The contrast statistic to be used, currently only `"hazardratio"`
#'   is supported. Can be disabled by specifying `"none"`, in which case only the log-rank test is performed.
#' @param test (`character(1)`) The test to be used, currently only `"logrank"` is supported.
#' @param control (`list`) Control parameters for the root-finding algorithm used to estimate the
#'   log hazard ratio, usually created with [surv_control()]. The arguments correspond to
#'   `tol`, `maxiter`, and `trace` in [stats::uniroot()].
#' @param ... Additional arguments passed to the survival analysis functions, in particular `hr_se_plugin_adjusted`
#'   (please see [here][survival_score_functions] for details).
#' @return A `surv_effect` object containing the results of the survival analysis.
#' @seealso [surv_effect_methods] for S3 methods.
#'
#' @details
#' The user can optionally specify a list of comparisons between treatment levels to be performed.
#' The list must have two elements:
#'
#' - Treatment level indices of the treatment group.
#' - Treatment level indices of the control group.
#'
#' So for example if you would like to compare level 3 with level 1, and also level 3 with level 2
#' (but not level 2 with level 1) then you can specify:
#' `comparisons = list(c(3, 3), c(1, 2))`
#'
#' @export
#'
#' @examples
#' # Adjusted for covariates meal.cal and age and adjusted for stratification by strata:
#' robin_surv(
#'   formula = Surv(time, status) ~ meal.cal + age + strata(strata),
#'   data = surv_data,
#'   treatment = sex ~ pb(strata)
#' )
#'
#' # Adjusted for stratification by strata and ecog but not for covariates:
#' robin_surv(
#'   formula = Surv(time, status) ~ 1 + strata(strata, ecog),
#'   data = surv_data,
#'   treatment = sex ~ sr(1)
#' )
#'
#' # Unadjusted for covariates and stratification:
#' robin_surv(
#'   formula = Surv(time, status) ~ 1,
#'   data = surv_data,
#'   treatment = sex ~ sr(1)
#' )
robin_surv <- function(
  formula,
  data,
  treatment,
  comparisons,
  contrast = c("hazardratio", "none"),
  test = "logrank",
  control = surv_control(),
  ...
) {
  attr(formula, ".Environment") <- environment()
  assert_formula(formula)
  assert_data_frame(data)
  assert_formula(treatment)
  assert_subset(all.vars(formula), names(data))
  assert_subset(all.vars(treatment), names(data))
  contrast <- match.arg(contrast)
  test <- match.arg(test)
  h_assert_surv_control(control)

  input <- h_prep_survival_input(formula, data, treatment)

  # Subset to complete records here, so that we can use this for the strata/events tabulation.
  data_columns_needed <- unique(c(
    input$treatment,
    input$time,
    input$status,
    input$strata,
    input$covariates,
    input$randomization_strata
  ))
  data <- stats::na.omit(input$data[data_columns_needed])
  events_table <- h_events_table(data, input)

  has_strata <- length(input$strata) > 0
  has_covariates <- length(input$covariates) > 0
  calc_function <- if (has_strata && has_covariates) {
    robin_surv_strata_cov
  } else if (has_strata) {
    robin_surv_strata
  } else if (has_covariates) {
    robin_surv_cov
  } else {
    robin_surv_no_strata_no_cov
  }

  if (missing(comparisons)) {
    comparisons <- pairwise(input$levels)
  } else {
    # Convert to integer and assign levels attribute to user defined list for user convenience.
    assert_list(comparisons, types = "integerish")
    comparisons <- lapply(comparisons, as.integer)
    attr(comparisons, "levels") <- input$levels
  }
  assert_list(comparisons, len = 2L, types = "integer")
  n_comparisons <- length(comparisons[[1]])
  assert_integer(comparisons[[1]], lower = 1L, upper = length(input$levels))
  assert_integer(comparisons[[2]], lower = 1L, upper = length(input$levels))

  # Variable to keep track whether a warning about insufficient inclusion of
  # randomization strata in the analysis model has already been required in a comparison.
  # We want to avoid checking for the need of a warning, or giving a warning,
  # multiple times.
  give_rand_strat_warning <- FALSE

  estimates <- lapply(
    seq_len(n_comparisons),
    function(i) {
      exp_level <- comparisons[[1]][i]
      control_level <- comparisons[[2]][i]
      result <- calc_function(
        vars = input,
        data = data,
        exp_level = exp_level,
        control_level = control_level,
        contrast = contrast,
        control = control,
        check_rand_strat_warning = !give_rand_strat_warning,
        ...
      )
      if (!give_rand_strat_warning) {
        # Only update if we have checked for it in this iteration, otherwise
        # we could overwrite a TRUE value with the default FALSE.
        give_rand_strat_warning <<- result$give_rand_strat_warning
      }
      result
    }
  )

  if (give_rand_strat_warning) {
    missing_vars <- setdiff(input$randomization_strata, c(input$covariates, input$strata))
    cov_string <- if (length(missing_vars) > 1) {
      paste0("interaction(", toString(missing_vars), ")")
    } else {
      missing_vars
    }
    strata_string <- paste0("strata(", toString(missing_vars), ")")
    warning(
      paste0(
        "It looks like you have not included all of the variables that were used ",
        "during randomization in your analysis `formula`. You can either:\n\n",
        "a. adjust for all joint levels in your `formula` using `+ ",
        cov_string,
        "` or\n",
        "b. perform a stratified test by adding to your `formula` the term `+ ",
        strata_string,
        "`\n\n",
        "NOTE: (b) changes the null hypothesis from your current model specification. ",
        "Please see the vignette `robincar-survival` for details."
      ),
      call. = FALSE
    )
  }

  result <- list(
    model = formula,
    vars = input,
    data = data,
    events_table = events_table,
    randomization = treatment,
    schema = input$schema,
    contrast = contrast,
    test = test,
    pair = comparisons,
    estimate = sapply(estimates, "[[", "estimate"),
    se = sapply(estimates, "[[", "se"),
    hr_n = sapply(estimates, "[[", "hr_n"),
    hr_sigma_l2 = sapply(estimates, "[[", "hr_sigma_l2"),
    test_stat = sapply(estimates, "[[", "test_stat"),
    p_value = sapply(estimates, "[[", "p_value"),
    test_score = sapply(estimates, "[[", "test_score"),
    test_n = sapply(estimates, "[[", "test_n"),
    test_sigma_l2 = sapply(estimates, "[[", "test_sigma_l2")
  )

  if (contrast == "hazardratio") {
    result$log_hr_coef_mat <- h_log_hr_coef_mat(result)
  }
  result$test_mat <- h_test_mat(result)

  class(result) <- "surv_effect"
  result
}

test_that("print method for surv_effect works as expected (unstratified, unadjusted)", {
  x <- robin_surv(
    formula = Surv(time, status) ~ 1,
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  expect_snapshot(print(x))
})

test_that("print method for surv_effect works as expected (stratified, unadjusted)", {
  x <- robin_surv(
    formula = Surv(time, status) ~ 1 + strata(strata),
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  expect_snapshot(print(x))
})

test_that("print method for surv_effect works as expected (unstratified, adjusted)", {
  x <- robin_surv(
    formula = Surv(time, status) ~ meal.cal + age,
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  expect_snapshot(print(x))
})

test_that("print method for surv_effect works as expected (stratified, adjusted)", {
  x <- robin_surv(
    formula = Surv(time, status) ~ meal.cal + age + strata(strata),
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  expect_snapshot(print(x))
})

test_that("print method for surv_effect works as expected when no hazard ratio was estimated", {
  x <- robin_surv(
    formula = Surv(time, status) ~ meal.cal + age + strata(strata),
    data = surv_data,
    treatment = sex ~ sr(1),
    contrast = "none"
  )
  expect_snapshot(print(x))
})

test_that("table method for surv_effect works as expected", {
  x <- robin_surv(
    formula = Surv(time, status) ~ meal.cal + age + strata(strata),
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  expect_snapshot(result <- table(x))
  expect_identical(result, x$events_table)
})

test_that("confint method for surv_effect works as expected", {
  x <- robin_surv(
    formula = Surv(time, status) ~ meal.cal + age + strata(strata),
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  expect_snapshot(result <- confint(x))
  expect_snapshot(
    expect_message(
      result <- confint(x, transform = exp),
      "The confidence interval is transformed.",
      fixed = TRUE
    )
  )
})

test_that("confint method returns error when no hazard ratio was estimated", {
  x <- robin_surv(
    formula = Surv(time, status) ~ meal.cal + age,
    data = surv_data,
    treatment = sex ~ sr(1),
    contrast = "none"
  )
  expect_error(
    confint(x),
    "No contrast was estimated; confidence interval is not available.",
    fixed = TRUE
  )
})

test_that("h_log_hr_est_via_score works as expected", {
  result <- h_log_hr_est_via_score(
    h_lr_score_no_strata_no_cov,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status"
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_log_hr_est_via_score does not give spurious warning", {
  result <- expect_silent(h_log_hr_est_via_score(
    h_lr_score_strat_cov,
    df = na.omit(surv_data),
    treatment = "ecog",
    time = "time",
    status = "status",
    strata = "sex",
    model = ~ age + meal.cal + wt.loss,
    randomization_strata = character()
  ))
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_log_hr_est_via_score extends the search interval as needed", {
  result <- h_log_hr_est_via_score(
    h_lr_score_no_strata_no_cov,
    interval = c(-0.2, 0.2),
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    randomization_strata = character()
  )
  expect_true(result$theta > 0.2)
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("surv_control works as expected", {
  expect_identical(
    surv_control(tol = 1e-6, maxiter = 10, trace = 1),
    list(tol = 1e-6, maxiter = 10L, trace = 1L)
  )
  expect_error(surv_control(tol = -1), "Assertion")
  expect_error(surv_control(maxiter = 0), "Assertion")
  expect_error(surv_control(trace = -1), "Assertion")
})

test_that("h_log_hr_est_via_score passes control to uniroot", {
  expect_error(
    h_log_hr_est_via_score(
      h_lr_score_no_strata_no_cov,
      interval = c(-0.2, 0.2),
      control = surv_control(maxiter = 1),
      df = surv_data,
      treatment = "sex",
      time = "time",
      status = "status",
      randomization_strata = character()
    ),
    "no sign change found in 1 iteration",
    fixed = TRUE
  )
})

test_that("h_lr_test_via_score works as expected", {
  result <- h_lr_test_via_score(
    h_lr_score_no_strata_no_cov,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status"
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("robin_surv_comparison works as expected without covariate adjustment", {
  input <- h_prep_survival_input(
    formula = survival::Surv(time, status) ~ 1,
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  result <- robin_surv_comparison(
    score_fun = h_lr_score_no_strata_no_cov,
    vars = input,
    data = input$data,
    exp_level = 2,
    control_level = 1,
    contrast = "hazardratio",
    treatment = input$treatment,
    time = input$time,
    status = input$status
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("robin_surv_comparison can skip the hazard ratio estimation", {
  input <- h_prep_survival_input(
    formula = survival::Surv(time, status) ~ 1,
    data = surv_data,
    treatment = sex ~ 1
  )
  result <- robin_surv_comparison(
    score_fun = h_lr_score_no_strata_no_cov,
    vars = input,
    data = input$data,
    exp_level = 2,
    control_level = 1,
    contrast = "none",
    treatment = input$treatment,
    time = input$time,
    status = input$status
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("robin_surv_no_strata_no_cov works as expected", {
  input <- h_prep_survival_input(
    formula = survival::Surv(time, status) ~ 1,
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  result <- robin_surv_no_strata_no_cov(
    vars = input,
    data = input$data,
    exp_level = 1,
    control_level = 2,
    contrast = "hazardratio"
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("robin_surv_no_strata_no_cov gives the same results as RobinCar functions", {
  input <- h_prep_survival_input(
    formula = survival::Surv(time, status) ~ 1,
    data = surv_data,
    treatment = ecog ~ sr(1)
  )
  input$data <- na.omit(input$data)
  result <- robin_surv_no_strata_no_cov(
    vars = input,
    data = input$data,
    exp_level = 2,
    control_level = 1,
    contrast = "hazardratio"
  )
  # These values are extracted from RobinCar (version 1.0.0) results, see
  # `tests-raw/test-survival.R`.
  robincar_result <- list(
    test_stat = -0.6188324,
    test_sigma_l2 = 0.1782103,
    estimate = -0.1131005,
    se = 0.1830198
  )
  expect_equal(result$test_stat, robincar_result$test_stat, tolerance = 1e-5)
  expect_equal(result$test_sigma_l2, robincar_result$test_sigma_l2, tolerance = 1e-4)
  expect_equal(result$estimate, robincar_result$estimate, tolerance = 1e-4)
  expect_equal(result$se, robincar_result$se, tolerance = 1e-4)
})

test_that("robin_surv_strata works as expected", {
  input <- h_prep_survival_input(
    formula = survival::Surv(time, status) ~ 1 + strata(strata),
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  result <- robin_surv_strata(
    vars = input,
    data = input$data,
    exp_level = 1,
    control_level = 2,
    contrast = "hazardratio"
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("robin_surv_strata works with multiple strata variables", {
  input <- h_prep_survival_input(
    formula = survival::Surv(time, status) ~ strata(strata, ecog),
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  result <- robin_surv_strata(
    vars = input,
    data = input$data,
    exp_level = 1,
    control_level = 2,
    contrast = "hazardratio"
  )
  surv_data$strata_ecog <- interaction(surv_data$strata, surv_data$ecog, drop = TRUE)
  input2 <- h_prep_survival_input(
    formula = survival::Surv(time, status) ~ strata(strata_ecog),
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  result2 <- robin_surv_strata(
    vars = input2,
    data = input2$data,
    exp_level = 1,
    control_level = 2,
    contrast = "hazardratio"
  )
  expect_equal(result, result2, ignore_formula_env = TRUE)
})

test_that("robin_surv_strata gives the same results as RobinCar functions", {
  input <- h_prep_survival_input(
    formula = survival::Surv(time, status) ~ strata(sex),
    data = surv_data,
    treatment = ecog ~ sr(1)
  )
  input$data <- na.omit(input$data)
  result <- robin_surv_strata(
    vars = input,
    data = input$data,
    exp_level = 2,
    control_level = 1,
    contrast = "hazardratio"
  )
  # These values are extracted from RobinCar (version 1.0.0) results, see
  # `tests-raw/test-survival.R`.
  robincar_result <- list(
    test_stat = -0.6171326,
    test_sigma_l2 = 0.1749327,
    estimate = -0.1138251,
    se = 0.1847554
  )
  expect_equal(result$test_stat, robincar_result$test_stat, tolerance = 1e-5)
  expect_equal(result$test_sigma_l2, robincar_result$test_sigma_l2, tolerance = 1e-4)
  expect_equal(result$estimate, robincar_result$estimate, tolerance = 1e-3)
  expect_equal(result$se, robincar_result$se, tolerance = 1e-3)
})

test_that("robin_surv_cov works as expected", {
  input <- h_prep_survival_input(
    formula = survival::Surv(time, status) ~ age,
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  result <- robin_surv_cov(
    vars = input,
    data = input$data,
    exp_level = 1,
    control_level = 2,
    contrast = "hazardratio"
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("robin_surv_cov gives the same results as RobinCar functions", {
  input <- h_prep_survival_input(
    formula = survival::Surv(time, status) ~ age,
    data = surv_data,
    treatment = ecog ~ sr(1)
  )
  input$data <- na.omit(input$data)
  result <- robin_surv_cov(
    vars = input,
    data = input$data,
    exp_level = 2,
    control_level = 1,
    contrast = "hazardratio",
    hr_se_plugin_adjusted = FALSE # To get the exact match with RobinCar.
  )
  # These values are extracted from RobinCar (version 1.0.0) results, see
  # `tests-raw/test-survival.R`.
  robincar_result <- list(
    test_stat = -0.4309439,
    test_sigma_l2 = 0.175718,
    estimate = -0.07914235,
    se = 0.181807
  )
  expect_equal(result$test_stat, robincar_result$test_stat, tolerance = 1e-5)
  expect_equal(result$test_sigma_l2, robincar_result$test_sigma_l2, tolerance = 1e-4)
  expect_equal(result$estimate, robincar_result$estimate, tolerance = 1e-3)
  expect_equal(result$se, robincar_result$se, tolerance = 1e-3)
})

test_that("robin_surv_strata_cov works as expected", {
  input <- h_prep_survival_input(
    formula = survival::Surv(time, status) ~ age + strata(sex),
    data = surv_data,
    treatment = ecog ~ sr(1)
  )
  result <- robin_surv_strata_cov(
    vars = input,
    data = input$data,
    exp_level = 2,
    control_level = 1,
    contrast = "hazardratio"
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("robin_surv_strata_cov works with multiple strata variables", {
  input <- h_prep_survival_input(
    formula = survival::Surv(time, status) ~ age + strata(strata, ecog),
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  result <- robin_surv_strata_cov(
    vars = input,
    data = input$data,
    exp_level = 2,
    control_level = 1,
    contrast = "hazardratio"
  )
  surv_data$strata_ecog <- interaction(surv_data$strata, surv_data$ecog, drop = TRUE)
  input2 <- h_prep_survival_input(
    formula = survival::Surv(time, status) ~ age + strata(strata_ecog),
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  result2 <- robin_surv_strata_cov(
    vars = input2,
    data = input2$data,
    exp_level = 2,
    control_level = 1,
    contrast = "hazardratio"
  )
  expect_equal(result, result2, ignore_formula_env = TRUE)
})

test_that("robin_surv_strata_cov gives the same results as RobinCar functions", {
  input <- h_prep_survival_input(
    formula = survival::Surv(time, status) ~ age + strata(sex),
    data = surv_data,
    treatment = ecog ~ sr(1)
  )
  input$data <- na.omit(input$data)
  result <- robin_surv_strata_cov(
    vars = input,
    data = input$data,
    exp_level = 2,
    control_level = 1,
    contrast = "hazardratio",
    hr_se_plugin_adjusted = FALSE # To get the exact match with RobinCar.
  )
  # These values are extracted from RobinCar (version 1.0.0) results, see
  # `tests-raw/test-survival.R`.
  robincar_result <- list(
    test_stat = -0.4612828,
    test_sigma_l2 = 0.1734316,
    estimate = -0.08566379,
    se = 0.1840128
  )
  expect_equal(result$test_stat, robincar_result$test_stat, tolerance = 1e-5)
  expect_equal(result$test_sigma_l2, robincar_result$test_sigma_l2, tolerance = 1e-4)
  expect_equal(result$estimate, robincar_result$estimate, tolerance = 1e-3)
  expect_equal(result$se, robincar_result$se, tolerance = 1e-3)
})

test_that("h_log_hr_coef_mat works as expected", {
  x <- list(
    estimate = 0.5,
    se = 1,
    pair = structure(
      list(2L, 1L),
      levels = c("A", "B")
    )
  )
  result <- h_log_hr_coef_mat(x)
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_log_hr_coef_mat works as expected for multiple comparisons", {
  x <- list(
    estimate = c(0.5, 0.7, 0.9),
    se = c(1, 2, 3),
    pair = structure(
      list(c(2L, 1L, 3L), c(1L, 3L, 2L)),
      levels = c("A", "B", "C")
    )
  )
  result <- h_log_hr_coef_mat(x)
  expect_snapshot_value(result, tolerance = 1e-4, style = "serialize")
})

test_that("h_test_mat works as expected", {
  x <- list(
    test_stat = c(0.5, 0.7),
    p_value = c(0.05, 0.01),
    pair = structure(
      list(c(2L, 1L), c(1L, 2L)),
      levels = c("A", "B")
    )
  )
  result <- h_test_mat(x)
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_events_table works as expected with strata", {
  vars <- list(
    treatment = "sex",
    time = "time",
    status = "status",
    strata = "strata"
  )
  result <- h_events_table(surv_data, vars)
  expected <- data.frame(
    strata = structure(c(1L, 1L, 2L, 2L, 3L, 3L, 4L), levels = c("0", "1", "2", "3"), class = "factor"),
    sex = structure(c(1L, 2L, 1L, 2L, 1L, 2L, 2L), levels = c("Female", "Male"), class = "factor"),
    Patients = c(27L, 36L, 42L, 71L, 21L, 29L, 1L),
    Events = c(9L, 28L, 28L, 54L, 16L, 28L, 1L)
  )
  expect_identical(result, expected)
})

test_that("h_events_table works as expected with multiple strata", {
  vars <- list(
    treatment = "sex",
    time = "time",
    status = "status",
    strata = c("strata", "ecog")
  )
  result <- h_events_table(surv_data, vars)
  expected <- data.frame(
    strata = structure(c(1L, 1L, 3L, 3L, 4L, 2L, 2L), levels = c("0", "1", "2", "3"), class = "factor"),
    ecog = structure(c(1L, 1L, 1L, 1L, 1L, 2L, 2L), levels = c("0", "1"), class = "factor"),
    sex = structure(c(1L, 2L, 1L, 2L, 2L, 1L, 2L), levels = c("Female", "Male"), class = "factor"),
    Patients = c(27L, 36L, 21L, 29L, 1L, 42L, 71L),
    Events = c(9L, 28L, 16L, 28L, 1L, 28L, 54L)
  )
  expect_identical(result, expected)
})

test_that("h_events_table works as expected without strata", {
  vars <- list(
    treatment = "sex",
    time = "time",
    status = "status",
    strata = NULL
  )
  result <- h_events_table(surv_data, vars)
  expected <- data.frame(
    sex = structure(1:2, levels = c("Female", "Male"), class = "factor"),
    Patients = c(90L, 138L),
    Events = c(53L, 112L)
  )
  expect_identical(result, expected)
})

test_that("robin_surv works as expected without strata or covariates", {
  result <- robin_surv(
    Surv(time, status) ~ 1,
    data = surv_data,
    treatment = ecog ~ sr(1),
    control = surv_control()
  )
  expect_s3_class(result, "surv_effect")
  expect_snapshot_value(result$log_hr_coef_mat, tolerance = 1e-4, style = "deparse")
  expect_snapshot_value(result$test_mat, tolerance = 1e-4, style = "serialize")
  expect_snapshot_value(result$events_table, tolerance = 1e-4, style = "serialize")
})

test_that("robin_surv gives the same results as RobinCar functions without strata or covariates", {
  result <- robin_surv(
    Surv(time, status) ~ 1,
    data = na.omit(surv_data),
    treatment = ecog ~ sr(1)
  )
  # These values are extracted from RobinCar (version 1.0.0) results, see
  # `tests-raw/test-survival.R`.
  robincar_result <- list(
    test_stat = -0.6188324,
    test_sigma_l2 = 0.1782103,
    test_p_val = 0.5360268,
    estimate = -0.1131005,
    se = 0.1830198
  )
  expect_equal(result$test_mat[, "Test Stat."], robincar_result$test_stat, tolerance = 1e-5)
  expect_equal(result$test_mat[, "Pr(>|z|)"], robincar_result$test_p_val, tolerance = 1e-5)
  expect_equal(result$log_hr_coef_mat[, "Estimate"], robincar_result$estimate, tolerance = 1e-4)
  expect_equal(result$log_hr_coef_mat[, "Std.Err"], robincar_result$se, tolerance = 1e-4)
})

test_that("robin_surv gives the same estimate as coxph", {
  skip_if_not_installed("speff2trial")

  data("ACTG175", package = "speff2trial", envir = environment())
  trial_dat <- ACTG175
  trial_dat <- trial_dat[trial_dat$arms %in% c(0, 3), ]
  trial_dat <- data.frame(
    id = trial_dat$pidnum,
    drugs = trial_dat$drugs,
    cd40 = trial_dat$cd40,
    days = trial_dat$days,
    obs = trial_dat$cens,
    didanosine = factor(trial_dat$arms == 3),
    strat = factor(trial_dat$strat)
  )

  coxres <- survival::coxph(
    survival::Surv(days, obs) ~ didanosine,
    data = trial_dat,
    ties = "breslow"
  )
  expect_warning(
    robinres <- robin_surv(
      survival::Surv(days, obs) ~ 1,
      treatment = didanosine ~ pb(strat),
      data = trial_dat
    ),
    "It looks like you have not included all of the variables that were used during randomization",
    fixed = TRUE
  )
  expect_equal(
    robinres$estimate,
    coxres$coefficients,
    ignore_attr = TRUE,
    tolerance = 1e-6
  )
})

test_that("robin_surv works as expected with strata", {
  result <- robin_surv(
    Surv(time, status) ~ 1 + strata(strata),
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  expect_s3_class(result, "surv_effect")
  expect_snapshot_value(result$log_hr_coef_mat, tolerance = 1e-4, style = "deparse")
  expect_snapshot_value(result$test_mat, tolerance = 1e-4, style = "serialize")
  expect_snapshot_value(result$events_table, tolerance = 1e-4, style = "serialize")
})

test_that("robin_surv gives the same results as RobinCar functions with strata", {
  result <- robin_surv(
    Surv(time, status) ~ 1 + strata(sex),
    data = na.omit(surv_data),
    treatment = ecog ~ sr(1)
  )
  # These values are extracted from RobinCar (version 1.0.0) results, see
  # `tests-raw/test-survival.R`.
  robincar_result <- list(
    test_stat = -0.6171326,
    test_sigma_l2 = 0.1749327,
    test_p_val = 0.5371473,
    estimate = -0.1138251,
    se = 0.1847554
  )
  expect_equal(result$test_mat[, "Test Stat."], robincar_result$test_stat, tolerance = 1e-5)
  expect_equal(result$test_mat[, "Pr(>|z|)"], robincar_result$test_p_val, tolerance = 1e-5)
  expect_equal(result$log_hr_coef_mat[, "Estimate"], robincar_result$estimate, tolerance = 1e-3)
  expect_equal(result$log_hr_coef_mat[, "Std.Err"], robincar_result$se, tolerance = 1e-3)
})

test_that("robin_surv works as expected with covariates", {
  result <- robin_surv(
    Surv(time, status) ~ age,
    data = surv_data,
    treatment = ecog ~ sr(1)
  )
  expect_s3_class(result, "surv_effect")
  expect_snapshot_value(result$log_hr_coef_mat, tolerance = 1e-4, style = "deparse")
  expect_snapshot_value(result$test_mat, tolerance = 1e-4, style = "serialize")
  expect_snapshot_value(result$events_table, tolerance = 1e-4, style = "serialize")
})

test_that("robin_surv gives the same results as RobinCar functions with covariates", {
  result <- robin_surv(
    Surv(time, status) ~ age,
    data = na.omit(surv_data),
    treatment = ecog ~ sr(1)
  )
  # These values are extracted from RobinCar (version 1.0.0) results, see
  # `tests-raw/test-survival.R`.
  robincar_result <- list(
    test_stat = -0.4309439,
    test_sigma_l2 = 0.175718,
    test_p_val = 0.6665092,
    estimate = -0.07914235,
    se = 0.181807
  )
  expect_equal(result$test_mat[, "Test Stat."], robincar_result$test_stat, tolerance = 1e-5)
  expect_equal(result$test_mat[, "Pr(>|z|)"], robincar_result$test_p_val, tolerance = 1e-5)
  expect_equal(result$log_hr_coef_mat[, "Estimate"], robincar_result$estimate, tolerance = 1e-3)
  expect_equal(result$log_hr_coef_mat[, "Std.Err"], robincar_result$se, tolerance = 1e-3)
})

test_that("robin_surv gives the same results as RobinCar for single factor covariate", {
  result <- robin_surv(
    Surv(time, status) ~ sex,
    data = na.omit(surv_data),
    treatment = ecog ~ sr(1),
    hr_se_plugin_adjusted = FALSE
  )
  # These values are extracted from RobinCar (version 1.0.0) results, see
  # `tests-raw/test-survival.R`.
  robincar_result <- list(
    test_stat = -0.7582934,
    test_sigma_l2 = 0.1715611,
    test_p_val = 0.4482754,
    estimate = -0.135294,
    se = 0.1797889
  )
  expect_equal(result$test_mat[, "Test Stat."], robincar_result$test_stat, tolerance = 1e-4)
  expect_equal(result$test_mat[, "Pr(>|z|)"], robincar_result$test_p_val, tolerance = 1e-4)
  expect_equal(result$log_hr_coef_mat[, "Estimate"], robincar_result$estimate, tolerance = 1e-1)
  expect_equal(result$log_hr_coef_mat[, "Std.Err"], robincar_result$se, tolerance = 1e-3)
})

test_that("robin_surv works as expected with strata and covariates", {
  result <- robin_surv(
    Surv(time, status) ~ age + ph.karno + strata(sex),
    data = surv_data,
    treatment = ecog ~ sr(1)
  )
  expect_s3_class(result, "surv_effect")
  expect_snapshot_value(result$log_hr_coef_mat, tolerance = 1e-4, style = "deparse")
  expect_snapshot_value(result$test_mat, tolerance = 1e-4, style = "serialize")
  expect_snapshot_value(result$events_table, tolerance = 1e-4, style = "serialize")
})

test_that("robin_surv gives the same results as RobinCar functions with strata and covariates", {
  result <- robin_surv(
    Surv(time, status) ~ age + strata(sex),
    data = na.omit(surv_data),
    treatment = ecog ~ sr(1)
  )
  # These values are extracted from RobinCar (version 1.0.0) results, see
  # `tests-raw/test-survival.R`.
  robincar_result <- list(
    test_stat = -0.4612828,
    test_sigma_l2 = 0.1734316,
    test_p_val = 0.6445957,
    estimate = -0.08566379,
    se = 0.1840128
  )
  expect_equal(result$test_mat[, "Test Stat."], robincar_result$test_stat, tolerance = 1e-5)
  expect_equal(result$test_mat[, "Pr(>|z|)"], robincar_result$test_p_val, tolerance = 1e-5)
  expect_equal(result$log_hr_coef_mat[, "Estimate"], robincar_result$estimate, tolerance = 1e-3)
  expect_equal(result$log_hr_coef_mat[, "Std.Err"], robincar_result$se, tolerance = 1e-3)
})

test_that("robin_surv also works with multiple pairwise comparisons", {
  result <- robin_surv(
    Surv(time, status) ~ 1,
    data = surv_data,
    treatment = strata ~ sr(1)
  )
  expect_s3_class(result, "surv_effect")
  comparisons <- c("1 v.s. 0", "2 v.s. 0", "3 v.s. 0", "2 v.s. 1", "3 v.s. 1", "3 v.s. 2")
  expect_matrix(result$log_hr_coef_mat, ncol = 4, nrow = 6)
  expect_names(rownames(result$log_hr_coef_mat), identical.to = comparisons)
  expect_matrix(result$test_mat, ncol = 2, nrow = 6)
  expect_names(rownames(result$test_mat), identical.to = comparisons)
})

test_that("robin_surv allows the user to optionally define the comparisons of interest", {
  result <- robin_surv(
    Surv(time, status) ~ 1,
    data = surv_data,
    treatment = strata ~ sr(1),
    comparisons = list(c(1, 2), c(3, 3))
  )
  expect_s3_class(result, "surv_effect")
  comparisons <- c("0 v.s. 2", "1 v.s. 2")
  expect_matrix(result$log_hr_coef_mat, ncol = 4, nrow = 2)
  expect_names(rownames(result$log_hr_coef_mat), identical.to = comparisons)
  expect_matrix(result$test_mat, ncol = 2, nrow = 2)
  expect_names(rownames(result$test_mat), identical.to = comparisons)
})

test_that("robin_surv allows to use unadjusted standard error", {
  result <- robin_surv(
    Surv(time, status) ~ age,
    data = surv_data,
    treatment = ecog ~ sr(1),
    hr_se_plugin_adjusted = FALSE
  )
  result_adjusted <- robin_surv(
    Surv(time, status) ~ age,
    data = surv_data,
    treatment = ecog ~ sr(1),
    hr_se_plugin_adjusted = TRUE
  )
  # Only the standard error should differ.
  expect_true(result$se != result_adjusted$se)
  expect_true(result$estimate == result_adjusted$estimate)
  expect_true(result$test_stat == result_adjusted$test_stat)
  expect_true(result$p_value == result_adjusted$p_value)
})

test_that("robin_surv gives the same results as RobinCar for strong correlation covariate with strata", {
  set.seed(2040)
  surv_data2 <- surv_data
  surv_data2$ecog <- factor(ifelse(
    surv_data$strata == 1,
    rbinom(nrow(surv_data), 1, prob = 0.2),
    surv_data$ecog
  ))
  surv_data2 <- surv_data2[surv_data2$strata %in% c(0, 1), ]

  result <- robin_surv(
    formula = Surv(time, status) ~ ecog + strata(strata),
    data = na.omit(surv_data2),
    treatment = sex ~ sr(1),
    hr_se_plugin_adjusted = FALSE
  )
  # These values are extracted from RobinCar (version 1.0.0) results, see
  # `tests-raw/test-survival.R`.
  robincar_result <- list(
    test_stat = 2.35142,
    test_sigma_l2 = 0.1589645,
    test_p_val = 0.01870189,
    estimate = 0.5466888,
    se = 0.2371971
  )
  expect_equal(result$test_mat[, "Test Stat."], robincar_result$test_stat, tolerance = 1e-4)
  expect_equal(result$test_mat[, "Pr(>|z|)"], robincar_result$test_p_val, tolerance = 1e-3)
  expect_equal(result$log_hr_coef_mat[, "Estimate"], robincar_result$estimate, tolerance = 1e-1)
  expect_equal(result$log_hr_coef_mat[, "Std.Err"], robincar_result$se, tolerance = 1e-2)
})

test_that("robin_surv works also with character variable in the correlation case", {
  set.seed(2040)
  surv_data2 <- surv_data
  surv_data2$ecog <- as.character(ifelse(
    surv_data$strata == 1,
    rbinom(nrow(surv_data), 1, prob = 0.2),
    surv_data$ecog
  ))
  surv_data2 <- surv_data2[surv_data2$strata %in% c(0, 1), ]
  # Here we only have ecog 1 for strata 0:
  assert_true(all(surv_data2$ecog[surv_data2$strata == 0] == "1"))

  result <- expect_silent(robin_surv(
    formula = Surv(time, status) ~ ecog + strata(strata),
    data = na.omit(surv_data2),
    treatment = sex ~ sr(1),
    hr_se_plugin_adjusted = FALSE
  ))
  expect_snapshot_value(result$log_hr_coef_mat, tolerance = 1e-2, style = "deparse")
})

test_that("robin_surv can skip the hazard ratio estimation", {
  result <- robin_surv(
    Surv(time, status) ~ age,
    data = surv_data,
    treatment = ecog ~ 1,
    contrast = "none"
  )
  expect_identical(result$contrast, "none")
  expect_scalar_na(result$estimate)
  expect_scalar_na(result$se)
  expect_scalar_na(result$hr_n)
  expect_scalar_na(result$hr_sigma_l2)
  expect_null(result$log_hr_coef_mat)
})

test_that("robin_surv can work with interaction covariates", {
  result <- expect_silent(robin_surv(
    Surv(time, status) ~ age + interaction(I(ph.karno < 80), I(wt.loss > 7)) + strata(ecog),
    data = surv_data,
    treatment = sex ~ sr(1)
  ))
  expect_s3_class(result, "surv_effect")
})

test_that("robin_surv works in a case where previously separate design matrices led to failure", {
  result <- robin_surv(
    Surv(time, status) ~ age + interaction(I(ph.karno < 50), I(wt.loss > 10)) + strata(strata),
    data = surv_data,
    treatment = sex ~ sr(1)
  )
  expect_s3_class(result, "surv_effect")
})

test_that("robin_surv gives a warning if stratified randomization was specified but simple log rank test used", {
  expect_snapshot(
    robin_surv(
      Surv(time, status) ~ 1,
      data = surv_data,
      treatment = sex ~ pb(ecog)
    )
  )
})

test_that("robin_surv does not give a warning if strata are sufficiently included in the analysis model", {
  full_strata <- with(surv_data, interaction(strata, ecog, drop = TRUE))
  incl_strata <- surv_data$strata
  assert_true(h_first_fct_nested_in_second(incl_strata, full_strata))

  result <- expect_silent(robin_surv(
    Surv(time, status) ~ 1 + strata(strata),
    data = surv_data,
    treatment = sex ~ pb(ecog, strata)
  ))
})

test_that("robin_surv does give a warning if strata are not sufficiently included in the analysis model", {
  full_strata <- surv_data$strata
  incl_strata <- surv_data$ecog
  assert_false(h_first_fct_nested_in_second(incl_strata, full_strata))

  expect_snapshot(
    robin_surv(
      Surv(time, status) ~ 1 + strata(ecog),
      data = surv_data,
      treatment = sex ~ pb(strata)
    )
  )
  expect_warning(
    robin_surv(
      Surv(time, status) ~ 1 + strata(ecog),
      data = surv_data,
      treatment = sex ~ pb(strata, ecog)
    ),
    "adjust for all joint levels in your `formula` using `+ strata` or",
    fixed = TRUE
  )
  result <- expect_silent(robin_surv(
    Surv(time, status) ~ 1 + strata(ecog) + strata(strata),
    data = surv_data,
    treatment = sex ~ pb(strata)
  ))
})

test_that("robin_surv does give a warning if strata are not sufficiently included in the covariates", {
  expect_snapshot(
    robin_surv(
      Surv(time, status) ~ 1 + ph.karno,
      data = surv_data,
      treatment = sex ~ pb(ecog)
    )
  )
  result <- expect_silent(robin_surv(
    Surv(time, status) ~ 1 + ph.karno + ecog,
    data = surv_data,
    treatment = sex ~ pb(ecog)
  ))

  surv_data2 <- droplevels(surv_data[surv_data$strata != "3", ])
  expect_warning(
    robin_surv(
      Surv(time, status) ~ 1 + ecog,
      data = surv_data2,
      treatment = sex ~ pb(strata, ecog)
    ),
    "adjust for all joint levels in your `formula` using `+ strata` or",
    fixed = TRUE
  )
  result <- expect_silent(robin_surv(
    Surv(time, status) ~ 1 + strata,
    data = surv_data2,
    treatment = sex ~ pb(ecog)
  ))
})

test_that("robin_surv warns if strata are insufficiently included in covariate adjusted stratified model", {
  expect_snapshot(
    robin_surv(
      Surv(time, status) ~ 1 + strata(ecog) + ph.karno,
      data = surv_data,
      treatment = sex ~ pb(strata)
    )
  )
  expect_warning(
    robin_surv(
      Surv(time, status) ~ 1 + strata(ecog) + ph.karno,
      data = surv_data,
      treatment = sex ~ pb(strata, ecog)
    ),
    "adjust for all joint levels in your `formula` using `+ strata` or",
    fixed = TRUE
  )
  result <- expect_silent(robin_surv(
    Surv(time, status) ~ 1 + strata(strata) + ph.karno,
    data = surv_data,
    treatment = sex ~ pb(strata)
  ))
})

test_that("robin_surv only gives a single warning for randomization strata with multiple comparisons", {
  expect_warning(
    result <- robin_surv(
      Surv(time, status) ~ 1,
      data = surv_data,
      treatment = strata ~ pb(ecog),
    ),
    "It looks like you have not included all of the variables that were used during randomization"
  )
  expect_s3_class(result, "surv_effect")
  comparisons <- c("1 v.s. 0", "2 v.s. 0", "3 v.s. 0", "2 v.s. 1", "3 v.s. 1", "3 v.s. 2")
  expect_matrix(result$log_hr_coef_mat, ncol = 4, nrow = 6)
  expect_names(rownames(result$log_hr_coef_mat), identical.to = comparisons)
})

#' Derive Outcome Values Based on Log Hazard Ratio
#'
#' Compute the derived outcome values based on a given log hazard ratio.
#'
#' @inheritParams survival_score_functions
#' @param covariates (`character`) The column names in `df` to be used for covariate adjustment.
#' @return A data frame containing the same data as the input `df`, but restructured with standardized column names
#'   `index`, `treatment`, `time`, `status`, the covariates and randomization stratification variables,
#'   and an additional column `O_hat` containing the derived outcome values.
#'   For the stratified version, the computations are done separately by stratum, and
#'   the resulting `data.frame` contains an additional `.stratum` column indicating the stratum number.
#' @details Please note that the `covariates` and `randomization_strata` must not include `index`,
#'   `treatment`, `time`, `status` to avoid naming conflicts.
#' @keywords internal
#' @name derived_outcome_vals
NULL

#' @describeIn derived_outcome_vals calculates the derived outcome values for the overall data set.
h_derived_outcome_vals <- function(theta, df, treatment, time, status, covariates, randomization_strata, n = nrow(df)) {
  assert_number(theta)
  assert_string(treatment)
  assert_string(time)
  assert_string(status)
  assert_character(covariates, min.len = 1L, any.missing = FALSE, unique = TRUE)
  assert_character(randomization_strata, any.missing = FALSE, unique = TRUE)
  assert_data_frame(df)
  assert_factor(df[[treatment]], n.levels = 2L)
  assert_numeric(df[[status]])
  assert_true(all(df[[status]] %in% c(0, 1)))
  assert_numeric(df[[time]], lower = 0)

  cov_and_rand_strata <- unique(c(covariates, randomization_strata))
  assert_subset(cov_and_rand_strata, names(df))
  assert_disjunct(cov_and_rand_strata, c("index", "treatment", "time", "status", "treatment_numeric", "O_hat"))

  # Standardize data set format, subset to relevant variables.
  df <- data.frame(
    index = seq_len(nrow(df)),
    treatment = df[[treatment]],
    treatment_numeric = as.numeric(df[[treatment]]) - 1L,
    time = df[[time]],
    status = df[[status]],
    df[cov_and_rand_strata]
  )
  assert_true(!anyNA(df))

  # Sort by time.
  df <- df[order(df$time), , drop = FALSE]

  # Number of patients with events at unique event times.
  df_events_unique <- h_n_events_per_time(
    df = df,
    time = "time",
    status = "status"
  )

  # Add derived outcome column.
  df$O_hat <- NA_real_

  # Calculate quantities which are the same across patients first.
  # These are in parallel to df_events_unique.

  # Hazard ratio.
  exp_theta <- exp(theta)

  # Proportions of patients at risk, per unique event time and treatment arm.
  # Corresponds to \exp(\theta) * \bar{Y}_1(t) and \bar{Y}_0(t).
  # So here theta enters.
  at_risk_matrix <- outer(df$time, df_events_unique$time, FUN = ">=")
  y_bar_1 <- exp_theta * colSums(df$treatment_numeric & at_risk_matrix) / n
  y_bar_0 <- colSums(!df$treatment_numeric & at_risk_matrix) / n
  y_bar <- y_bar_0 + y_bar_1

  # Proportion of patients having an event at this time.
  # Corresponds to d\bar{N}(t). Here we need to be careful about tied event times,
  # therefore we see how many patients have an event at each unique time and divide that by n.
  dn_bar <- df_events_unique$n_events / n

  # Loop over all patients.
  for (i in seq_len(nrow(df))) {
    # Treatment arm?
    trt_i <- df$treatment_numeric[i]

    # Event in this patient?
    delta_i <- df$status[i] == 1

    # Time for this patient.
    t_i <- df$time[i]

    # Does this patient have an event at this unique event time? Corresponds to dN_ij(t).
    dn_ij <- delta_i * (df_events_unique$time == t_i)

    # Is this patient at risk at this unique event time? Corresponds to Y_ij(t).
    # Here theta enters, too.
    y_ij <- as.numeric(t_i >= df_events_unique$time) * ifelse(trt_i, exp_theta, 1)

    # Calculate the weights, Y_bar in opposite treatment arm divided by Y_bar overall.
    weights <- (trt_i * y_bar_0 + (1 - trt_i) * y_bar_1) / y_bar

    # Compute martingale residuals.
    martingale_residuals <- dn_ij - y_ij * dn_bar / y_bar

    # Sum across all unique event times.
    df$O_hat[i] <- sum(weights * martingale_residuals)
  }

  # Return in original order with relevant columns only.
  include_cols <- c("index", "treatment", "time", "status", "O_hat", cov_and_rand_strata)
  df[order(df$index), include_cols, drop = FALSE]
}

#' @describeIn derived_outcome_vals calculates the derived outcome values for each stratum separately.
h_strat_derived_outcome_vals <- function(theta, df, treatment, time, status, strata, covariates, randomization_strata) {
  assert_character(strata, any.missing = FALSE, min.len = 1L, unique = TRUE)
  assert_data_frame(df)
  assert_disjunct(names(df), ".stratum")
  lapply(df[strata], assert_factor)

  assert_true(!anyNA(df))
  n <- nrow(df)

  strata_formula <- paste("~", paste(strata, collapse = "+"))
  df_split <- split(df, f = as.formula(strata_formula), drop = TRUE)

  df_with_outcomes_split <- lapply(
    df_split,
    FUN = h_derived_outcome_vals,
    theta = theta,
    treatment = treatment,
    time = time,
    status = status,
    covariates = covariates,
    randomization_strata = randomization_strata,
    n = n
  )
  strata_number <- seq_along(df_with_outcomes_split)
  df_with_outcomes_split <- Map(
    function(df_stratum, stratum_number) {
      df_stratum$.stratum <- stratum_number
      df_stratum
    },
    df_with_outcomes_split,
    strata_number
  )
  do.call(rbind, df_with_outcomes_split)
}

#' Get Linear Model Input Data
#'
#' Prepare the input data for a linear model based on the provided data frame and model formula.
#'
#' @param df (`data.frame`) Including the covariates needed for the `model`, as well as the derived outcome `O_hat`
#'   and the `treatment` factor.
#' @param df_with_stratum (`data.frame`) A data frame with an additional column for the stratum, as returned by
#'   [h_strat_derived_outcome_vals()].
#' @param model (`formula`) The right-hand side only model formula.
#' @return A list containing for each element of the `treatment` factor a list with the
#'   corresponding model matrix `X` and the response vector `y`. For the stratified version,
#'   the model matrix `X` includes the `.stratum` column.
#' @keywords internal
#' @name get_lm_input
NULL

#' @describeIn get_lm_input Get the linear model input data for the overall data set.
h_get_lm_input <- function(df, model) {
  assert_data_frame(df)
  assert_formula(model)
  assert_true(length(model) == 2L) # Ensures right-hand side only formula.
  assert_subset(c("treatment", "O_hat", all.vars(model)), names(df))
  assert_factor(df$treatment)

  # Add outcome:
  model <- stats::update(model, O_hat ~ .)
  includes_intercept <- attr(stats::terms(model), "intercept") == 1
  assert_true(includes_intercept)

  # Create overall design matrix and response vector.
  mf <- stats::model.frame(model, data = df)
  x <- stats::model.matrix(model, data = mf)
  assert_true(identical(colnames(x)[1L], "(Intercept)"))
  x <- x[, -1L, drop = FALSE] # Remove intercept.
  y <- stats::model.response(mf)

  # Split both design matrix and response vector by treatment arm.
  index_by_trt <- split(seq_along(y), f = df$treatment)
  lapply(
    index_by_trt,
    function(this_index) {
      list(X = x[this_index, , drop = FALSE], y = y[this_index])
    }
  )
}

#' @describeIn get_lm_input Get the linear model input data with attached stratum column.
h_get_strat_lm_input <- function(df_with_stratum, model) {
  assert_data_frame(df_with_stratum)
  assert_subset(".stratum", names(df_with_stratum))
  model_with_stratum <- stats::update(model, ~ . + .stratum) # Important: no left-hand side.
  h_get_lm_input(df = df_with_stratum, model = model_with_stratum)
}

#' Calculate Coefficient Estimates and Corresponding Residuals from Linear Model Input
#'
#' Calculate the coefficient estimates for each treatment arm from the linear model input data.
#' Also returns the corresponding residuals.
#'
#' @param lm_input (`list`) A list containing the linear model input data for each treatment arm, as returned by
#'   [h_get_lm_input()].
#' @param strat_lm_input (`list`) A list containing the linear model input data
#'   for each treatment arm and including the `.strata` column in the design matrix,
#'   as returned by [h_get_strat_lm_input()].
#' @return A list with:
#'
#'   - `beta_est`: the coefficient estimates for each treatment arm.
#'   - `residuals`: the corresponding residuals for each treatment arm.
#'
#' @keywords internal
#' @name get_lm_results
NULL

#' @describeIn get_lm_results Calculate the coefficient estimates for the overall data set.
h_get_lm_results <- function(lm_input) {
  assert_list(lm_input, types = "list")

  # Fit the model separately for each treatment arm.
  beta_est <- list()
  residuals <- list()

  for (group in names(lm_input)) {
    assert_matrix(lm_input[[group]]$X, any.missing = FALSE)
    assert_numeric(lm_input[[group]]$y, any.missing = FALSE)

    # Get the design matrix for this treatment arm.
    x <- lm_input[[group]]$X

    # Center it.
    x <- scale(x, center = TRUE, scale = FALSE)

    # Get the derived outcome values, the response.
    y <- lm_input[[group]]$y

    # Fit the model without intercept.
    lm_fit <- stats::lm.fit(x, y, singular.ok = FALSE)

    # Save the coefficients.
    beta_est[[group]] <- lm_fit$coefficients

    # Save the residuals, adjusted for the fact that we centered x and don't
    # have an intercept in the model.
    residuals[[group]] <- lm_fit$residuals - mean(y)
  }

  list(
    beta_est = beta_est,
    residuals = residuals
  )
}

#' @describeIn get_lm_results Calculate the coefficient estimates using the stratified input.
h_get_strat_lm_results <- function(strat_lm_input) {
  assert_list(strat_lm_input, types = "list", len = 2L, names = "unique")
  group_names <- names(strat_lm_input)

  # Get coefficient estimates separately for each treatment arm.
  beta_est <- list()
  resids <- list()

  for (group in group_names) {
    x <- strat_lm_input[[group]]$X
    y <- strat_lm_input[[group]]$y

    stratum_col <- match(".stratum", colnames(x))
    stratum <- as.integer(x[, stratum_col])
    x <- x[, -stratum_col, drop = FALSE]
    unique_strata <- unique(stratum)

    xtxs <- list()
    xtys <- list()

    for (stratum_index in seq_along(unique_strata)) {
      # Save the corresponding cross products
      # for this group and stratum.
      in_stratum <- stratum == unique_strata[stratum_index]

      # Get the design matrix for this treatment arm.
      this_x <- x[in_stratum, , drop = FALSE]

      # Center it.
      this_x <- scale(this_x, center = TRUE, scale = FALSE)

      # Save centered design matrix back (for residual calculation later).
      x[in_stratum, ] <- this_x

      # Get the derived outcome values, the response.
      this_y <- y[in_stratum]

      # Save the centered response back (for residual calculation later).
      y[in_stratum] <- this_y - mean(this_y)

      # Save the cross products.
      xtxs[[stratum_index]] <- crossprod(this_x)
      xtys[[stratum_index]] <- crossprod(this_x, this_y)
    }

    # Sum across strata.
    xtx <- Reduce("+", xtxs)
    xty <- Reduce("+", xtys)

    # Get the coefficients.
    beta_est[[group]] <- solve(xtx, xty)

    # Compute the residuals.
    resids[[group]] <- y - as.numeric(x %*% beta_est[[group]])
  }

  list(
    beta_est = beta_est,
    residuals = resids
  )
}

test_that("h_derived_outcome_vals works as expected", {
  surv_data_full <- na.omit(surv_data)
  result <- h_derived_outcome_vals(
    theta = 0,
    df = surv_data_full,
    treatment = "sex",
    time = "time",
    status = "status",
    covariates = c("age", "ph.karno"),
    randomization_strata = character(),
    n = 400
  )
  expect_data_frame(result, ncol = 4 + 2 + 1, nrow = nrow(surv_data_full))
  head_result <- result[seq_len(6), ]
  head_expected <- data.frame(
    index = 1:6,
    treatment = factor(c("Male", "Male", "Male", "Female", "Female", "Male")),
    time = c(455, 210, 1022, 310, 361, 218),
    status = c(1, 1, 0, 1, 1, 1),
    O_hat = c(
      -0.059407303167448,
      0.280849881485069,
      -1.1574498129749,
      0.128415337453864,
      0.0645341495986705,
      0.27539351150966
    ),
    age = c(68, 57, 74, 68, 71, 53),
    ph.karno = c(90, 90, 50, 70, 60, 70)
  )
  expect_equal(head_result, head_expected, tolerance = 1e-4, ignore_attr = TRUE)
})

test_that("h_strat_derived_outcome_vals works as expected", {
  surv_data_full <- na.omit(surv_data)
  result <- h_strat_derived_outcome_vals(
    theta = 0,
    df = surv_data_full,
    treatment = "sex",
    time = "time",
    status = "status",
    strata = "strata",
    covariates = c("age", "ph.karno"),
    randomization_strata = character()
  )
  expect_data_frame(result, nrow = nrow(surv_data_full))
  expect_names(
    names(result),
    identical.to = c("index", "treatment", "time", "status", "O_hat", "age", "ph.karno", ".stratum")
  )
  expect_identical(as.integer(table(result$.stratum)), as.integer(table(surv_data_full$strata)))
})

test_that("h_strat_derived_outcome_vals works with multiple strata", {
  surv_data_full <- na.omit(surv_data)
  result <- h_strat_derived_outcome_vals(
    theta = 0,
    df = surv_data_full,
    treatment = "sex",
    time = "time",
    status = "status",
    strata = c("strata", "ecog"),
    covariates = c("age", "ph.karno"),
    randomization_strata = "ecog"
  )
  expect_data_frame(result, nrow = nrow(surv_data_full))
  expect_names(
    names(result),
    identical.to = c("index", "treatment", "time", "status", "O_hat", "age", "ph.karno", "ecog", ".stratum")
  )
  expect_identical(
    as.integer(table(result$.stratum)),
    as.integer(table(interaction(
      surv_data_full$strata,
      surv_data_full$ecog,
      drop = TRUE
    )))
  )
})

test_that("h_get_lm_input works as expected", {
  set.seed(941)
  df <- data.frame(
    index = 1:5,
    treatment = factor(c(0, 1, 0, 1, 0)),
    covariate1 = rnorm(5),
    covariate2 = rnorm(5),
    O_hat = rnorm(5)
  )
  result <- h_get_lm_input(df, model = ~ covariate1 + covariate2)
  expect_list(result, len = 2L)
  expect_list(result[["0"]], len = 2L)
  expect_matrix(result[["0"]][["X"]], ncol = 2L, nrow = 3L)
  expect_numeric(result[["0"]][["y"]], len = 3L)
  expect_list(result[["1"]], len = 2L)
  expect_matrix(result[["1"]][["X"]], ncol = 2L, nrow = 2L)
  expect_numeric(result[["1"]][["y"]], len = 2L)
})

test_that("h_get_lm_input fails when there is no intercept", {
  set.seed(941)
  df <- data.frame(
    index = 1:5,
    treatment = factor(c(0, 1, 0, 1, 0)),
    covariate1 = rnorm(5),
    covariate2 = rnorm(5),
    O_hat = rnorm(5)
  )
  expect_error(
    h_get_lm_input(df, model = ~ 0 + covariate1 + covariate2),
    "Assertion on 'includes_intercept' failed"
  )
})

test_that("h_get_lm_input works correctly across treatment levels with character covariates", {
  set.seed(941)
  df <- data.frame(
    index = 1:5,
    treatment = factor(rep(c(0, 1), each = 5)),
    # Here we have A, B for treatment 0 only, C, D for treatment 1 only:
    covariate1 = c("A", "B", "A", "B", "A", "C", "D", "C", "D", "C"),
    covariate2 = rnorm(5),
    O_hat = rnorm(5)
  )
  result <- h_get_lm_input(df, model = ~ covariate1 + covariate2)
  expect_list(result, len = 2L)
  x_cols <- lapply(result, function(lst) colnames(lst[["X"]]))
  expect_true(identical(x_cols[[1]], x_cols[[2]]))
  # Note that this case will lead to rank-deficient design matrices, but at least
  # the user will get an appropriate error message, instead of a wrong result downstream.
})

test_that("h_get_strat_lm_input works as expected", {
  set.seed(941)
  df_with_stratum <- rbind(
    data.frame(
      index = 1:3,
      treatment = factor(c(0, 1, 0), labels = c("A", "B")),
      covariate1 = rnorm(3),
      covariate2 = rnorm(3),
      O_hat = rnorm(3),
      .stratum = 1
    ),
    data.frame(
      index = 4:5,
      treatment = factor(c(1, 0), labels = c("A", "B")),
      covariate1 = rnorm(2),
      covariate2 = rnorm(2),
      O_hat = rnorm(2),
      .stratum = 2
    )
  )
  result <- h_get_strat_lm_input(df_with_stratum, model = ~ covariate1 + covariate2)
  expect_list(result, len = 2L)
  expect_names(names(result), identical.to = c("A", "B"))
  # Check treatment group A:
  expect_list(result[["A"]], len = 2L)
  expect_matrix(result[["A"]][["X"]], ncol = 3L, nrow = 3L)
  expect_subset(".stratum", colnames(result[["A"]][["X"]]))
  expect_numeric(result[["A"]][["y"]], len = 3L)
  # Check treatment group B:
  expect_list(result[["B"]], len = 2L)
  expect_matrix(result[["B"]][["X"]], ncol = 3L, nrow = 2L)
  expect_subset(".stratum", colnames(result[["B"]][["X"]]))
  expect_numeric(result[["B"]][["y"]], len = 2L)
})

test_that("h_get_lm_results works as expected", {
  set.seed(941)
  nobs <- 10
  df <- data.frame(
    treatment = factor(sample(c("A", "B"), nobs, replace = TRUE)),
    covariate1 = rnorm(nobs),
    covariate2 = rnorm(nobs),
    O_hat = rnorm(nobs)
  )
  lm_input <- h_get_lm_input(df, model = ~ covariate1 + covariate2)
  result <- h_get_lm_results(lm_input)
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_get_strat_lm_results works as expected", {
  set.seed(941)
  nobs <- 10
  df_with_stratum <- rbind(
    data.frame(
      treatment = factor(sample(c("A", "B"), nobs, replace = TRUE)),
      covariate1 = rnorm(nobs),
      covariate2 = rnorm(nobs),
      O_hat = rnorm(nobs),
      .stratum = 1
    ),
    data.frame(
      treatment = factor(sample(c("A", "B"), nobs, replace = TRUE)),
      covariate1 = rnorm(nobs),
      covariate2 = rnorm(nobs),
      O_hat = rnorm(nobs),
      .stratum = 2
    )
  )
  strat_lm_input <- h_get_strat_lm_input(df_with_stratum, model = ~ covariate1 + covariate2)
  result <- h_get_strat_lm_results(strat_lm_input)
  expect_list(result, len = 2L)
  expect_names(names(result), identical.to = c("beta_est", "residuals"))
  expect_snapshot_value(result, tolerance = 1e-4, style = "serialize")
})

#' Heteroskedasticity-consistent covariance matrix for predictions
#' @description The heteroskedasticity-consistent covariance matrix for predictions
#' is obtained with `sandwich::vocvHC` using sandwich method.
#' @param x (`prediction_cf`) Counter-factual prediction.
#' @param type (`character`) Type of HC covariance matrix.
#' @param ... Additional arguments for `sandwich::vcovHC`.
#' @export
#' @return Matrix of the heteroskedasticity-consistent covariance for the predictions.
vcovHC <- function(x, type = "HC3", ...) { # nolint
  assert_class(x, "prediction_cf")
  fit <- x$fit
  vc <- sandwich::vcovHC(fit, type = type)
  mm <- x$model_matrix
  n <- nrow(mm) / length(x$estimate)
  md <- family(fit)$mu.eta(x$predictions_linear) / n
  z <- block_sum(as.vector(md) * mm, n)
  ret <- z %*% vc %*% t(z)
  dimnames(ret) <- list(names(x$estimate), names(x$estimate))
  ret
}

#' Find Data in a Fit
#' @export
#' @param fit A fit object.
#' @param ... Additional arguments.
#' @return A data frame used in the fit.
find_data <- function(fit, ...) {
  UseMethod("find_data")
}
#' @export
find_data.glm <- function(fit, ...) {
  fit$data
}
#' @export
find_data.lm <- function(fit, ...) {
  stop("data must be provided explicitly for lm objects")
}

test_that("find_data works for glm", {
  expect_identical(
    find_data(fit_glm),
    fit_glm$data
  )
})

test_that("find_data fails for lm", {
  expect_error(
    find_data(fit_lm),
    "data must be provided explicitly for lm objects"
  )
})

#' `RobinCar2` Package
#'
#' `RobinCar2` implements unbiased prediction and robust inference of variance of a fit in R.
#'
#' @aliases RobinCar2-package
#' @import checkmate
#' @importFrom numDeriv grad
#' @importFrom stats predict residuals fitted model.response model.matrix coefficients family
#' @importFrom stats gaussian terms glm var family pnorm var as.formula qnorm lm confint
#' @importFrom sandwich vcovHC
#' @importFrom MASS negative.binomial
#' @importFrom utils combn tail
"_PACKAGE"

#' S3 Methods for `surv_effect`
#' @param x (`surv_effect`) the obtained result from [robin_surv()].
#' @param ... ignored additional arguments (for compatibility).
#' @name surv_effect_methods
#'
#' @examples
#' x <- robin_surv(
#'   formula = Surv(time, status) ~ meal.cal + age + strata(strata),
#'   data = surv_data,
#'   treatment = sex ~ pb(strata)
#' )
NULL

#' @export
#' @describeIn surv_effect_methods prints the `surv_effect` object.
#' @examples
#' print(x)
print.surv_effect <- function(x, ...) {
  cat("Model        : ", deparse(as.formula(x$model)), "\n", sep = "")

  cat(
    "Randomization: ",
    deparse(x$randomization),
    " (",
    randomization_schema$schema[randomization_schema$id == x$schema],
    ")\n",
    sep = ""
  )

  strata_variables <- x$vars$strata
  uses_stratification <- length(strata_variables) > 0
  if (uses_stratification) {
    cat("Stratification variables: ", toString(strata_variables), "\n")
  }

  covariates <- x$vars$covariates
  uses_covariates <- length(covariates) > 0
  if (uses_covariates) {
    cat(
      "Covariates adjusted for: ",
      toString(covariates),
      " (including interactions with ",
      x$vars$treatment,
      ")\n",
      sep = ""
    )
  }

  contr_type <- switch(x$contrast,
    hazardratio = paste0(
      if (uses_covariates) "Covariate-adjusted ",
      if (uses_stratification) "Stratified ",
      "Log Hazard Ratio"
    ),
    none = "None"
  )
  cat(sprintf("\nContrast     : %s\n", contr_type), sep = "")

  if (x$contrast == "hazardratio") {
    cat("\n")
    stats::printCoefmat(
      x$log_hr_coef_mat
    )
  }

  cat("\n")

  test_type <- switch(x$test,
    logrank = paste0(
      if (uses_covariates) "Covariate-adjusted ",
      if (uses_stratification) "Stratified ",
      "Log-Rank"
    )
  )
  cat(sprintf("Test         : %s\n\n", test_type), sep = "")

  stats::printCoefmat(
    x$test_mat,
    tst.ind = 1L,
    has.Pvalue = TRUE
  )
}

#' @export
#' @rdname surv_effect_methods
table <- function(x, ...) UseMethod("table")

#' @export
#' @rdname surv_effect_methods
table.default <- function(x, ...) base::table(x, ...)

#' @export
#' @describeIn surv_effect_methods prints and returns invisibly the events table of the `surv_effect` object.
#' @examples
#' table(x)
table.surv_effect <- function(x, ...) {
  cat(
    "Number of patients and events per",
    ifelse(length(x$vars$strata), " stratum and ", " "),
    "treatment arm:\n",
    sep = ""
  )
  print(x$events_table)
  invisible(x$events_table)
}

#' Confidence interval function.
#' @rdname confint
#' @export
confint.surv_effect <- function(object, parm, level = 0.95, transform, ...) {
  if (object$contrast == "hazardratio") {
    h_confint(object$log_hr_coef_mat, parm = parm, level = level, transform = transform, ...)
  } else {
    stop(
      "No contrast was estimated; confidence interval is not available."
    )
  }
}

#' S3 Methods for `prediction_cf`
#' @param x (`prediction_cf`)\cr the obtained counter-factual prediction object.
#' @param level (`number`)\cr the significance level.
#' @name prediction_cf_methods
#' @return No return value.
NULL

#' @describeIn prediction_cf_methods prints the prediction_cf object.
#' @exportS3Method
#' @keywords internal
print.prediction_cf <- function(x, level = 0.95, ...) {
  assert_number(level, lower = 0, upper = 1)
  cat("Model        : ", deparse(as.formula(x$fit)), "\n")
  cat(
    "Randomization: ",
    deparse(x$treatment_formula),
    " (",
    randomization_schema$schema[randomization_schema$id == x$schema],
    ")\n"
  )
  cat("Variance Type: ", x$variance_name, "\n")
  cat("Marginal Mean: \n")
  stats::printCoefmat(
    confint(x, level = level, include_se = TRUE),
    ...
  )
}

#' Confidence Interval
#' @description Obtain the confidence interval for the marginal mean or the contrast.
#' @name confint
#' @param object Object to construct confidence interval.
#' @param parm (`character` or `integer`) Names of the parameters to construct confidence interval.
#' @param level (`numeric`) Confidence level.
#' @param transform (`function`) Transform function.
#' @param include_se (`flag`) Whether to include the standard error as a column in the result matrix.
#' @param ... Not used.
#' @export
#' @return A `matrix` of the confidence interval.
#' @examples
#' robin_res <- robin_glm(
#'   y_b ~ treatment * s1,
#'   data = glm_data, treatment = treatment ~ s1, contrast = "log_risk_ratio"
#' )
#' confint(robin_res$marginal_mean, level = 0.7)
#' confint(robin_res$contrast, parm = 1:3, level = 0.9)
confint.prediction_cf <- function(object, parm, level = 0.95, include_se = FALSE, ...) {
  x <- cbind(
    object$estimate,
    sqrt(diag(object$variance))
  )
  rownames(x) <- names(object$estimate)
  colnames(x) <- c("Estimate", "Std.Err")
  h_confint(x, parm = parm, level = level, transform = identity, include_se = include_se, ...)
}

#' Covariate adjusted glm model
#' @param formula (`formula`) A formula of analysis.
#' @param data (`data.frame`) Input data frame.
#' @param treatment (`formula` or `character(1)`) A formula of treatment assignment or assignment by stratification,
#' or a string name of treatment assignment.
#' @param contrast (`function` or `character(1)`) A function to calculate the treatment effect, or character of
#' "difference", "risk_ratio", "odds_ratio" for default contrasts.
#' @param contrast_jac (`function`) A function to calculate the Jacobian of the contrast function. Ignored if using
#' default contrasts.
#' @param vcov (`function`) A function to calculate the variance-covariance matrix of the treatment effect,
#' including `vcovHC` and `vcovG`.
#' @param family (`family`) A family object of the glm model.
#' @param vcov_args (`list`) Additional arguments passed to `vcov`.
#' @param pair Pairwise treatment comparison.
#' @param ... Additional arguments passed to `glm` or `glm.nb`.
#' @details
#' If family is `MASS::negative.binomial(NA)`, the function will use `MASS::glm.nb` instead of `glm`.
#' @export
#' @return A robin_output object, with `marginal_mean` and `contrast` components.
#' @examples
#' robin_glm(
#'   y ~ treatment * s1,
#'   data = glm_data,
#'   treatment = treatment ~ s1, contrast = "difference"
#' )
robin_glm <- function(
  formula,
  data,
  treatment,
  contrast = c("difference", "risk_ratio", "odds_ratio", "log_risk_ratio", "log_odds_ratio"),
  contrast_jac = NULL,
  vcov = "vcovG",
  family = gaussian(),
  vcov_args = list(),
  pair,
  ...
) {
  attr(formula, ".Environment") <- environment()
  if (is.function(family)) {
    family <- family()
  }
  assert_subset(all.vars(formula), names(data))
  assert_subset(all.vars(treatment), names(data))
  has_interaction <- h_interaction(formula, treatment)
  use_vcovhc <- identical(vcov, "vcovHC") || identical(vcov, vcovHC)
  if (test_character(contrast)) {
    contrast <- match.arg(contrast)
  }
  if (use_vcovhc && (has_interaction || !identical(contrast, "difference") || !identical(family$family, "gaussian"))) {
    stop(
      "Huber-White variance estimator is ONLY supported when the expected outcome difference is estimated",
      "using a linear model without treatment-covariate interactions; see the 2023 FDA guidance."
    )
  }
  # check if using negative.binomial family with NA as theta.
  # If so, use MASS::glm.nb instead of glm.
  if (identical(family$family, "Negative Binomial(NA)")) {
    fit <- MASS::glm.nb(formula, data = data, ...)
  } else {
    fit <- glm(formula, family = family, data = data, ...)
  }
  pc <- eval(bquote(predict_counterfactual(fit, treatment, data, vcov = .(substitute(vcov)), vcov_args = vcov_args)))
  if (missing(pair)) {
    pair <- pairwise(names(pc$estimate))
  }

  if (test_character(contrast)) {
    if (contrast %in% c("odds_ratio", "risk_ratio")) {
      warning(
        c(
          "Consider using the log transformation `log_odds_ratio` and `log_risk_ratio` ",
          "to replace `odds_ratio` and `risk_ratio` to improve the performance of normal approximation."
        )
      )
    }
    trt_eff <- get(contrast)(pc, pair = pair, contrast_name = contrast)
  } else {
    assert_function(contrast, args = c("x", "y"))
    assert_function(contrast_jac, null.ok = TRUE, args = c("x", "y"))
    if (is.null(contrast_jac)) {
      contrast_jac <- eff_jacob(contrast)
    }
    contrast_name_full <- deparse(substitute(contrast))
    contrast_name <- paste(contrast_name_full[1], if (length(contrast_name_full) > 1) "...")
    trt_eff <- treatment_effect(
      pc,
      eff_measure = contrast, eff_jacobian = contrast_jac, pair = pair, contrast_name = contrast_name
    )
  }
  structure(
    list(
      marginal_mean = pc,
      contrast = trt_eff
    ),
    class = "robin_output"
  )
}

#' @export
print.robin_output <- function(x, ...) {
  print(x$marginal_mean)
  cat(sprintf("\nContrast     :  %s\n", x$contrast$contrast))
  print(x$contrast)
}

# robin_glm ----

test_that("robin_glm works correctly", {
  expect_silent(
    robin_glm(
      y ~ treatment * s1,
      data = glm_data,
      treatment = treatment ~ s1,
      contrast = "difference"
    )
  )
  expect_silent(
    robin_glm(y_b ~ treatment * s1, data = glm_data, treatment = treatment ~ s1, contrast = "difference")
  )
  expect_silent(
    robin_glm(y_b ~ treatment * s1, data = glm_data, treatment = treatment ~ s1, contrast = "log_risk_ratio")
  )
  expect_silent(
    robin_glm(y_b ~ treatment * s1, data = glm_data, treatment = treatment ~ s1, contrast = "log_odds_ratio")
  )
  expect_warning(
    robin_glm(y_b ~ treatment * s1, data = glm_data, treatment = treatment ~ s1, contrast = "risk_ratio"),
    "Consider using the log"
  )
  expect_warning(
    robin_glm(y_b ~ treatment * s1, data = glm_data, treatment = treatment ~ s1, contrast = "odds_ratio"),
    "Consider using the log"
  )
  expect_error(
    robin_glm(
      y_b ~ treatment * s1,
      data = glm_data,
      treatment = treatment ~ s1,
      contrast = "odds_ratio",
      vcov = "vcovHC"
    ),
    "Huber-White variance estimator is ONLY"
  )
  expect_error(
    robin_glm(
      y_b ~ treatment * s1,
      data = glm_data,
      treatment = treatment ~ s1,
      contrast = "difference",
      vcov = "vcovHC"
    ),
    "Huber-White variance estimator is ONLY"
  )
  expect_silent(robin_glm(y_b ~ treatment * s1, data = glm_data, treatment = treatment ~ s1, contrast = h_diff))
})


test_that("robin_glm works for glm.nb", {
  glm_data2 <- glm_data
  glm_data2$y_b <- rep(seq_len(10), nrow(glm_data2) / 10)
  expect_silent(
    robin_glm(
      y_b ~ treatment * s1,
      data = glm_data2,
      treatment = treatment ~ s1,
      family = MASS::negative.binomial(theta = NA),
      contrast = "difference"
    )
  )
})

test_that("robin_glm can be printed correctly", {
  expect_snapshot(robin_res1)
  expect_snapshot(robin_res2)
  expect_snapshot(
    robin_glm(y_b ~ treatment + s1, data = glm_data, treatment = treatment ~ s1)
  )
})

#' Randomization schema
#' @keywords internal
randomization_schema <- data.frame(
  schema = c("Pocock-Simon", "Permuted-Block", "Simple"),
  id = c("ps", "pb", "sr"),
  stringsAsFactors = FALSE
)

#' Extract Variable Names
#'
#' @param treatment (`string` or `formula`) string name of the treatment, or a formula.
#'
#' @details Extract the formula elements, including `treatment`, `schema` and `strata`.
#'
#' @return A list of three elements, `treatment`, `schema` and `strata`.
#' @keywords internal
h_get_vars <- function(treatment) {
  assert_formula(treatment)
  if (!identical(length(treatment), 3L)) {
    stop("treatment formula must be of type treatment ~ strata")
  }
  if (!is.name(treatment[[2]])) {
    stop("left hand side of the treatment formula should be a single name!")
  }
  treatvar <- as.character(treatment[[2]])
  tms <- terms(treatment, specials = randomization_schema$id)
  schema <- names(Filter(Negate(is.null), attr(tms, "specials")))
  if (length(schema) > 1) {
    stop("only one randomization schema is allowed!")
  } else if (length(schema) == 0) {
    schema <- "sr"
  }
  strata <- setdiff(all.vars(treatment[[3]]), ".")
  list(
    treatment = treatvar,
    schema = schema,
    strata = strata
  )
}

#' Prepare Survival Input
#'
#' @param formula (`formula`) with a left hand side of the form `Surv(time, status)` and a right hand side
#'   defining optional covariates or just `1` if there are no covariates.
#' @param data (`data.frame`) containing the variables in the formula.
#' @inheritParams h_get_vars
#'
#' @details Note that `formula` can also contain an externally defined [survival::Surv]
#'   object. In this case, the `time` and `status` variables are extracted
#'   and added to the `data` input. Note that it is up to the user to ensure that in this
#'   case the column binding is correct, i.e., that the rows of the `data` match
#'   with the rows of the `Surv` object. In addition, the same named variables must not appear
#'   in both the `data` and the `Surv` object, to avoid ambiguity (this is a difference
#'   vs. the behavior of [survival::coxph()] for better transparency).
#'
#' @return A list containing the following elements:
#' - `data`: The potentially updated data set.
#' - `time`: Name of the time variable.
#' - `status`: Name of the status variable.
#' - `treatment`: Name of the treatment variable.
#' - `strata`: Name of the strata variable.
#' - `schema`: Randomization schema.
#' - `randomization_strata`: Names of the randomization strata variables.
#' - `covariates`: Names of the covariates in the model.
#' - `model`: A formula only including the covariates, but not treatment or strata variables.
#' - `n_levels`: Number of treatment levels.
#' - `levels`: Names of the treatment levels.
#'
#' @keywords internal
h_prep_survival_input <- function(formula, data, treatment) {
  # Process the treatment formula.
  trt_vars <- h_get_vars(treatment)
  assert_data_frame(data)
  assert_formula(formula)
  assert_true(identical(length(formula), 3L))
  assert_subset(c(trt_vars$treatment, trt_vars$strata), colnames(data))
  assert(
    test_character(data[[trt_vars$treatment]]),
    test_factor(data[[trt_vars$treatment]])
  )

  if (test_character(data[[trt_vars$treatment]])) {
    data[[trt_vars$treatment]] <- factor(data[[trt_vars$treatment]])
  }
  trt_lvls <- levels(data[[trt_vars$treatment]])
  n_lvls <- length(trt_lvls)

  # Process analysis model formula.
  tms <- terms(formula, specials = "strata")
  specials <- survival::untangle.specials(tms, "strata")
  analysis_strata <- if (length(specials$vars) > 0) {
    all.vars(as.formula(paste("~", paste(specials$vars, collapse = " + "))))
  } else {
    character()
  }

  covariates <- setdiff(all.vars(formula[[3]]), analysis_strata)
  assert_disjunct(trt_vars$treatment, covariates)

  # Extract survival time and censoring indicator from the left hand side of the formula.
  lhs <- formula[[2]]
  if (inherits(lhs, "call") && (lhs[[1]] == as.name("Surv") || lhs[[1]] == as.name("survival::Surv"))) {
    surv_vars <- as.character(lhs)[-1]
    assert_subset(surv_vars, colnames(data))
    time_var <- surv_vars[1]
    status_var <- surv_vars[2]
  } else if (survival::is.Surv(lhs_evaluated <- try(eval(lhs), silent = TRUE))) {
    assert_true(identical(attr(lhs_evaluated, "type"), "right"))
    lhs_matrix <- as.matrix(lhs_evaluated)
    assert_true(identical(nrow(lhs_matrix), nrow(data)))
    lhs_names <- colnames(lhs_matrix)
    time_var <- lhs_names[1]
    status_var <- lhs_names[2]
    if (any(c(time_var, status_var) %in% names(data))) {
      stop("Ambiguous names provided in Surv object and in data")
    }
    data <- cbind(data, lhs_matrix)
  } else {
    stop("Left hand side of formula must be a Surv() object.")
  }

  # Extract model without left hand side and without strata() terms.
  covariates_formula <- if (length(specials$vars) > 0) {
    stats::update(formula, as.formula(paste0("~ . -", paste(specials$vars, collapse = "-"))))
  } else {
    formula
  }
  model <- as.formula(as.call(as.list(covariates_formula)[-2L]))

  list(
    data = data,
    time = time_var,
    status = status_var,
    treatment = trt_vars$treatment,
    randomization_strata = trt_vars$strata,
    strata = analysis_strata,
    schema = trt_vars$schema,
    covariates = covariates,
    model = model,
    n_levels = n_lvls,
    levels = trt_lvls
  )
}

#' Count Number of Events per Unique Event Time
#'
#' This function counts the number of events at each unique event time point in a survival dataset.
#'
#' @details If there are no events in the dataset, it returns an empty `data.frame`.
#'
#' @param df (`data.frame`) containing the survival data.
#' @param time (`string`) name of the time variable.
#' @param status (`string`) name of the status variable, where 1 indicates an event and 0 indicates censoring.
#' @return A `data.frame` with two columns: `time` and `n_events`, where `n_events` is the
#'   number of events at each time point.
#' @keywords internal
h_n_events_per_time <- function(df, time, status) {
  assert_data_frame(df)
  assert_string(time)
  assert_string(status)
  assert_numeric(df[[time]], any.missing = FALSE)
  assert_numeric(df[[status]], any.missing = FALSE)
  assert_true(all(df[[status]] %in% c(0, 1)))

  has_event <- df[[status]] == 1
  df_events <- df[has_event, , drop = FALSE]
  if (nrow(df_events) == 0) {
    return(data.frame(time = numeric(0), n_events = integer(0)))
  }
  times_count <- stats::aggregate(
    df_events[[status]],
    by = list(time = df_events[[time]]),
    FUN = length
  )
  data.frame(
    time = times_count$time,
    n_events = times_count$x
  )
}

#' Block Sum of a matrix
#' @keywords internal
block_sum <- function(x, n) {
  assert_matrix(x)
  nr <- nrow(x) / n
  matrix(colSums(matrix(x, nrow = n)), nrow = nr)
}

#' @export
#' @rdname contrast
pairwise <- function(levels, x = levels) {
  assert(
    test_integerish(x),
    test_character(x)
  )
  all_combs <- combn(x, 2L)
  custom_contrast(levels, all_combs[2, ], all_combs[1, ])
}

#' @export
#' @rdname contrast
against_ref <- function(levels, ref = levels[1], x = tail(levels, -1)) {
  assert(
    check_string(ref),
    check_int(ref)
  )
  custom_contrast(levels, x, rep(ref, length(x)))
}
#' Create Contrast of Pairs
#' @param x (`vector`) A vector of treatment levels.
#' @param y (`vector`) A vector of treatment levels.
#' @param ref (`string` or `int`) Reference level.
#' @param levels (`character`) Levels of the treatment.
#' @export
#' @rdname contrast
#' @return A list of `contrast` object with following elements:
#' - Index of the treatment group.
#' - Index of the reference group.
#' Additional attributes include `levels` and `max_levels` indicating the
#' names of the treatment levels and the maximum number of levels.
custom_contrast <- function(levels, x, y) {
  assert_character(levels)
  if (test_integerish(x)) {
    assert_integerish(x)
  } else {
    assert_character(x)
    assert_subset(x, levels)
  }
  if (test_integerish(y)) {
    assert_integerish(y, len = length(x))
  } else {
    assert_character(y, len = length(x))
    assert_subset(y, levels)
  }
  structure(
    list(
      if (test_integerish(x)) x else match(x, levels),
      if (test_integerish(y)) y else match(y, levels)
    ),
    max_levels = length(levels),
    levels = levels,
    class = "contrast"
  )
}

#' Update levels in a contrast pair
#' @keywords internal
update_levels <- function(pair, levels) {
  assert_class(pair, "contrast")
  assert_character(levels)
  neworder <- match(attr(pair, "levels"), levels)
  custom_contrast(
    levels,
    neworder[pair[[1]]],
    neworder[pair[[2]]]
  )
}

#' Obtain the Jacobian matrix
#' @keywords internal
jac_mat <- function(jac, pair) {
  assert_matrix(jac, ncols = 2, nrows = length(pair[[1]]))
  assert_class(pair, "contrast")
  ret <- matrix(0, nrow = nrow(jac), ncol = attr(pair, "max_levels"))
  ret[cbind(seq_len(nrow(jac)), pair[[1]])] <- jac[, 1]
  ret[cbind(seq_len(nrow(jac)), pair[[2]])] <- jac[, 2]
  ret
}

#' Sum vectors in a list
#' @keywords internal
sum_vectors_in_list <- function(lst) {
  assert_list(lst, min.len = 1L, types = "numeric")
  len1 <- length(lst[[1L]])
  lapply(lst, assert_numeric, any.missing = FALSE, len = len1)
  tmp <- matrix(
    unlist(lst, recursive = FALSE, use.names = FALSE),
    nrow = len1,
    ncol = length(lst)
  )
  .rowSums(tmp, m = len1, n = length(lst))
}

#' Confidence interval calculations which are common across effect results.
#' @keywords internal
h_confint <- function(x, parm, level = 0.95, transform, include_se = FALSE, ...) {
  assert_matrix(x)
  assert_names(colnames(x), must.include = c("Estimate", "Std.Err"))
  assert_names(rownames(x), type = "unique")
  assert_number(level, lower = 0, upper = 1)
  assert_flag(include_se)
  if (!missing(parm)) {
    assert(
      check_integerish(parm, lower = 1, upper = nrow(x)),
      check_subset(parm, row.names(x))
    )
  }
  if (!missing(transform)) {
    assert_function(transform)
  }
  est <- x[, "Estimate"]
  se <- x[, "Std.Err"]
  z <- qnorm((1 + level) / 2)
  ret <- matrix(
    c(
      est,
      if (include_se) se else NULL,
      est - se * z,
      est + se * z
    ),
    nrow = nrow(x)
  )
  colnames(ret) <- c(
    "Estimate",
    if (include_se) "Std.Err" else NULL,
    sprintf("%s %%", c((1 - level) / 2, (1 + level) / 2) * 100)
  )
  rownames(ret) <- rownames(x)
  if (missing(transform)) {
    transform <- identity
  }
  if (!identical(transform, identity)) {
    message("The confidence interval is transformed.")
  }
  if (!missing(parm)) {
    ret <- ret[parm, , drop = FALSE]
  }
  transform(ret)
}

#' Check Whether First Factor is Nested in Second Factor
#'
#' This function checks whether one factor variable is nested in another in terms of their levels
#' and the mapping of observations to these levels:
#'
#' - Both factors must have the same length.
#' - The positions of `NA` values must be identical in both factors.
#' - After removing `NA` values, `f1` must have at least as many observed levels as `f2`.
#' - The mapping of observations to levels must be consistent, meaning that each level in `f1`
#'   corresponds to exactly one level in `f2`. On the other hand, multiple levels in `f1`
#'   can map to the same level in `f2`.
#'
#' @param f1 A factor variable, supposed to be more fine grained than `f2`.
#' @param f2 A factor variable, supposed to be more coarse grained than `f1`.
#' @return `TRUE` if `f1` is nested in `f2`, `FALSE` otherwise.
#'
#' @keywords internal
h_first_fct_nested_in_second <- function(f1, f2) {
  assert_factor(f1)
  assert_factor(f2)

  if (length(f1) != length(f2)) {
    return(FALSE)
  }

  f1_na <- which(is.na(f1))
  f2_na <- which(is.na(f2))
  if (!identical(f1_na, f2_na)) {
    return(FALSE)
  }

  f1 <- droplevels(stats::na.omit(f1))
  f2 <- droplevels(stats::na.omit(f2))
  if (nlevels(f1) < nlevels(f2)) {
    return(FALSE)
  }

  f1_int <- as.integer(f1)
  f2_int <- as.integer(f2)
  f1_f2_int <- paste(f1_int, f2_int, sep = "-")
  unique_combinations <- unique(f1_f2_int)
  length(unique_combinations) == nlevels(f1)
}

test_that("h_get_vars works for formula", {
  res <- expect_silent(h_get_vars(abc ~ 1))
  expect_identical(res, list(treatment = "abc", schema = "sr", strata = character(0)))
  expect_identical(res, list(treatment = "abc", schema = "sr", strata = character(0)))

  expect_error(
    h_get_vars("treatment"),
    "Must be a formula, not character"
  )
  expect_error(
    h_get_vars(trt ~ pb(s1) + ps(s2)),
    "only one randomization schema is allowed!"
  )
  expect_error(
    h_get_vars(~ ps(s2)),
    "treatment formula must be of type treatment ~ strata"
  )
  expect_error(
    h_get_vars(NULL),
    "Must be a formula, not 'NULL'"
  )
  expect_error(
    h_get_vars(log(a) ~ b),
    "left hand side of the treatment formula should be a single name!"
  )
  expect_error(
    h_get_vars(a + b ~ strata(b)),
    "left hand side of the treatment formula should be a single name!"
  )
})

test_that("h_get_vars works for formula with schemas", {
  res <- expect_silent(h_get_vars(a ~ b + c))
  expect_identical(res, list(treatment = "a", schema = "sr", strata = c("b", "c")))

  res <- expect_silent(h_get_vars(a ~ pb(1) + b))
  expect_identical(res, list(treatment = "a", schema = "pb", strata = "b"))

  res <- expect_silent(h_get_vars(a ~ ps(b) + c))
  expect_identical(res, list(treatment = "a", schema = "ps", strata = c("b", "c")))

  res <- expect_silent(h_get_vars(a ~ strata(b)))
  expect_identical(res, list(treatment = "a", schema = "sr", strata = "b"))
})

test_that("h_get_vars is backwards compatible for use of sp instead of sr", {
  res <- expect_silent(h_get_vars(a ~ sp(b) + c))
  expect_identical(res, list(treatment = "a", schema = "sr", strata = c("b", "c")))
})

test_that("h_prep_survival_input works with strata", {
  result <- expect_silent(h_prep_survival_input(
    formula = survival::Surv(time, status) ~ age +
      ph.karno +
      meal.cal +
      strata(strata) +
      interaction(I(age > 50), I(ph.karno < 20)),
    data = surv_data,
    treatment = sex ~ pb(strata)
  ))
  expected <- list(
    data = surv_data,
    time = "time",
    status = "status",
    treatment = "sex",
    randomization_strata = "strata",
    strata = "strata",
    schema = "pb",
    covariates = c("age", "ph.karno", "meal.cal"),
    model = ~ age + ph.karno + meal.cal + interaction(I(age > 50), I(ph.karno < 20)),
    n_levels = 2L,
    levels = c("Female", "Male")
  )
  expect_equal(result, expected, ignore_formula_env = TRUE)
})

test_that("h_prep_survival_input works with multiple strata", {
  result <- expect_silent(h_prep_survival_input(
    formula = survival::Surv(time, status) ~ age + ph.karno + meal.cal + strata(strata, ecog),
    data = surv_data,
    treatment = sex ~ sr(1) # We do not necessarily need to have the strata in the randomization.
  ))
  expected <- list(
    data = surv_data,
    time = "time",
    status = "status",
    treatment = "sex",
    randomization_strata = character(),
    strata = c("strata", "ecog"),
    schema = "sr",
    covariates = c("age", "ph.karno", "meal.cal"),
    model = ~ age + ph.karno + meal.cal,
    n_levels = 2L,
    levels = c("Female", "Male")
  )
  expect_equal(result, expected, ignore_formula_env = TRUE)
})

test_that("h_prep_survival_input works with multiple strata specified in separate strata() terms", {
  result <- expect_silent(h_prep_survival_input(
    formula = survival::Surv(time, status) ~ age + ph.karno + meal.cal + strata(strata) + strata(ecog),
    data = surv_data,
    treatment = sex ~ sr(1) # We do not necessarily need to have the strata in the randomization.
  ))
  expected <- list(
    data = surv_data,
    time = "time",
    status = "status",
    treatment = "sex",
    randomization_strata = character(),
    strata = c("strata", "ecog"),
    schema = "sr",
    covariates = c("age", "ph.karno", "meal.cal"),
    model = ~ age + ph.karno + meal.cal,
    n_levels = 2L,
    levels = c("Female", "Male")
  )
  expect_equal(result, expected, ignore_formula_env = TRUE)
})

test_that("h_prep_survival_input works without strata", {
  result <- expect_silent(h_prep_survival_input(
    formula = survival::Surv(time, status) ~ age + ph.karno + meal.cal,
    data = surv_data,
    # We specify strata for the randomization, but there are none in the analysis formula.
    treatment = sex ~ pb(strata)
  ))
  expected <- list(
    data = surv_data,
    time = "time",
    status = "status",
    treatment = "sex",
    randomization_strata = "strata",
    strata = character(),
    schema = "pb",
    covariates = c("age", "ph.karno", "meal.cal"),
    model = ~ age + ph.karno + meal.cal,
    n_levels = 2L,
    levels = c("Female", "Male")
  )
  expect_equal(result, expected, ignore_formula_env = TRUE)
})

test_that("h_prep_survival_input works without covariates", {
  result <- expect_silent(h_prep_survival_input(
    formula = survival::Surv(time, status) ~ 1,
    data = surv_data,
    treatment = sex ~ pb(strata)
  ))
  expected <- list(
    data = surv_data,
    time = "time",
    status = "status",
    treatment = "sex",
    randomization_strata = "strata",
    strata = character(),
    schema = "pb",
    covariates = character(),
    model = ~1,
    n_levels = 2L,
    levels = c("Female", "Male")
  )
  expect_equal(result, expected, ignore_formula_env = TRUE)
})

test_that("h_prep_survival_input works without covariates and without strata", {
  result <- expect_silent(h_prep_survival_input(
    formula = survival::Surv(time, status) ~ 1,
    data = surv_data,
    treatment = sex ~ sr(1)
  ))
  expected <- list(
    data = surv_data,
    time = "time",
    status = "status",
    treatment = "sex",
    randomization_strata = character(),
    strata = character(),
    schema = "sr",
    covariates = character(),
    model = ~1,
    n_levels = 2L,
    levels = c("Female", "Male")
  )
  expect_equal(result, expected, ignore_formula_env = TRUE)
})

test_that("h_n_events_per_time works as expected", {
  result <- expect_silent(h_n_events_per_time(
    surv_data,
    time = "time",
    status = "status"
  ))

  expect_data_frame(result)
  expect_numeric(result$time)
  expect_true(!any(duplicated(result$time)))
  expect_true(all(result$time %in% surv_data$time))
  expect_integer(result$n_events)
  expect_true(sum(result$n_events) == sum(surv_data$status))
})

test_that("h_n_events_per_time works when there are no events", {
  surv_dat_no_events <- surv_data
  surv_dat_no_events$status <- 0

  result <- expect_silent(h_n_events_per_time(
    surv_dat_no_events,
    time = "time",
    status = "status"
  ))

  expect_data_frame(result, nrows = 0L)
  expect_numeric(result$time)
  expect_integer(result$n_events)
})

test_that("sum_vectors_in_list works as expected", {
  lst <- list(a = 1:3, b = 4:6, c = 7:9)
  result <- sum_vectors_in_list(lst)
  expected <- c(12, 15, 18)
  expect_equal(result, expected)
})

test_that("h_first_fct_nested_in_second works as expected without NAs", {
  f1 <- factor(c("A", "B", "C"))
  f2 <- factor(c("C", "A", "B"))
  f3 <- factor(c("A", "B", "B"))
  f4 <- factor(c("A", "B", "C", "D"))

  expect_true(h_first_fct_nested_in_second(f1, f2))
  expect_true(h_first_fct_nested_in_second(f2, f1))
  expect_true(h_first_fct_nested_in_second(f1, f3))
  expect_false(h_first_fct_nested_in_second(f3, f1))
  expect_false(h_first_fct_nested_in_second(f1, f4))
})

test_that("h_first_fct_nested_in_second works as expected with NAs", {
  f1 <- factor(c("A", "B", "C", NA))
  f2 <- factor(c("C", "A", "B", NA))
  f3 <- factor(c(NA, "B", "C", "D"))
  f4 <- factor(c("A", "B", "C", "D", NA))
  f5 <- factor(c("E", "E", "F", "F", NA))

  expect_true(h_first_fct_nested_in_second(f1, f2))
  expect_true(h_first_fct_nested_in_second(f2, f1))
  expect_false(h_first_fct_nested_in_second(f1, f3))
  expect_false(h_first_fct_nested_in_second(f1, f4))
  expect_true(h_first_fct_nested_in_second(f4, f5))
  expect_false(h_first_fct_nested_in_second(f5, f4))
})

# Other tests which don't work within the testthat framework.
library(RobinCar2)

# test: "h_prep_survival_input works with survival::Surv objects"
surv_obj <- with(surv_data, survival::Surv(time, status))
# This works with coxph e.g.:
example <- survival::coxph(surv_obj ~ sex, data = surv_data)
result <- RobinCar2:::h_prep_survival_input(
  formula = surv_obj ~ 1,
  # We have another restriction here to avoid ambiguity, therefore
  # need to remove the time and status columns from the data.
  data = subset(surv_data, select = -c(time, status)),
  treatment = sex ~ sr(1)
)
expected <- list(
  data = cbind(
    subset(surv_data, select = -c(time, status)),
    subset(surv_data, select = c(time, status))
  ), # The two removed columns have been added back.
  time = "time",
  status = "status",
  treatment = "sex",
  randomization_strata = character(),
  strata = character(),
  schema = "sr",
  covariates = character(),
  model = ~1,
  n_levels = 2L,
  levels = c("Female", "Male")
)
testthat::expect_equal(result, expected, ignore_formula_env = TRUE)

#' Log-Rank Score Functions for Survival Analysis
#'
#' These functions compute the log-rank score statistics for a survival analysis.
#' Depending on the function, these are stratified and/or adjusted for covariates.
#'
#' @param theta (`number`) The assumed log hazard ratio of the second vs. the first level of the
#'   treatment arm variable.
#' @param df (`data.frame`) The data frame containing the survival data.
#' @param treatment (`string`) The name of the treatment arm variable in `df`. It should be a factor with
#'   two levels, where the first level is the reference group.
#' @param time (`string`) The name of the time variable in `df`, representing the survival time.
#' @param status (`string`) The name of the status variable in `df`, with 0 for censored and 1 for event.
#' @param strata (`character`) The names of the strata variables in `df`, which must be factors.
#' @param randomization_strata (`character`) The names of the randomization strata variables in `df`.
#'   These are used to check whether the means of the covariate adjustment residuals are unbiased across
#'   these strata.
#' @param model (`formula`) The model formula for covariate adjustment, e.g., `~ cov1 + cov2`.
#' @param theta_hat (`number`) The estimated log hazard ratio when not adjusting for covariates.
#' @param n (`count`) The number of observations. Note that this can be higher than the number of rows
#'   when used in stratified analyses computations.
#' @param use_ties_factor (`flag`) Whether to use the ties factor in the variance calculation. This is used
#'   when calculating the score test statistic, but not when estimating the log hazard ratio.
#' @param calculate_variance (`flag`) Whether to calculate the variance. This is useful to avoid
#'   unnecessary computations when only the score function value is needed, e.g., during root finding.
#' @param hr_se_plugin_adjusted (`flag`) Defines the method for calculating the standard error of the
#'   log hazard ratio estimate when adjusting for covariates, see details.
#' @return The score function value(s), with the following attributes:
#'   - `sigma_l2`: The variance of the log-rank statistic.
#'   - `se_theta_l`: The corresponding standard error term for the log hazard ratio.
#'   - `n`: The number of observations used in the calculation.
#'
#' @details
#'   - The `hr_se_plugin_adjusted` flag is relevant only for the standard error of the covariate adjusted
#'   log hazard ratio estimate: When `TRUE`, the adjusted hazard ratio estimate
#'   is plugged in into the variance formula, as per the original publication.
#'   On the other hand, when `FALSE`, the unadjusted estimate is used instead.  This is explained in more
#'   detail in the vignette "Survival Analysis with RobinCar2" in Section "Covariate adjusted analysis without strata".
#'
#'   - Note that for the not covariate adjusted score functions, these also work
#'   with a `numeric` `theta` vector of length > 1.
#' @name survival_score_functions
#' @keywords internal
NULL

#' @describeIn survival_score_functions without strata or covariates.
h_lr_score_no_strata_no_cov <- function(
  theta,
  df,
  treatment,
  time,
  status,
  randomization_strata = character(),
  n = nrow(df),
  use_ties_factor = TRUE,
  calculate_variance = TRUE,
  check_rand_strat_warning = FALSE
) {
  assert_numeric(theta, min.len = 1L, finite = TRUE)
  assert_data_frame(df)
  assert_string(treatment)
  assert_string(time)
  assert_string(status)
  assert_factor(df[[treatment]], n.levels = 2L, any.missing = FALSE)
  assert_numeric(df[[status]], any.missing = FALSE)
  assert_true(all(df[[status]] %in% c(0, 1)))
  assert_numeric(df[[time]], lower = 0, any.missing = FALSE)
  assert_character(randomization_strata)
  assert_count(n)
  assert_true(n >= nrow(df))
  assert_flag(use_ties_factor)
  assert_flag(calculate_variance)
  assert_flag(check_rand_strat_warning)

  # Check whether to warn about insufficient inclusion of randomization strata.
  give_rand_strat_warning <- (check_rand_strat_warning && length(randomization_strata) > 0)

  # Standardize data set format, subset to relevant variables.
  df_stand <- data.frame(
    treatment_numeric = as.numeric(df[[treatment]]) - 1L,
    time = df[[time]],
    status = df[[status]]
  )

  # Sort by time.
  df_stand <- df_stand[order(df_stand$time), , drop = FALSE]

  # Patients with events.
  df_events <- df_stand[df_stand$status == 1L, , drop = FALSE]

  # Add number of events per unique event time.
  df_events_per_time <- h_n_events_per_time(df_stand, time = "time", status = "status")
  df_events$n_events <- df_events_per_time$n_events[match(df_events$time, df_events_per_time$time)]

  # Calculate the log rank statistic u_l and the variance sigma_l2 iteratively.
  u_l <- 0
  if (calculate_variance) {
    sigma_l2 <- 0
  }
  for (i in seq_len(nrow(df_events))) {
    # This event time.
    t_i <- df_events$time[i]

    # Number of overall events at this time.
    n_events_ti <- df_events$n_events[i]

    # This treatment arm indicator.
    trt_i <- df_events$treatment_numeric[i]

    # Proportions of patients at risk at time t_i, per arm.

    # Note: Also here we need to use sum divided by n, otherwise
    # we will get wrong results when there are ties.
    y_bar_1_ti <- sum(df_stand$treatment_numeric & df_stand$time >= t_i) / n
    y_bar_0_ti <- sum(!df_stand$treatment_numeric & df_stand$time >= t_i) / n

    # Adjusted proportion of patients at risk in the treatment arm:
    y_bar_1_ti_coxph <- y_bar_1_ti * exp(theta)

    # Increment u_l.
    u_l <- u_l + (trt_i - y_bar_1_ti_coxph / (y_bar_1_ti_coxph + y_bar_0_ti))

    if (calculate_variance) {
      # Increment sigma_l2.
      y_bar_all_ti <- y_bar_1_ti_coxph + y_bar_0_ti

      sigma_l2_factor_ti <- if (!use_ties_factor || n_events_ti == 1) {
        1
      } else {
        (n * y_bar_all_ti - n_events_ti) / (n * y_bar_all_ti - 1)
      }
      sigma_l2_summand <- y_bar_1_ti_coxph * y_bar_0_ti * sigma_l2_factor_ti / y_bar_all_ti^2
      sigma_l2 <- sigma_l2 + sigma_l2_summand
    }
  }
  u_l <- u_l / n
  sigma_l2 <- if (calculate_variance) sigma_l2 / n else NA_real_
  se_theta_l <- if (calculate_variance) sqrt(1 / (n * sigma_l2)) else NA_real_
  structure(
    u_l,
    sigma_l2 = sigma_l2,
    se_theta_l = se_theta_l,
    n = n,
    give_rand_strat_warning = give_rand_strat_warning
  )
}

#' @describeIn survival_score_functions with strata but without covariates.
h_lr_score_strat <- function(
  theta,
  df,
  treatment,
  time,
  status,
  strata,
  randomization_strata = character(),
  use_ties_factor = TRUE,
  calculate_variance = TRUE,
  check_rand_strat_warning = FALSE
) {
  assert_string(treatment)
  assert_string(time)
  assert_string(status)
  assert_character(strata, any.missing = FALSE, min.len = 1L, unique = TRUE)
  assert_data_frame(df)
  lapply(df[strata], assert_factor)
  lapply(df[randomization_strata], assert_factor)
  assert_flag(calculate_variance)
  assert_flag(check_rand_strat_warning)

  give_rand_strat_warning <- if (check_rand_strat_warning) {
    length(randomization_strata) > 0 &&
      !all(randomization_strata %in% strata) &&
      !h_first_fct_nested_in_second(interaction(df[strata]), interaction(df[randomization_strata]))
  } else {
    FALSE
  }

  df <- stats::na.omit(df[, c(treatment, time, status, strata)])
  n <- nrow(df)

  strata_formula <- paste("~", paste(strata, collapse = "+"))
  df_split <- split(df, f = as.formula(strata_formula), drop = TRUE)

  strata_results <- lapply(
    df_split,
    FUN = h_lr_score_no_strata_no_cov,
    theta = theta,
    treatment = treatment,
    time = time,
    status = status,
    n = n,
    randomization_strata = randomization_strata,
    use_ties_factor = use_ties_factor,
    calculate_variance = calculate_variance,
    check_rand_strat_warning = FALSE
  )

  u_sl <- sum_vectors_in_list(strata_results)
  sigma_sl2 <- if (calculate_variance) sum_vectors_in_list(lapply(strata_results, attr, "sigma_l2")) else NA_real_
  se_theta_sl <- if (calculate_variance) sqrt(1 / (n * sigma_sl2)) else NA_real_
  structure(
    u_sl,
    sigma_l2 = sigma_sl2,
    se_theta_l = se_theta_sl,
    n = n,
    give_rand_strat_warning = give_rand_strat_warning
  )
}

#' @describeIn survival_score_functions with covariates but without strata.
h_lr_score_cov <- function(
  theta,
  df,
  treatment,
  time,
  status,
  model,
  randomization_strata = character(),
  theta_hat = theta,
  use_ties_factor = TRUE,
  hr_se_plugin_adjusted = TRUE,
  calculate_variance = TRUE,
  check_rand_strat_warning = FALSE
) {
  assert_data_frame(df)
  assert_string(treatment)
  assert_string(time)
  assert_string(status)
  assert_formula(model)
  assert_character(randomization_strata)
  covariates <- all.vars(model)
  assert_subset(c(treatment, time, status, covariates), names(df))
  assert_factor(df[[treatment]], n.levels = 2L, any.missing = FALSE)
  assert_flag(hr_se_plugin_adjusted)
  assert_flag(calculate_variance)
  assert_flag(check_rand_strat_warning)

  # Subset to complete records here.
  required_cols <- unique(c(treatment, time, status, covariates, randomization_strata))
  df <- stats::na.omit(df[required_cols])
  n <- nrow(df)

  # Calculate derived outcomes and regress them on covariates based on theta_hat.
  df_with_covs_ovals <- h_derived_outcome_vals(
    theta = theta_hat,
    df = df,
    treatment = treatment,
    time = time,
    status = status,
    covariates = covariates,
    randomization_strata = randomization_strata,
    n = n
  )
  lm_input <- h_get_lm_input(df = df_with_covs_ovals, model = model)
  lm_results <- h_get_lm_results(lm_input)
  beta_est <- lm_results$beta_est

  give_rand_strat_warning <- if (check_rand_strat_warning) {
    length(randomization_strata) > 0 &&
      !h_unbiased_means_across_strata(
        residuals_per_group = lm_results$residuals,
        df = df_with_covs_ovals,
        randomization_strata = randomization_strata
      )
  } else {
    FALSE
  }

  # Obtain unadjusted result.
  unadj_score <- h_lr_score_no_strata_no_cov(
    theta = theta,
    df = df,
    treatment = treatment,
    time = time,
    status = status,
    n = n,
    randomization_strata = randomization_strata,
    use_ties_factor = use_ties_factor,
    calculate_variance = calculate_variance,
    check_rand_strat_warning = FALSE
  )

  # We assume here that the observed proportion of treatment 1 in the data set corresponds to the preplanned
  # proportion of treatment 1 in the trial.
  pi <- mean(as.numeric(df[[treatment]]) - 1)

  # Overall column wise average of design matrices.
  x_all <- rbind(lm_input[[1]]$X, lm_input[[2]]$X)
  x_bar <- colMeans(x_all)

  # Center the design matrices with this overall average.
  x_0 <- scale(lm_input[[1]]$X, center = x_bar, scale = FALSE)
  x_1 <- scale(lm_input[[2]]$X, center = x_bar, scale = FALSE)

  # Compute adjustment term for the score.
  u_l_adj_term <- (sum(x_1 %*% beta_est[[2]]) - sum(x_0 %*% beta_est[[1]])) / n

  # Compute adjusted score statistic.
  u_cl <- as.numeric(unadj_score) - u_l_adj_term

  if (calculate_variance) {
    # Define standard error calculation.
    g_theta_cl <- if (hr_se_plugin_adjusted) {
      attr(unadj_score, "sigma_l2")
    } else {
      unadj_score_theta_hat <- h_lr_score_no_strata_no_cov(
        theta = theta_hat, # Here is the only difference.
        df = df,
        treatment = treatment,
        time = time,
        status = status,
        n = n,
        randomization_strata = randomization_strata,
        use_ties_factor = use_ties_factor,
        calculate_variance = TRUE,
        check_rand_strat_warning = FALSE
      )
      attr(unadj_score_theta_hat, "sigma_l2")
    }

    # Compute adjustment term for the variance estimate.
    cov_x <- stats::cov(x_all)
    beta_est_sum <- beta_est[[1]] + beta_est[[2]]
    sigma_l2_adj_term <- pi * (1 - pi) * as.numeric(t(beta_est_sum) %*% cov_x %*% beta_est_sum)

    # Compute standard error for theta estimate, based on above results incl. choice for `g_theta_cl`.
    sigma_cl2 <- g_theta_cl - sigma_l2_adj_term
    var_theta_cl <- sigma_cl2 / (g_theta_cl^2) / n
    se_theta_cl <- sqrt(var_theta_cl)
  }

  structure(
    u_cl,
    se_theta_l = if (calculate_variance) se_theta_cl else NA_real_,
    sigma_l2 = if (calculate_variance) sigma_cl2 else NA_real_,
    n = n,
    give_rand_strat_warning = give_rand_strat_warning
  )
}

#' @describeIn survival_score_functions with strata and covariates.
h_lr_score_strat_cov <- function(
  theta,
  df,
  treatment,
  time,
  status,
  strata,
  model,
  randomization_strata = character(),
  theta_hat = theta,
  use_ties_factor = TRUE,
  hr_se_plugin_adjusted = TRUE,
  calculate_variance = TRUE,
  check_rand_strat_warning = FALSE
) {
  assert_data_frame(df)
  assert_string(treatment)
  assert_string(time)
  assert_string(status)
  assert_character(strata, any.missing = FALSE, min.len = 1L, unique = TRUE)
  assert_formula(model)
  assert_character(randomization_strata)
  covariates <- all.vars(model)
  assert_subset(c(treatment, time, status, strata, covariates), names(df))
  assert_flag(hr_se_plugin_adjusted)
  assert_flag(calculate_variance)
  assert_flag(check_rand_strat_warning)

  # Subset to complete records here.
  required_cols <- unique(c(treatment, time, status, covariates, strata, randomization_strata))
  df <- stats::na.omit(df[required_cols])
  n <- nrow(df)

  # Ensure that all character covariates are converted to factors.
  # (Otherwise we could just have one character value in one stratum, or incompatible
  # design matrices between the strata.)
  for (cov in covariates) {
    if (is.character(df[[cov]])) {
      df[[cov]] <- factor(df[[cov]])
    }
  }

  # Calculate derived outcomes and regress them on covariates.
  df_with_covs_ovals_stratum <- h_strat_derived_outcome_vals(
    theta = theta_hat,
    df,
    treatment,
    time,
    status,
    strata,
    covariates = covariates,
    randomization_strata = randomization_strata
  )
  strat_lm_input <- h_get_strat_lm_input(df_with_covs_ovals_stratum, model)
  lm_results <- h_get_strat_lm_results(strat_lm_input)
  beta_est <- lm_results$beta_est

  give_rand_strat_warning <- if (check_rand_strat_warning) {
    length(randomization_strata) > 0 &&
      !h_unbiased_means_across_strata(
        residuals_per_group = lm_results$residuals,
        df = df_with_covs_ovals_stratum,
        randomization_strata = randomization_strata
      )
  } else {
    FALSE
  }

  # Obtain unadjusted results.
  strat_unadj_score <- h_lr_score_strat(
    theta,
    df,
    treatment,
    time,
    status,
    strata,
    randomization_strata,
    use_ties_factor = use_ties_factor,
    calculate_variance = calculate_variance,
    check_rand_strat_warning = FALSE
  )

  # We assume here that the observed proportion of treatment 1 in the data set
  # corresponds to the preplanned proportion of treatment 1 in the trial.
  pi <- mean(as.numeric(df[[treatment]]) - 1)

  groups <- levels(df[[treatment]])
  cont_grp <- groups[1]
  trt_grp <- groups[2]

  # Overall column wise average of design matrices, separately for each stratum.
  x_0 <- strat_lm_input[[cont_grp]]$X
  x_1 <- strat_lm_input[[trt_grp]]$X
  x_all <- rbind(x_0, x_1)

  stratum_col <- match(".stratum", colnames(x_all))
  strat_x_all <- split.data.frame(x_all[, -stratum_col, drop = FALSE], f = x_all[, stratum_col])
  strat_x_bar <- lapply(strat_x_all, colMeans)
  strat_x_bar_matrix <- do.call(rbind, strat_x_bar)
  strat_x_bar_stratum <- as.integer(names(strat_x_all))

  # Center the design matrices with this overall average.
  which_x_bar_for_x_0 <- match(unname(x_0[, stratum_col]), strat_x_bar_stratum)
  x_0 <- x_0[, -stratum_col, drop = FALSE] - strat_x_bar_matrix[which_x_bar_for_x_0, , drop = FALSE]

  which_x_bar_for_x_1 <- match(unname(x_1[, stratum_col]), strat_x_bar_stratum)
  x_1 <- x_1[, -stratum_col, drop = FALSE] - strat_x_bar_matrix[which_x_bar_for_x_1, , drop = FALSE]

  # Compute adjustment term for the stratified score.
  u_sl_adj_term <- (sum(x_1 %*% beta_est[[trt_grp]]) - sum(x_0 %*% beta_est[[cont_grp]])) / n

  # Compute adjusted covariate adjusted stratified score.
  u_csl <- as.numeric(strat_unadj_score) - u_sl_adj_term

  if (calculate_variance) {
    # Compute adjustment term for sigma_sl2.
    strat_n <- sapply(strat_x_all, nrow)
    strat_use <- names(which(strat_n > 1))
    strat_n <- strat_n[strat_use]
    overall_n <- sum(strat_n)
    strat_cov_x <- lapply(strat_x_all[strat_use], stats::cov)
    weighted_cov_x <- Map(function(x, n) x * n / overall_n, strat_cov_x, strat_n)
    weighted_sum_cov_x <- Reduce("+", weighted_cov_x)

    beta_est_sum <- beta_est[[cont_grp]] + beta_est[[trt_grp]]
    sigma_sl2_adj_term <- pi * (1 - pi) * as.numeric(t(beta_est_sum) %*% weighted_sum_cov_x %*% beta_est_sum)

    # Define standard error calculation.
    g_theta_csl <- if (hr_se_plugin_adjusted) {
      attr(strat_unadj_score, "sigma_l2")
    } else {
      strat_unadj_score_theta_hat <- h_lr_score_strat(
        theta = theta_hat, # Here is the only difference.
        df = df,
        treatment = treatment,
        time = time,
        status = status,
        strata = strata,
        randomization_strata = randomization_strata,
        use_ties_factor = use_ties_factor,
        calculate_variance = TRUE,
        check_rand_strat_warning = FALSE
      )
      attr(strat_unadj_score_theta_hat, "sigma_l2")
    }

    # Compute standard error for theta estimate.
    var_theta_csl <- (g_theta_csl - sigma_sl2_adj_term) / (g_theta_csl^2) / n
    se_theta_csl <- sqrt(var_theta_csl)
    sigma_csl2 <- g_theta_csl - sigma_sl2_adj_term
  }

  structure(
    u_csl,
    se_theta_l = if (calculate_variance) se_theta_csl else NA_real_,
    sigma_l2 = if (calculate_variance) sigma_csl2 else NA_real_,
    n = n,
    give_rand_strat_warning = give_rand_strat_warning
  )
}

test_that("h_lr_score_no_strata_no_cov works as expected with default options", {
  result <- h_lr_score_no_strata_no_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status"
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_lr_score_no_strata_no_cov works as expected with custom n", {
  result <- h_lr_score_no_strata_no_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    n = 400
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_lr_score_no_strata_no_cov works as expected when not using ties factor", {
  result <- h_lr_score_no_strata_no_cov(
    theta = 0.5,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    use_ties_factor = FALSE
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_lr_score_no_strata_no_cov works as expected with multiple theta values", {
  theta_vals <- c(0, 0.5, 1)
  result <- h_lr_score_no_strata_no_cov(
    theta = theta_vals,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    use_ties_factor = FALSE
  )
  expected <- sapply(theta_vals, function(theta) {
    h_lr_score_no_strata_no_cov(
      theta = theta,
      df = surv_data,
      treatment = "sex",
      time = "time",
      status = "status",
      use_ties_factor = FALSE
    )
  })
  expect_identical(as.numeric(result), expected)
})

test_that("h_lr_score_no_strata_no_cov works when not calculating the variance", {
  result <- h_lr_score_no_strata_no_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    calculate_variance = FALSE
  )
  expect_equal(as.numeric(result), 0.08955378, tolerance = 1e-6)
  expect_scalar_na(attr(result, "sigma_l2"))
  expect_scalar_na(attr(result, "se_theta_l"))
  expect_equal(attr(result, "n"), nrow(surv_data))
})

test_that("h_lr_score_strat works as expected with default options", {
  result <- h_lr_score_strat(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    strata = "strata"
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_lr_score_strat works with multiple strata variables", {
  result <- h_lr_score_strat(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    strata = c("strata", "ecog")
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_lr_score_strat works as expected when not using ties factor", {
  result <- h_lr_score_strat(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    strata = "strata",
    use_ties_factor = FALSE
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_lr_score_strat works as expected when not calculating the variance", {
  result <- h_lr_score_strat(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    strata = "strata",
    use_ties_factor = FALSE,
    calculate_variance = FALSE
  )
  expect_equal(as.numeric(result), 0.08968712, tolerance = 1e-6)
  expect_scalar_na(attr(result, "sigma_l2"))
  expect_scalar_na(attr(result, "se_theta_l"))
  expect_equal(attr(result, "n"), 227)
})

test_that("h_lr_score_strat works as expected with multiple theta values", {
  theta_vals <- c(0, 0.5, 1)
  result <- h_lr_score_strat(
    theta = theta_vals,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    strata = "strata",
    use_ties_factor = FALSE
  )
  expected <- sapply(theta_vals, function(theta) {
    h_lr_score_strat(
      theta = theta,
      df = surv_data,
      treatment = "sex",
      time = "time",
      status = "status",
      strata = "strata",
      use_ties_factor = FALSE
    )
  })
  expect_identical(as.numeric(result), expected)
})

test_that("h_lr_score_cov works as expected with default options", {
  result <- h_lr_score_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    model = ~age
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_lr_score_cov works as expected when not using ties factor", {
  result <- h_lr_score_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    model = ~age,
    use_ties_factor = FALSE
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_lr_score_cov works as expected when not calculating the variance", {
  result <- h_lr_score_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    model = ~age,
    use_ties_factor = FALSE,
    calculate_variance = FALSE
  )
  expect_equal(as.numeric(result), 0.08512662, tolerance = 1e-6)
  expect_scalar_na(attr(result, "sigma_l2"))
  expect_scalar_na(attr(result, "se_theta_l"))
  expect_equal(attr(result, "n"), 228)
})

test_that("h_lr_score_cov works when using the unadjusted standard error option", {
  result <- h_lr_score_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    model = ~age,
    hr_se_plugin_adjusted = FALSE,
    use_ties_factor = FALSE,
    theta_hat = 0.2
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")

  result_default <- h_lr_score_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    model = ~age,
    use_ties_factor = FALSE,
    theta_hat = 0.2
  )
  expect_snapshot_value(result_default, tolerance = 1e-4, style = "deparse")

  expect_equal(as.numeric(result), as.numeric(result_default))
  expect_true(
    !isTRUE(all.equal(
      attr(result, "sigma_l2"),
      attr(result_default, "sigma_l2")
    ))
  )
})

test_that("h_lr_score_cov works with a single factor covariate", {
  result <- h_lr_score_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    model = ~ecog
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_lr_score_strat_cov works as expected with default options", {
  result <- h_lr_score_strat_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    strata = "strata",
    model = ~age
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_lr_score_strat_cov works with multiple strata variables", {
  result <- h_lr_score_strat_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    strata = c("strata", "ecog"),
    model = ~age
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_lr_score_strat_cov works as expected when not using ties factor", {
  result <- h_lr_score_strat_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    strata = "strata",
    model = ~age,
    use_ties_factor = FALSE
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")
})

test_that("h_lr_score_strat_cov works as expected when not calculating the variance", {
  result <- h_lr_score_strat_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    strata = "strata",
    model = ~age,
    use_ties_factor = FALSE,
    calculate_variance = FALSE
  )
  expect_equal(as.numeric(result), 0.08975217, tolerance = 1e-6)
  expect_scalar_na(attr(result, "se_theta_l"))
  expect_scalar_na(attr(result, "sigma_l2"))
  expect_equal(attr(result, "n"), 227)
})

test_that("h_lr_score_strat_cov works as expected with unadjusted standard error", {
  result <- h_lr_score_strat_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    strata = "strata",
    model = ~age,
    use_ties_factor = FALSE,
    hr_se_plugin_adjusted = FALSE,
    theta_hat = 0.3
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")

  result_default <- h_lr_score_strat_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    strata = "strata",
    model = ~age,
    use_ties_factor = FALSE,
    hr_se_plugin_adjusted = FALSE,
    theta_hat = 0.3
  )
  expect_snapshot_value(result, tolerance = 1e-4, style = "deparse")

  result_default <- h_lr_score_strat_cov(
    theta = 0,
    df = surv_data,
    treatment = "sex",
    time = "time",
    status = "status",
    strata = "strata",
    model = ~age,
    use_ties_factor = FALSE,
    theta_hat = 0.3
  )
  expect_snapshot_value(result_default, tolerance = 1e-4, style = "deparse")

  expect_equal(as.numeric(result), as.numeric(result_default))
  expect_true(
    !isTRUE(all.equal(
      attr(result, "sigma_l2"),
      attr(result_default, "sigma_l2")
    ))
  )
})

#' Generalized Covariance (ANHECOVA)
#'
#' @param x (`prediction_cf`) Counter-factual prediction.
#' @param decompose (`flag`) whether to use decompose method to calculate the variance.
#' @param ... Not used.
#'
#' @return Named covariance matrix.
#' @export
# nolint start
vcovG <- function(x, decompose = TRUE, ...) {
  # nolint end
  assert_class(x, "prediction_cf")
  assert_flag(decompose)
  resi <- x$residual
  est <- x$estimate
  preds <- x$predictions
  var_preds <- var(preds)
  y <- x$response
  trt <- x$treatment
  pi_t <- as.numeric(table(trt) / length(trt))
  trt_lvls <- levels(trt)
  group_idx <- x$group_idx

  idx <- split(seq_len(length(trt)), trt)
  cov_ymu <- vapply(idx, function(is) stats::cov(y[is], preds[is, ]), FUN.VALUE = rep(0, ncol(preds)))

  if (decompose) {
    vcov_sr <- (vapply(idx, function(is) stats::var(y[is]), FUN.VALUE = 0) + diag(var_preds) - 2 * diag(cov_ymu)) / pi_t
  } else {
    vcov_sr <- vapply(idx, function(is) stats::var(resi[is]), FUN.VALUE = 0) / pi_t
  }

  v <- diag(vcov_sr) + cov_ymu + t(cov_ymu) - var_preds
  v <- v - h_get_erb(resi, group_idx, trt, pi_t, x$schema)
  ret <- v / length(resi)
  dimnames(ret) <- list(trt_lvls, trt_lvls)
  return(ret)
}

#' Obtain Adjustment for Proportion of Treatment Assignment
#' @keywords internal
#' @param pi (`numeric`) vector of proportions.
#' @return Numeric matrix.
h_adjust_pi <- function(pi) {
  assert_numeric(pi)
  diag(pi, nrow = length(pi)) - pi %*% t(pi)
}

#' Obtain Adjustment for Covariance Matrix
#' @param resi (`numeric`) vector of residuals.
#' @param group_idx (`list` of `integer`) index for each groups.
#' @param trt (`factor`) of treatment assignment.
#' @param pi (`numeric`) proportion of treatment assignment.
#' @param randomization (`string`) name of the randomization schema.
#' @keywords internal
h_get_erb <- function(resi, group_idx, trt, pi, randomization) {
  assert_list(group_idx, types = "integer")
  if (length(group_idx) == 1L) {
    return(0)
  }
  assert_string(randomization)
  if (randomization %in% c("sr", "ps")) {
    return(0)
  }
  assert_numeric(resi)
  assert_factor(trt)
  assert_numeric(pi)

  omegaz_sr <- h_adjust_pi(pi)
  n_tot <- length(resi)
  resi_per_strata <- vapply(
    group_idx,
    function(ii) {
      v <- vapply(split(resi[ii], trt[ii]), mean, FUN.VALUE = 0)
      v * sqrt(length(ii) / n_tot)
    },
    FUN.VALUE = rep(0, length(levels(trt)))
  )
  rb_z <- resi_per_strata / as.numeric(pi)
  tcrossprod(rb_z) * omegaz_sr
}

#' Prediction Bias
#' @description Obtain prediction bias within each stratum.
#' @param residual (`numeric`) residuals.
#' @param treatment (`factor`) treatment.
#' @param group_idx (`list` of `integer`) indices for each stratum group.
#'
#' @return Numeric matrix of bias in each stratum.
#' @keywords internal
bias <- function(residual, treatment, group_idx) {
  assert_numeric(residual)
  assert_factor(treatment, len = length(residual))
  assert_list(group_idx, types = "integer")
  grp <- unlist(group_idx)
  assert_integer(grp, lower = 1L, upper = max(grp))

  trt_lvls <- levels(treatment)

  d <- matrix(NA_real_, nrow = length(grp), ncol = length(trt_lvls))
  for (i in group_idx) {
    mval <- vapply(split(residual[i], treatment[i]), mean, FUN.VALUE = 0)
    d[i, ] <- matrix(mval, nrow = length(i), ncol = length(mval), byrow = TRUE)
  }
  d
}


#' Check Unbiased Means of Residuals Across Randomization Strata and Treatment Groups
#'
#' This function checks whether the means of residuals are approximately zero
#' across specified randomization strata for each of the two treatment groups.
#' It is used in [robin_surv()] downstream functions to verify the correct inclusion of randomization strata
#' variables in the analysis survival model.
#'
#' @param residuals_per_group (`list` of `numeric`) A named list of numeric vectors containing residuals for each
#'   of the two treatment groups.
#' @param df (`data.frame`) The data frame containing the `treatment` and randomization strata variables, produced by
#'   [h_derived_outcome_vals()] or [h_strat_derived_outcome_vals()].
#' @param randomization_strata (`character`) A character vector of names of the randomization strata variables
#'   in `df`.
#' @param eps (`numeric`) A small tolerance value to determine if means are close to zero.
#'
#' @return `TRUE` if the means of residuals across randomization strata are within the specified tolerance
#'   for both treatment groups, `FALSE` otherwise.
#'
#' @seealso [bias()] for the underlying bias (means of residuals) calculation.
#'
#' @keywords internal
h_unbiased_means_across_strata <- function(
  residuals_per_group,
  df,
  randomization_strata,
  eps = sqrt(.Machine$double.eps)
) {
  assert_list(residuals_per_group, types = "numeric", len = 2L)
  assert_data_frame(df)
  assert_character(randomization_strata, min.len = 1L, unique = TRUE)
  assert_disjunct("treatment", randomization_strata)
  assert_names(names(df), must.include = c("treatment", randomization_strata))

  residuals_overall <- numeric(nrow(df))
  for (group in names(residuals_per_group)) {
    group_indices <- which(df$treatment == group)
    residuals_overall[group_indices] <- residuals_per_group[[group]]
  }
  group_idx <- split(
    seq_len(nrow(df)),
    f = df[randomization_strata],
    drop = TRUE
  )
  bias_vals <- bias(
    residual = residuals_overall,
    treatment = df$treatment,
    group_idx = group_idx
  )
  all(
    abs(bias_vals) < eps,
    na.rm = TRUE # Because some strata may not have both treatment groups.
  )
}

test_that("bias works for guassian", {
  expect_snapshot_value(
    bias(residuals(fit_glm), treatment = glm_data$treatment, group_idx = list(seq_len(nrow(glm_data)))),
    style = "deparse",
    tolerance = 1e-4
  )
})

test_that("bias can return NaN when there is a single patient in a group", {
  result <- bias(
    residual = c(1.0, 2.0, 3.0),
    treatment = factor(c("A", "B", "B")),
    group_idx = list(
      c(1L, 2L),
      c(3L)
    )
  )
  expected <- matrix(
    c(1, 1, NaN, 2, 2, 3),
    nrow = 3,
    ncol = 2
  )
  expect_equal(result, expected)
})

test_that("h_unbiased_means_across_strata works for zero residuals", {
  n_obs_per_group <- 10
  n_obs_total <- n_obs_per_group * 2

  residuals_per_group <- list(
    group1 = rep(0, n_obs_per_group),
    group2 = rep(0, n_obs_per_group)
  )
  df <- data.frame(
    treatment = factor(rep(c("group1", "group2"), each = n_obs_per_group)),
    strat_var = factor(rep(c("A", "B"), length.out = n_obs_total))
  )
  randomization_strata <- "strat_var"
  result <- expect_silent(h_unbiased_means_across_strata(
    residuals_per_group = residuals_per_group,
    df = df,
    randomization_strata = randomization_strata
  ))
  expect_true(result)
})

test_that("h_unbiased_means_across_strata works for non-zero residuals", {
  n_obs_per_group <- 10
  n_obs_total <- n_obs_per_group * 2

  residuals_per_group <- list(
    group1 = rep(0.1, n_obs_per_group),
    group2 = rep(0, n_obs_per_group)
  )
  df <- data.frame(
    treatment = factor(rep(c("group1", "group2"), each = n_obs_per_group)),
    strat_var = factor(rep(c("A", "B"), length.out = n_obs_total))
  )
  randomization_strata <- "strat_var"
  result <- expect_silent(h_unbiased_means_across_strata(
    residuals_per_group = residuals_per_group,
    df = df,
    randomization_strata = randomization_strata
  ))
  expect_false(result)
})

test_that("h_unbiased_means_across_strata works if one stratum just has one patient", {
  n_obs_per_group <- 10
  n_obs_total <- n_obs_per_group * 2

  residuals_per_group <- list(
    group1 = rep(0.1, n_obs_per_group),
    group2 = rep(0, n_obs_per_group)
  )
  df <- data.frame(
    treatment = factor(rep(c("group1", "group2"), each = n_obs_per_group)),
    # Stratum C just contains a single patient.
    strat_var = factor(c(rep(c("A", "B"), length.out = n_obs_total - 1), "C"))
  )
  randomization_strata <- "strat_var"
  result <- expect_silent(h_unbiased_means_across_strata(
    residuals_per_group = residuals_per_group,
    df = df,
    randomization_strata = randomization_strata
  ))
  expect_false(result)
})

#' Example Trial Data for GLMs with Permute-Block Randomization
#'
#' This dataset contains the trial example data for GLMs with permute block randomization.
#'
#' @format A data frame with 600 rows and 7 columns:
#'   \describe{
#'     \item{id}{The ID of the patients.}
#'     \item{treatment}{The treatment assignment, "pbo", "trt1" and "trt2"}
#'     \item{s1}{The first stratification variable, "a" and "b".}
#'     \item{s2}{The second stratification variable, "c" and "d".}
#'     \item{covar}{The covariate following normal distribution.}
#'     \item{y}{The continuous response.}
#'     \item{y_b}{The binary response.}
#'   }
#'
#' @source The data is generated by the `create_glm_data.R` script.
#'
#' @keywords dataset
"glm_data"

#' Survival Example Data
#'
#' This dataset contains survival data from the `survival`
#' package's [survival::lung] dataset, modified to include factors for
#' `sex` and `strata`, as well as a binary `status` variable which is 1
#' for death and 0 for censored.
#'
#' @source The data is generated by the `create_surv_data.R` script.
#'
#' @keywords dataset
"surv_data"

#' Treatment Effect
#' @description Obtain treatment effect and variance from counter-factual prediction
#'
#' @param object Object from which to obtain treatment effect.
#' @param pair (`contrast`) Contrast choices.
#' @param eff_measure (`function`) Treatment effect measurement function.
#' @param eff_jacobian (`function`) Treatment effect jacobian function.
#' @param contrast_name (`string`) Name of the contrast.
#' @param ... Additional arguments for variance.
#' @return A list of `treatment_effect` object with following elements:
#' - `estimate`: estimate of the treatment effect.
#' - `pair`: `contrast` object indicating the pairwise treatment effect.
#' - `contrast`: name of the contrast function.
#' - `euqal_val`: the value for no treatment effect given the contrast.
#' - `variance`: the variance of the treatment effect.
#' - `jacobian`: the Jacobian matrix.
#' - `contrast_mat`: contrast summary matrix.
#'
#' @export
treatment_effect <- function(
    object,
    pair = pairwise(names(object$estimate)),
    eff_measure,
    eff_jacobian = eff_jacob(eff_measure),
    contrast_name,
    ...) {
  UseMethod("treatment_effect")
}

#' @export
treatment_effect.prediction_cf <- function(
    object,
    pair = pairwise(names(object$estimate)),
    eff_measure,
    eff_jacobian = eff_jacob(eff_measure),
    contrast_name = deparse(substitute(eff_measure)),
    ...) {
  assert_function(eff_measure)
  assert_class(pair, "contrast")
  assert_string(contrast_name)
  # make sure levels match
  pair <- update_levels(pair, names(object$estimate))
  trt_effect <- unname(eff_measure(object$estimate[pair[[1]]], object$estimate[pair[[2]]]))
  trt_jac <- eff_jacobian(object$estimate[pair[[1]]], object$estimate[pair[[2]]])
  trt_jac_mat <- jac_mat(trt_jac, pair)
  equal_val <- eff_measure(object$estimate[1], object$estimate[1])

  contrast_var <- trt_jac_mat %*% object$variance %*% t(trt_jac_mat)

  if (is.null(contrast_var)) {
    trt_sd <- rep(NA, length(object))
  } else {
    trt_sd <- sqrt(diag(contrast_var))
  }

  z_value <- as.numeric(trt_effect - equal_val) / trt_sd
  p <- 2 * pnorm(abs(z_value), lower.tail = FALSE)
  contrast_mat <- matrix(
    c(
      trt_effect,
      trt_sd,
      z_value,
      p
    ),
    nrow = length(trt_effect)
  )
  colnames(contrast_mat) <- c("Estimate", "Std.Err", "Z Value", "Pr(>|z|)")
  row.names(contrast_mat) <- sprintf("%s v.s. %s", attr(pair, "levels")[pair[[1]]], attr(pair, "levels")[pair[[2]]])
  structure(
    list(
      estimate = trt_effect,
      pair = pair,
      contrast = contrast_name,
      equal_val = equal_val,
      variance = contrast_var,
      jacobian = trt_jac_mat,
      contrast_mat = contrast_mat
    ),
    class = "treatment_effect"
  )
}


#' @export
#' @inheritParams predict_counterfactual
treatment_effect.lm <- function(
    object,
    pair,
    eff_measure,
    eff_jacobian = eff_jacob(eff_measure),
    contrast_name = deparse(substitute(eff_measure)),
    vcov = "vcovG",
    vcov_args = list(),
    treatment,
    data = find_data(object),
    ...) {
  pc <- predict_counterfactual(object, data = data, treatment, vcov = vcov, vcov_args = vcov_args)
  if (missing(pair)) {
    pair <- pairwise(names(pc$estimate))
  }
  treatment_effect(pc, pair = pair, eff_measure = eff_measure, eff_jacobian = eff_jacobian, ...)
}

#' @export
treatment_effect.glm <- function(
    object,
    pair,
    eff_measure,
    eff_jacobian = eff_jacob(eff_measure),
    contrast_name = deparse(substitute(eff_measure)),
    vcov = "vcovG",
    vcov_args = list(),
    treatment,
    data = find_data(object),
    ...) {
  pc <- predict_counterfactual(object, treatment, data, vcov = vcov, vcov_args = vcov_args)
  if (missing(pair)) {
    pair <- pairwise(names(pc$estimate))
  }
  treatment_effect(pc, pair = pair, eff_measure = eff_measure, eff_jacobian = eff_jacobian, ...)
}

#' @rdname treatment_effect
difference <- function(object, ...) {
  treatment_effect(object, eff_measure = h_diff, eff_jacobian = h_jac_diff, ...)
}
#' @rdname treatment_effect
risk_ratio <- function(object, ...) {
  treatment_effect(object, eff_measure = h_risk_ratio, eff_jacobian = h_jac_risk_ratio, ...)
}
#' @rdname treatment_effect
odds_ratio <- function(object, ...) {
  treatment_effect(object, eff_measure = h_odds_ratio, eff_jacobian = h_jac_odds_ratio, ...)
}

#' @rdname treatment_effect
log_risk_ratio <- function(object, ...) {
  treatment_effect(object, eff_measure = h_log_risk_ratio, eff_jacobian = h_jac_log_risk_ratio, ...)
}
#' @rdname treatment_effect
log_odds_ratio <- function(object, ...) {
  treatment_effect(object, eff_measure = h_log_odds_ratio, eff_jacobian = h_jac_log_odds_ratio, ...)
}

#' Contrast Functions and Jacobians
#' @rdname contrast
#' @param x (`numeric`) Vector of values.
#' @return Vector of contrasts, or matrix of jacobians.
#' @examples
#' h_diff(1:3, 4:6)
#' h_jac_risk_ratio(1:3, 4:6)
#' @export
h_diff <- function(x, y) {
  assert_numeric(x)
  assert_numeric(y, len = length(x))
  x - y
}

#' @rdname contrast
#' @export
h_jac_diff <- function(x, y) {
  assert_numeric(x)
  assert_numeric(y, len = length(x))
  n <- length(x)
  matrix(c(1, -1), nrow = n, ncol = 2, byrow = TRUE)
}

#' @rdname contrast
#' @export
h_risk_ratio <- function(x, y) {
  assert_numeric(x, lower = 0)
  assert_numeric(y, lower = 0, len = length(x))
  x / y
}

#' @rdname contrast
#' @export
h_jac_risk_ratio <- function(x, y) {
  assert_numeric(x, lower = 0)
  assert_numeric(y, lower = 0, len = length(x))
  cbind(1 / y, -x / y^2)
}

#' @rdname contrast
#' @export
h_odds_ratio <- function(x, y) {
  assert_numeric(x, lower = 0, upper = 1)
  assert_numeric(y, lower = 0, upper = 1, len = length(x))
  h_risk_ratio(x / (1 - x), y / (1 - y))
}

#' @rdname contrast
#' @export
h_jac_odds_ratio <- function(x, y) {
  assert_numeric(x, lower = 0, upper = 1)
  assert_numeric(y, lower = 0, upper = 1, len = length(x))
  cbind(1 / (1 - x)^2 / y * (1 - y), -x / (1 - x) / y^2)
}

#' @rdname contrast
#' @export
h_log_risk_ratio <- function(x, y) {
  log(h_risk_ratio(x, y))
}

#' @rdname contrast
#' @export
h_jac_log_risk_ratio <- function(x, y) {
  cbind(1 / x, -1 / y)
}

#' @rdname contrast
#' @export
h_log_odds_ratio <- function(x, y) {
  log(h_odds_ratio(x, y))
}

#' @rdname contrast
#' @export
h_jac_log_odds_ratio <- function(x, y) {
  assert_numeric(x, lower = 0, upper = 1)
  assert_numeric(y, lower = 0, upper = 1, len = length(x))
  cbind(1 / x + 1 / (1 - x), -1 / y - 1 / (1 - y))
}

#' @rdname contrast
#' @param f (`function`) Function with argument x and y to compute treatment effect.
#' @export
eff_jacob <- function(f) {
  assert_function(f, args = c("x", "y"))
  function(x, y) {
    cbind(
      numDeriv::grad(function(x) f(x = x, y = y), x),
      numDeriv::grad(function(y) f(x = x, y = y), y)
    )
  }
}

#' @export
print.treatment_effect <- function(x, ...) {
  stats::printCoefmat(
    x$contrast_mat
  )
}

#' Confidence interval function.
#' @rdname confint
#' @export
confint.treatment_effect <- function(object, parm, level = 0.95, transform, ...) {
  if (missing(transform)) {
    transform <- if (
      object$contrast %in%
        c(
          "log_odds_ratio",
          "log_risk_ratio",
          "h_log_odds_ratio",
          "h_log_risk_ratio"
        )
    ) {
      exp
    } else {
      identity
    }
  }
  h_confint(object$contrast_mat, parm = parm, level = level, transform = transform, ...)
}

test_that("h_diff works as expected", {
  x <- c(1, 3, 2)
  y <- c(2, 4, 4)
  expect_identical(
    h_diff(x, y),
    c(-1, -1, -2)
  )
  x <- c(1, 2)
  y <- c(0, 1)
  expect_identical(
    h_diff(x, y),
    c(1, 1)
  )
})

test_that("h_jac_diff work as expected", {
  x <- c(1, 3, 2)
  y <- c(2, 4, 4)
  expect_identical(
    h_jac_diff(x, y),
    matrix(c(1, -1), nrow = 3, ncol = 2, byrow = TRUE),
    tolerance = 1e-8
  )
  x <- c(1, 2)
  y <- c(0, 1)
  expect_identical(
    h_jac_diff(x, y),
    matrix(c(1, -1), nrow = 2, ncol = 2, byrow = TRUE),
    tolerance = 1e-8
  )
})

test_that("h_risk_ratio works as expected", {
  x <- c(1, 3, 2)
  y <- c(2, 3, 1)
  expect_identical(
    h_risk_ratio(x, y),
    c(1 / 2, 1, 2)
  )
  x <- c(1, 2)
  y <- c(2, 4)
  expect_identical(
    h_risk_ratio(x, y),
    c(1 / 2, 1 / 2)
  )
})

test_that("h_jac_risk_ratio work as expected", {
  x <- c(0.2, 0.5, 0.6)
  y <- c(0.4, 0.2, 0.3)
  expect_identical(
    h_jac_risk_ratio(x, y),
    matrix(c(2.5, 5, 10 / 3, -1.25, -12.5, -20 / 3), nrow = 3),
    tolerance = 1e-8
  )
  x <- c(0.2, 0.5)
  y <- c(0.5, 0.2)
  expect_identical(
    h_jac_risk_ratio(x, y),
    matrix(c(2, 5, -0.8, -12.5), nrow = 2),
    tolerance = 1e-8
  )
})

test_that("h_odds_ratio works as expected", {
  x <- c(0.5, 0.4, 0.6)
  y <- c(0.2, 0.4, 0.3)
  expect_identical(
    h_odds_ratio(x, y),
    c(4, 1, 3.5),
    tolerance = 1e-8
  )
  x <- c(0.5, 0.4)
  y <- c(0.4, 0.5)
  expect_identical(
    h_odds_ratio(x, y),
    c(1.5, 2 / 3),
    tolerance = 1e-8
  )
})

test_that("h_jac_odds_ratio work as expected", {
  x <- c(0.2, 0.5, 0.6)
  y <- c(0.5, 0.2, 0.6)
  expect_identical(
    h_jac_odds_ratio(x, y),
    matrix(c(1.5625, 16, 25 / 6, -1, -25, -25 / 6), nrow = 3),
    tolerance = 1e-8
  )
  x <- c(0.2, 0.5)
  y <- c(0.5, 0.2)
  expect_identical(
    h_jac_odds_ratio(x, y),
    matrix(c(1.5625, 16, -1, -25), nrow = 2),
    tolerance = 1e-8
  )
})


test_that("h_log_risk_ratio works as expected", {
  x <- c(1, 3, 2)
  y <- c(2, 3, 1)
  expect_identical(
    h_log_risk_ratio(x, y),
    log(c(1 / 2, 1, 2))
  )
  x <- c(1, 2)
  y <- c(2, 4)
  expect_identical(
    h_log_risk_ratio(x, y),
    log(c(1 / 2, 1 / 2))
  )
})

test_that("h_jac_risk_ratio work as expected", {
  x <- c(0.2, 0.5, 0.6)
  y <- c(0.4, 0.2, 0.3)
  expect_identical(
    h_jac_log_risk_ratio(x, y),
    matrix(c(5, 2, 5 / 3, -2.5, -5, -10 / 3), nrow = 3),
    tolerance = 1e-8
  )
  x <- c(0.2, 0.5)
  y <- c(0.5, 0.2)
  expect_identical(
    h_jac_log_risk_ratio(x, y),
    matrix(c(5, 2, -2, -5), nrow = 2),
    tolerance = 1e-8
  )
})

test_that("h_log_odds_ratio works as expected", {
  x <- c(0.5, 0.4, 0.6)
  y <- c(0.2, 0.4, 0.3)
  expect_identical(
    h_log_odds_ratio(x, y),
    log(c(4, 1, 3.5)),
    tolerance = 1e-8
  )
  x <- c(0.5, 0.4)
  y <- c(0.4, 0.5)
  expect_identical(
    h_log_odds_ratio(x, y),
    log(c(1.5, 2 / 3)),
    tolerance = 1e-8
  )
})

test_that("h_jac_log_odds_ratio work as expected", {
  x <- c(0.2, 0.5, 0.6)
  y <- c(0.5, 0.2, 0.6)
  expect_identical(
    h_jac_log_odds_ratio(x, y),
    matrix(c(6.25, 4, 25 / 6, -4, -6.25, -25 / 6), nrow = 3),
    tolerance = 1e-8
  )
  x <- c(0.2, 0.5)
  y <- c(0.5, 0.2)
  expect_identical(
    h_jac_log_odds_ratio(x, y),
    matrix(c(6.25, 4, -4, -6.25), nrow = 2),
    tolerance = 1e-8
  )
})

test_that("eff_jacob works as expected", {
  x <- c(0.2, 0.5, 0.6)
  y <- c(0.5, 0.2, 0.6)
  expect_identical(
    h_jac_diff(x, y),
    eff_jacob(h_diff)(x, y),
    tolerance = 1e-8
  )

  expect_identical(
    h_jac_risk_ratio(x, y),
    eff_jacob(h_risk_ratio)(x, y),
    tolerance = 1e-8
  )

  expect_identical(
    h_jac_odds_ratio(x, y),
    eff_jacob(h_odds_ratio)(x, y),
    tolerance = 1e-8
  )

  expect_identical(
    h_jac_log_risk_ratio(x, y),
    eff_jacob(h_log_risk_ratio)(x, y),
    tolerance = 1e-8
  )

  expect_identical(
    h_jac_log_odds_ratio(x, y),
    eff_jacob(h_log_odds_ratio)(x, y),
    tolerance = 1e-8
  )
})

test_that("treatment_effect works as expected", {
  pc <- predict_counterfactual(fit_binom, treatment ~ s1)
  df <- expect_silent(difference(pc))
  rr <- expect_silent(risk_ratio(pc))
  or <- expect_silent(odds_ratio(pc))
  expect_identical(
    df$estimate,
    c(0.22459921, 0.26528993, 0.04069073),
    ignore_attr = TRUE,
    tolerance = 1e-6
  )
  expect_identical(
    df$pair[[1]],
    c(2L, 3L, 3L)
  )
  expect_identical(
    diag(df$variance),
    c(0.002276376, 0.002259492, 0.002298305),
    tolerance = 1e-6
  )
  expect_identical(
    rr$estimate,
    c(1.630726, 1.744994, 1.070072),
    ignore_attr = TRUE,
    tolerance = 1e-6
  )
  expect_identical(
    rr$pair[[1]],
    c(2L, 3L, 3L)
  )
  expect_identical(
    diag(rr$variance),
    c(0.032539074, 0.035867956, 0.007316219),
    tolerance = 1e-6
  )
  expect_identical(
    or$estimate,
    c(2.504219, 2.967691, 1.185076),
    ignore_attr = TRUE,
    tolerance = 1e-6
  )
  expect_identical(
    diag(or$variance),
    c(0.25578808, 0.36889840, 0.05635703),
    tolerance = 1e-6
  )
})

test_that("treatment_effect works as expected for custom contrast", {
  pc <- predict_counterfactual(fit_binom, treatment ~ s1)
  h_contrast <- function(x, y) {
    (x - y)^3
  }
  expect_silent(treatment_effect(pc, eff_measure = h_contrast))
})

test_that("treatment_effect works for lm/glm object", {
  expect_snapshot(treatment_effect(fit_binom, treatment = treatment ~ s1, eff_measure = h_diff))
  expect_snapshot(treatment_effect(fit_lm, treatment = treatment ~ s1, eff_measure = h_diff, data = glm_data))
})

test_that("treatment_effect works if variance is not used", {
  expect_snapshot(treatment_effect(fit_binom, treatment = treatment ~ s1, eff_measure = h_diff, variance = NULL))
})

test_that("treatment_effect works if pair is defined", {
  expect_snapshot(
    treatment_effect(
      fit_binom,
      pair = against_ref(c("pbo", "trt1", "trt2")),
      treatment = treatment ~ s1,
      eff_measure = h_diff
    )
  )
})

test_that("confint method for treatment_effect works as expected", {
  expect_snapshot(
    confint(treatment_effect(fit_binom, treatment = treatment ~ s1, eff_measure = h_diff))
  )
})

test_that("confint works correctly", {
  expect_snapshot(confint(robin_res1$marginal_mean))
  expect_snapshot(confint(robin_res1$contrast))

  expect_snapshot(confint(robin_res2$marginal_mean))
  expect_snapshot(confint(robin_res2$contrast))
})

test_that("confint works with parm argument", {
  expect_snapshot(confint(robin_res1$marginal_mean, parm = 1:2))
  expect_snapshot(confint(robin_res1$marginal_mean, parm = c("pbo", "trt2")))

  expect_snapshot(confint(robin_res1$contrast, parm = 1:2))
  expect_snapshot(confint(robin_res1$contrast, parm = c("trt1 v.s. pbo")))

  expect_error(confint(robin_res1$marginal_mean, parm = 4))
  expect_error(confint(robin_res1$marginal_mean, parm = "trt3"))
  expect_error(confint(robin_res1$contrast, parm = 4))
  expect_error(confint(robin_res1$contrast, parm = "trt v.s. pbo"))
})

test_that("confint works with level argument", {
  expect_snapshot(confint(robin_res1$marginal_mean, level = 0.8))
  expect_snapshot(confint(robin_res1$contrast, level = 0.7))
})

test_that("confint works with transform argument", {
  expect_snapshot(confint(robin_res1$contrast, transform = exp))
  expect_snapshot(confint(robin_res1$contrast, transform = identity))
})

---
title: "Package Introduction"
package: RobinCar2
output:
  rmarkdown::html_vignette:
          toc: true
vignette: |
  %\VignetteIndexEntry{Package Introduction}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options:
  chunk_output_type: console
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

RobinCar2 provides robust covariate adjustment methods for estimating and
inferring treatment effects through generalized linear models (glm) under
different randomization schema.

# Common Usage
A minimal call of `robin_lm()` and `robin_glm()`, consisting of only formula,
data arguments and the randomization scheme, will produce an object of class
`treatment_effect`.

```{r}
library(RobinCar2)
head(glm_data)
```

## Data Introduction

In the `glm_data`, we have the following columns:

1. `id` is the patient identifier.
1. `treatment` is the treatment assignment.
1. `s1` is the first stratification factor.
1. `s2` is the second stratification factor.
1. `covar` is the continuous covariate.
1. `y` is the continuous outcome.
1. `y_b` is the binary outcome.

## Obtain Treatment Effect for Continuous Outcomes Using the General Variance

For the continuous outcome `y`, the linear model includes `covar` as a covariate,
`s1` as a stratification factor. The randomization scheme is a permuted-block
randomization stratified by `s1`. The model formula also includes the treatment
by stratification interaction as `y ~ treatment * s1 + covar`.

```{r}
robin_lm(y ~ treatment * s1 + covar,
  data = glm_data,
  treatment = treatment ~ pb(s1)
)
```

We can also use the Huber-White variance estimator by setting `vcov = "vcovHC"`.
Please note that in this case, the model formula should not contain the
treatment by stratification (covariate) interaction.

```{r}
robin_lm(y ~ treatment + s1 + covar,
  data = glm_data,
  treatment = treatment ~ pb(s1),
  vcov = "vcovHC"
)
```

Note that `robin_glm` can also handle continuous outcomes using the default
family and the default link function `family = gaussian()`.

## Obtain Treatment Effect for Binary Outcomes

For binary outcomes, the logistic model includes `covar` as a covariate,
`s1` as a stratification factor. The randomization scheme is a permuted-block
randomization stratified by `s1`. The model formula also includes the treatment
by stratification interaction as `y_b ~ treatment * s1 + covar`. Note here we need
to specify `family` to be `binomial(link = "logit")`.

```{r}
robin_glm(y_b ~ treatment * s1 + covar,
  data = glm_data,
  treatment = treatment ~ pb(s1),
  family = binomial(link = "logit")
)
```

## Obtain Treatment Effect for Counts

For counts, the log link model includes `covar` as a covariate,
`s1` as a stratification factor. The randomization scheme is a permuted-block
randomization stratified by `s1`. The model formula also includes the treatment
by stratification interaction as `y_count ~ treatment * s1 + covar`. Note here
we need to specify `family` to be `poisson(link = "log")` to use the Poisson
model or to be `MASS::negative.binomial(theta = NA)` to use the negative
binomial model. A fixed `theta` could be provided if it is known.

```{r}
glm_data$y_count <- rpois(nrow(glm_data), lambda = 20)
robin_glm(
  y_count ~ treatment * s1 + covar,
  data = glm_data,
  treatment = treatment ~ pb(s1),
  family = MASS::negative.binomial(theta = 1)
)
```

## Using Different Covariate-Adaptive Randomization Schema

If the randomization schema is not permuted-block randomization, we can use
other randomization schema. Currently RobinCar2 supports `sr` for the simple
randomization (this was previously called `sp`), `pb` for the permuted-block 
randomization, and `ps` for the Pocock-Simon randomization.

```{r}
robin_glm(y_b ~ treatment * s1 + covar,
  data = glm_data,
  treatment = treatment ~ ps(s1),
  family = binomial(link = "logit")
)
```

## Obtain the Confidence Intervals for the Marginal Means and contrast

To obtain the confidence interval, we can use `confint` function.

Given the following model
```{r}
robin_res <- robin_glm(y_b ~ treatment * s1 + covar,
  data = glm_data,
  treatment = treatment ~ ps(s1),
  family = binomial(link = "logit"),
  contrast = "log_risk_ratio"
)
robin_res
```

It is easy to obtain the confidence interval matrix for the marginal mean, at specified level.
If `parm` is not provided, the complete matrix (all treatment groups) will be provided.

```{r}
confint(robin_res$marginal_mean, parm = 1:2, level = 0.7)
confint(robin_res$marginal_mean, level = 0.7)
```

Similarly for the contrast, however it has an additional argument `transform` to provide the confidence interval at transformed level.
Thus, standard error is removed because it does not make sense anymore.
By default, if the `log_risk_ratio` or `log_odds_ratio` is used as contrast, the `confint` will transform it back using exponential function.
You can also specify the `transform` to be `identity` to avoid the transformation.

```{r}
confint(robin_res$contrast)
confint(robin_res$contrast, transform = identity)
```

# print method for surv_effect works as expected (unstratified, unadjusted)

    Code
      print(x)
    Output
      Model        : Surv(time, status) ~ 1
      Randomization: sex ~ sr(1) (Simple)
      
      Contrast     : Log Hazard Ratio
      
                       Estimate Std.Err Z Value Pr(>|z|)   
      Male v.s. Female  0.53037 0.16718  3.1724 0.001512 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
      
      Test         : Log-Rank
      
                       Test Stat. Pr(>|z|)   
      Male v.s. Female     3.2135 0.001311 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

# print method for surv_effect works as expected (stratified, unadjusted)

    Code
      print(x)
    Output
      Model        : Surv(time, status) ~ 1 + strata(strata)
      Randomization: sex ~ sr(1) (Simple)
      Stratification variables:  strata 
      
      Contrast     : Stratified Log Hazard Ratio
      
                       Estimate Std.Err Z Value Pr(>|z|)   
      Male v.s. Female   0.5536  0.1706  3.2451 0.001174 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
      
      Test         : Stratified Log-Rank
      
                       Test Stat. Pr(>|z|)   
      Male v.s. Female     3.2856 0.001018 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

# print method for surv_effect works as expected (unstratified, adjusted)

    Code
      print(x)
    Output
      Model        : Surv(time, status) ~ meal.cal + age
      Randomization: sex ~ sr(1) (Simple)
      Covariates adjusted for: meal.cal, age (including interactions with sex)
      
      Contrast     : Covariate-adjusted Log Hazard Ratio
      
                       Estimate Std.Err Z Value Pr(>|z|)   
      Male v.s. Female  0.48335 0.18631  2.5944 0.009477 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
      
      Test         : Covariate-adjusted Log-Rank
      
                       Test Stat. Pr(>|z|)   
      Male v.s. Female     2.6858 0.007236 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

# print method for surv_effect works as expected (stratified, adjusted)

    Code
      print(x)
    Output
      Model        : Surv(time, status) ~ meal.cal + age + strata(strata)
      Randomization: sex ~ sr(1) (Simple)
      Stratification variables:  strata 
      Covariates adjusted for: meal.cal, age (including interactions with sex)
      
      Contrast     : Covariate-adjusted Stratified Log Hazard Ratio
      
                       Estimate Std.Err Z Value Pr(>|z|)   
      Male v.s. Female  0.54791 0.19118   2.866 0.004157 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
      
      Test         : Covariate-adjusted Stratified Log-Rank
      
                       Test Stat. Pr(>|z|)   
      Male v.s. Female     2.9496 0.003181 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

# print method for surv_effect works as expected when no hazard ratio was estimated

    Code
      print(x)
    Output
      Model        : Surv(time, status) ~ meal.cal + age + strata(strata)
      Randomization: sex ~ sr(1) (Simple)
      Stratification variables:  strata 
      Covariates adjusted for: meal.cal, age (including interactions with sex)
      
      Contrast     : None
      
      Test         : Covariate-adjusted Stratified Log-Rank
      
                       Test Stat. Pr(>|z|)   
      Male v.s. Female     2.9496 0.003181 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

# table method for surv_effect works as expected

    Code
      result <- table(x)
    Output
      Number of patients and events per stratum and treatment arm:
        strata    sex Patients Events
      1      0 Female       20      6
      2      0   Male       30     24
      3      1 Female       29     21
      4      1   Male       57     43
      5      2 Female       18     14
      6      2   Male       25     24
      7      3   Male        1      1

# confint method for surv_effect works as expected

    Code
      result <- confint(x)

---

    Code
      expect_message(result <- confint(x, transform = exp),
      "The confidence interval is transformed.", fixed = TRUE)

# h_lr_score_no_strata_no_cov works as expected with default options

    structure(0.0895537761860842, sigma_l2 = 0.17706769289317, se_theta_l = 0.15738484553678, n = 228L, give_rand_strat_warning = FALSE)

# h_lr_score_no_strata_no_cov works as expected with custom n

    structure(0.051045652426068, sigma_l2 = 0.100928584949107, se_theta_l = 0.15738484553678, n = 400, give_rand_strat_warning = FALSE)

# h_lr_score_no_strata_no_cov works as expected when not using ties factor

    structure(0.00479534279339572, sigma_l2 = 0.158585973213507, se_theta_l = 0.166303040954794, n = 228L, give_rand_strat_warning = FALSE)

# h_lr_score_strat works as expected with default options

    structure(0.0896871248297144, sigma_l2 = 0.169145720705825, se_theta_l = 0.161382546477903, n = 227L, give_rand_strat_warning = FALSE)

# h_lr_score_strat works with multiple strata variables

    structure(0.0896871248297144, sigma_l2 = 0.169145720705825, se_theta_l = 0.161382546477903, n = 227L, give_rand_strat_warning = FALSE)

# h_lr_score_strat works as expected when not using ties factor

    structure(0.0896871248297144, sigma_l2 = 0.169404689852017, se_theta_l = 0.161259146456279, n = 227L, give_rand_strat_warning = FALSE)

# h_lr_score_cov works as expected with default options

    structure(0.0851266162073163, se_theta_l = 0.156797551454514, sigma_l2 = 0.175748674116587, n = 228L, give_rand_strat_warning = FALSE)

# h_lr_score_cov works as expected when not using ties factor

    structure(0.0851266162073163, se_theta_l = 0.156595350460101, sigma_l2 = 0.176209700432439, n = 228L, give_rand_strat_warning = FALSE)

# h_lr_score_cov works when using the unadjusted standard error option

    structure(0.0850667744102501, se_theta_l = 0.159015041100831, sigma_l2 = 0.170735177622095, n = 228L, give_rand_strat_warning = FALSE)

---

    structure(0.0850667744102501, se_theta_l = 0.156579398029655, sigma_l2 = 0.176173801156959, n = 228L, give_rand_strat_warning = FALSE)

# h_lr_score_cov works with a single factor covariate

    structure(0.0889518680173707, se_theta_l = 0.157769376630947, sigma_l2 = 0.176637348077421, n = 227L, give_rand_strat_warning = FALSE)

# h_lr_score_strat_cov works as expected with default options

    structure(0.0897521729142622, se_theta_l = 0.161382421810696, sigma_l2 = 0.169145459377487, n = 227L, give_rand_strat_warning = FALSE)

# h_lr_score_strat_cov works with multiple strata variables

    structure(0.0897521729142622, se_theta_l = 0.161382421810696, sigma_l2 = 0.169145459377487, n = 227L, give_rand_strat_warning = FALSE)

# h_lr_score_strat_cov works as expected when not using ties factor

    structure(0.0897521729142622, se_theta_l = 0.161259022074831, sigma_l2 = 0.169404428523679, n = 227L, give_rand_strat_warning = FALSE)

# h_lr_score_strat_cov works as expected with unadjusted standard error

    structure(0.089793780031155, se_theta_l = 0.164884345428146, sigma_l2 = 0.162036005256482, n = 227L, give_rand_strat_warning = FALSE)

---

    structure(0.089793780031155, se_theta_l = 0.164884345428146, sigma_l2 = 0.162036005256482, n = 227L, give_rand_strat_warning = FALSE)

---

    structure(0.089793780031155, se_theta_l = 0.161258812068077, sigma_l2 = 0.169403987295021, n = 227L, give_rand_strat_warning = FALSE)

# h_get_lm_results works as expected

    list(beta_est = list(A = c(covariate1 = 0.124630109689139, covariate2 = 0.417147008547665
    ), B = c(covariate1 = 1.8889272308284, covariate2 = -0.824394901815606
    )), residuals = list(A = c("1" = 0.328159679550758, "2" = 0.784356412897211, 
    "4" = -0.487223558432553, "7" = 0.179869428929531, "8" = -0.237599941348964, 
    "9" = -0.567562021595984), B = c("3" = 0.207437533059178, "5" = 0.00713842971687903, 
    "6" = -0.185427485486945, "10" = -0.029148477289112)))

# h_get_strat_lm_results works as expected

    WAoAAAACAAQFAgACAwAAAAITAAAAAgAAAhMAAAACAAACDgAAAAI/2EodY7qjWT/Yhf+la9RI
    AAAEAgAAAAEABAAJAAAAA2RpbQAAAA0AAAACAAAAAgAAAAEAAAQCAAAAAQAEAAkAAAAIZGlt
    bmFtZXMAAAATAAAAAgAAABAAAAACAAQACQAAAApjb3ZhcmlhdGUxAAQACQAAAApjb3Zhcmlh
    dGUyAAAA/gAAAP4AAAIOAAAAAj/wvxc1pMcSv+gFJa7236oAAAQCAAAB/wAAAA0AAAACAAAA
    AgAAAAEAAAQCAAAC/wAAABMAAAACAAAAEAAAAAIABAAJAAAACmNvdmFyaWF0ZTEABAAJAAAA
    CmNvdmFyaWF0ZTIAAAD+AAAA/gAABAIAAAABAAQACQAAAAVuYW1lcwAAABAAAAACAAQACQAA
    AAFBAAQACQAAAAFCAAAA/gAAAhMAAAACAAACDgAAAAu/tDGyIAgxfD/wKxLtnpFov9+JqMTO
    xPI/xSMSkMK6bL+ZG5UDNbOgv+KLAzDruyY/+de0QI9Snz/aaNrYVMlYv+GJ39s2z2y/7CFQ
    4UUqQr/jOKUwzRA8AAAEAgAAA/8AAAAQAAAACwAEAAkAAAABMQAEAAkAAAABMgAEAAkAAAAB
    NAAEAAkAAAABNwAEAAkAAAABOAAEAAkAAAABOQAEAAkAAAACMTEABAAJAAAAAjEyAAQACQAA
    AAIxNgAEAAkAAAACMTcABAAJAAAAAjE5AAAA/gAAAg4AAAAJP9HKSfx49fo/yoMS8V3yML+g
    fiRxSj28v9z8Dub+p1w/92hRkoZfVL/3wer7XqF6P+glamCsz02/qszDp4CpqL/lxWtUhEBs
    AAAEAgAAA/8AAAAQAAAACQAEAAkAAAABMwAEAAkAAAABNQAEAAkAAAABNgAEAAkAAAACMTAA
    BAAJAAAAAjEzAAQACQAAAAIxNAAEAAkAAAACMTUABAAJAAAAAjE4AAQACQAAAAIyMAAAAP4A
    AAQCAAAD/wAAABAAAAACAAQACQAAAAFBAAQACQAAAAFCAAAA/gAABAIAAAP/AAAAEAAAAAIA
    BAAJAAAACGJldGFfZXN0AAQACQAAAAlyZXNpZHVhbHMAAAD+

# h_log_hr_est_via_score works as expected

    list(theta = 0.530368384142102, se = 0.167180005660338, sigma_l2 = 0.156926569811932, 
        n = 228L)

# h_log_hr_est_via_score does not give spurious warning

    list(theta = -0.0507718560099652, se = 0.182997810903141, sigma_l2 = 0.17218348832935, 
        n = 167L)

# h_log_hr_est_via_score extends the search interval as needed

    list(theta = 0.530397994806822, se = 0.167180879290955, sigma_l2 = 0.156924929726076, 
        n = 228L)

# h_lr_test_via_score works as expected

    list(u_l = 0.0895537761860842, sigma_l2 = 0.17706769289317, tau_l = 3.21352484896035, 
        pval = 0.0013111645203555, n = 228L, give_rand_strat_warning = FALSE)

# robin_surv_comparison works as expected without covariate adjustment

    list(estimate = 0.530368384142102, se = 0.167180005660338, hr_n = 228L, 
        hr_sigma_l2 = 0.156926569811932, test_stat = 3.21352484896035, 
        p_value = 0.0013111645203555, test_score = 0.0895537761860842, 
        test_n = 228L, test_sigma_l2 = 0.17706769289317, give_rand_strat_warning = FALSE)

# robin_surv_comparison can skip the hazard ratio estimation

    list(estimate = NA_real_, se = NA_real_, hr_n = NA_integer_, 
        hr_sigma_l2 = NA_real_, test_stat = 3.21352484896035, p_value = 0.0013111645203555, 
        test_score = 0.0895537761860842, test_n = 228L, test_sigma_l2 = 0.17706769289317, 
        give_rand_strat_warning = FALSE)

# robin_surv_no_strata_no_cov works as expected

    list(estimate = -0.530368384142102, se = 0.167180005660338, hr_n = 228L, 
        hr_sigma_l2 = 0.156926569811932, test_stat = -3.21352484896035, 
        p_value = 0.0013111645203555, test_score = -0.0895537761860842, 
        test_n = 228L, test_sigma_l2 = 0.17706769289317, give_rand_strat_warning = FALSE)

# robin_surv_strata works as expected

    list(estimate = -0.553595870294275, se = 0.17059512525311, hr_n = 227L, 
        hr_sigma_l2 = 0.151370382611702, test_stat = -3.2855836062253, 
        p_value = 0.00101771334472415, test_score = -0.0896871248297144, 
        test_n = 227L, test_sigma_l2 = 0.169145720705825, give_rand_strat_warning = FALSE)

# robin_surv_cov works as expected

    list(estimate = -0.5016113861505, se = 0.16561813992359, hr_n = 228L, 
        hr_sigma_l2 = 0.157110113177543, test_stat = -3.06610317830763, 
        p_value = 0.0021686846372504, test_score = -0.0851266162073163, 
        test_n = 228L, test_sigma_l2 = 0.175748674116587, give_rand_strat_warning = FALSE)

# robin_surv_strata_cov works as expected

    list(estimate = -0.0136495961352817, se = 0.158066245456898, 
        hr_n = 227L, hr_sigma_l2 = 0.172478508438683, test_stat = -0.0835488066000914, 
        p_value = 0.933415170960403, test_score = -0.00229971887336489, 
        test_n = 227L, test_sigma_l2 = 0.171986696328211, give_rand_strat_warning = FALSE)

# h_log_hr_coef_mat works as expected

    structure(c(0.5, 1, 0.5, 0.617075077451974), dim = c(1L, 4L), dimnames = list(
        "B v.s. A", c("Estimate", "Std.Err", "Z Value", "Pr(>|z|)"
        )))

# h_log_hr_coef_mat works as expected for multiple comparisons

    WAoAAAACAAQFAQACAwAAAAIOAAAADD/gAAAAAAAAP+ZmZmZmZmY/7MzMzMzMzT/wAAAAAAAA
    QAAAAAAAAABACAAAAAAAAD/gAAAAAAAAP9ZmZmZmZmY/0zMzMzMzMz/jvxQ7mqcTP+c+KqcH
    D68/6HQjpnfpDgAABAIAAAABAAQACQAAAANkaW0AAAANAAAAAgAAAAMAAAAEAAAEAgAAAAEA
    BAAJAAAACGRpbW5hbWVzAAAAEwAAAAIAAAAQAAAAAwAEAAkAAAAIQiB2LnMuIEEABAAJAAAA
    CEEgdi5zLiBDAAQACQAAAAhDIHYucy4gQgAAABAAAAAEAAQACQAAAAhFc3RpbWF0ZQAEAAkA
    AAAHU3RkLkVycgAEAAkAAAAHWiBWYWx1ZQAEAAkAAAAIUHIoPnx6fCkAAAD+

# h_test_mat works as expected

    structure(c(0.5, 0.7, 0.05, 0.01), dim = c(2L, 2L), dimnames = list(
        c("B v.s. A", "A v.s. B"), c("Test Stat.", "Pr(>|z|)")))

# robin_surv works as expected without strata or covariates

    structure(c(-0.0374048047550865, 0.156467896653491, -0.239057375698749, 
    0.811061092843519), dim = c(1L, 4L), dimnames = list("1 v.s. 0", 
        c("Estimate", "Std.Err", "Z Value", "Pr(>|z|)")))

---

    WAoAAAACAAQFAQACAwAAAAIOAAAAAr/Oo7/aMxoGP+nyNmcckPoAAAQCAAAAAQAEAAkAAAAD
    ZGltAAAADQAAAAIAAAABAAAAAgAABAIAAAABAAQACQAAAAhkaW1uYW1lcwAAABMAAAACAAAA
    EAAAAAEABAAJAAAACDEgdi5zLiAwAAAAEAAAAAIABAAJAAAAClRlc3QgU3RhdC4ABAAJAAAA
    CFByKD58enwpAAAA/g==

---

    WAoAAAACAAQFAQACAwAAAAMTAAAAAwAAAw0AAAACAAAAAQAAAAIAAAQCAAAAAQAEAAkAAAAG
    bGV2ZWxzAAAAEAAAAAIABAAJAAAAATAABAAJAAAAATEAAAQCAAAAAQAEAAkAAAAFY2xhc3MA
    AAAQAAAAAQAEAAkAAAAGZmFjdG9yAAAA/gAAAA0AAAACAAAAcgAAAHEAAAANAAAAAgAAAFIA
    AABSAAAEAgAAAAEABAAJAAAABW5hbWVzAAAAEAAAAAMABAAJAAAABGVjb2cABAAJAAAACFBh
    dGllbnRzAAQACQAAAAZFdmVudHMAAAQCAAAAAQAEAAkAAAAJcm93Lm5hbWVzAAAADQAAAAKA
    AAAA/////gAABAIAAAL/AAAAEAAAAAEABAAJAAAACmRhdGEuZnJhbWUAAAD+

# robin_surv works as expected with strata

    structure(c(0.553595870294275, 0.17059512525311, 3.24508610356194, 
    0.00117415114967595), dim = c(1L, 4L), dimnames = list("Male v.s. Female", 
        c("Estimate", "Std.Err", "Z Value", "Pr(>|z|)")))

---

    WAoAAAACAAQFAQACAwAAAAIOAAAAAkAKSOAOyBdTP1CsmWIRFpkAAAQCAAAAAQAEAAkAAAAD
    ZGltAAAADQAAAAIAAAABAAAAAgAABAIAAAABAAQACQAAAAhkaW1uYW1lcwAAABMAAAACAAAA
    EAAAAAEABAAJAAAAEE1hbGUgdi5zLiBGZW1hbGUAAAAQAAAAAgAEAAkAAAAKVGVzdCBTdGF0
    LgAEAAkAAAAIUHIoPnx6fCkAAAD+

---

    WAoAAAACAAQFAQACAwAAAAMTAAAABAAAAw0AAAAHAAAAAQAAAAEAAAACAAAAAgAAAAMAAAAD
    AAAABAAABAIAAAABAAQACQAAAAZsZXZlbHMAAAAQAAAABAAEAAkAAAABMAAEAAkAAAABMQAE
    AAkAAAABMgAEAAkAAAABMwAABAIAAAABAAQACQAAAAVjbGFzcwAAABAAAAABAAQACQAAAAZm
    YWN0b3IAAAD+AAADDQAAAAcAAAABAAAAAgAAAAEAAAACAAAAAQAAAAIAAAACAAAEAgAAAf8A
    AAAQAAAAAgAEAAkAAAAGRmVtYWxlAAQACQAAAARNYWxlAAAEAgAAAv8AAAAQAAAAAQAEAAkA
    AAAGZmFjdG9yAAAA/gAAAA0AAAAHAAAAGwAAACQAAAAqAAAARwAAABUAAAAdAAAAAQAAAA0A
    AAAHAAAACQAAABwAAAAcAAAANgAAABAAAAAcAAAAAQAABAIAAAABAAQACQAAAAVuYW1lcwAA
    ABAAAAAEAAQACQAAAAZzdHJhdGEABAAJAAAAA3NleAAEAAkAAAAIUGF0aWVudHMABAAJAAAA
    BkV2ZW50cwAABAIAAAABAAQACQAAAAlyb3cubmFtZXMAAAANAAAAAoAAAAD////5AAAEAgAA
    Av8AAAAQAAAAAQAEAAkAAAAKZGF0YS5mcmFtZQAAAP4=

# robin_surv works as expected with covariates

    structure(c(-0.00445020431030736, 0.155227462755351, -0.0286689238573794, 
    0.977128641344646), dim = c(1L, 4L), dimnames = list("1 v.s. 0", 
        c("Estimate", "Std.Err", "Z Value", "Pr(>|z|)")))

---

    WAoAAAACAAQFAQACAwAAAAIOAAAAAr+bioZy8DyXP+9QOWI2QekAAAQCAAAAAQAEAAkAAAAD
    ZGltAAAADQAAAAIAAAABAAAAAgAABAIAAAABAAQACQAAAAhkaW1uYW1lcwAAABMAAAACAAAA
    EAAAAAEABAAJAAAACDEgdi5zLiAwAAAAEAAAAAIABAAJAAAAClRlc3QgU3RhdC4ABAAJAAAA
    CFByKD58enwpAAAA/g==

---

    WAoAAAACAAQFAQACAwAAAAMTAAAAAwAAAw0AAAACAAAAAQAAAAIAAAQCAAAAAQAEAAkAAAAG
    bGV2ZWxzAAAAEAAAAAIABAAJAAAAATAABAAJAAAAATEAAAQCAAAAAQAEAAkAAAAFY2xhc3MA
    AAAQAAAAAQAEAAkAAAAGZmFjdG9yAAAA/gAAAA0AAAACAAAAcgAAAHEAAAANAAAAAgAAAFIA
    AABSAAAEAgAAAAEABAAJAAAABW5hbWVzAAAAEAAAAAMABAAJAAAABGVjb2cABAAJAAAACFBh
    dGllbnRzAAQACQAAAAZFdmVudHMAAAQCAAAAAQAEAAkAAAAJcm93Lm5hbWVzAAAADQAAAAKA
    AAAA/////gAABAIAAAL/AAAAEAAAAAEABAAJAAAACmRhdGEuZnJhbWUAAAD+

# robin_surv works as expected with strata and covariates

    structure(c(-0.0122920932408829, 0.159009954919017, -0.0773039225571954, 
    0.938381770498044), dim = c(1L, 4L), dimnames = list("1 v.s. 0", 
        c("Estimate", "Std.Err", "Z Value", "Pr(>|z|)")))

---

    WAoAAAACAAQFAQACAwAAAAIOAAAAAr+zWeUldOI4P+4SY/wj4GMAAAQCAAAAAQAEAAkAAAAD
    ZGltAAAADQAAAAIAAAABAAAAAgAABAIAAAABAAQACQAAAAhkaW1uYW1lcwAAABMAAAACAAAA
    EAAAAAEABAAJAAAACDEgdi5zLiAwAAAAEAAAAAIABAAJAAAAClRlc3QgU3RhdC4ABAAJAAAA
    CFByKD58enwpAAAA/g==

---

    WAoAAAACAAQFAQACAwAAAAMTAAAABAAAAw0AAAAEAAAAAQAAAAEAAAACAAAAAgAABAIAAAAB
    AAQACQAAAAZsZXZlbHMAAAAQAAAAAgAEAAkAAAAGRmVtYWxlAAQACQAAAARNYWxlAAAEAgAA
    AAEABAAJAAAABWNsYXNzAAAAEAAAAAEABAAJAAAABmZhY3RvcgAAAP4AAAMNAAAABAAAAAEA
    AAACAAAAAQAAAAIAAAQCAAAB/wAAABAAAAACAAQACQAAAAEwAAQACQAAAAExAAAEAgAAAv8A
    AAAQAAAAAQAEAAkAAAAGZmFjdG9yAAAA/gAAAA0AAAAEAAAAMAAAACoAAABBAAAARwAAAA0A
    AAAEAAAAGQAAABwAAAA4AAAANgAABAIAAAABAAQACQAAAAVuYW1lcwAAABAAAAAEAAQACQAA
    AANzZXgABAAJAAAABGVjb2cABAAJAAAACFBhdGllbnRzAAQACQAAAAZFdmVudHMAAAQCAAAA
    AQAEAAkAAAAJcm93Lm5hbWVzAAAADQAAAAKAAAAA/////AAABAIAAAL/AAAAEAAAAAEABAAJ
    AAAACmRhdGEuZnJhbWUAAAD+

# robin_surv works also with character variable in the correlation case

    structure(c(0.546676517528469, 0.237207661991659, 2.30463262838319, 
    0.0211871569907112), dim = c(1L, 4L), dimnames = list("Male v.s. Female", 
        c("Estimate", "Std.Err", "Z Value", "Pr(>|z|)")))

# robin_surv gives a warning if stratified randomization was specified but simple log rank test used

    Code
      robin_surv(Surv(time, status) ~ 1, data = surv_data, treatment = sex ~ pb(ecog))
    Condition
      Warning:
      It looks like you have not included all of the variables that were used during randomization in your analysis `formula`. You can either:
      
      a. adjust for all joint levels in your `formula` using `+ ecog` or
      b. perform a stratified test by adding to your `formula` the term `+ strata(ecog)`
      
      NOTE: (b) changes the null hypothesis from your current model specification. Please see the vignette `robincar-survival` for details.
    Output
      Model        : Surv(time, status) ~ 1
      Randomization: sex ~ pb(ecog) (Permuted-Block)
      
      Contrast     : Log Hazard Ratio
      
                       Estimate Std.Err Z Value Pr(>|z|)   
      Male v.s. Female  0.52309 0.16742  3.1244 0.001782 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
      
      Test         : Log-Rank
      
                       Test Stat. Pr(>|z|)   
      Male v.s. Female     3.1636 0.001558 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

# robin_surv does give a warning if strata are not sufficiently included in the analysis model

    Code
      robin_surv(Surv(time, status) ~ 1 + strata(ecog), data = surv_data, treatment = sex ~
        pb(strata))
    Condition
      Warning:
      It looks like you have not included all of the variables that were used during randomization in your analysis `formula`. You can either:
      
      a. adjust for all joint levels in your `formula` using `+ strata` or
      b. perform a stratified test by adding to your `formula` the term `+ strata(strata)`
      
      NOTE: (b) changes the null hypothesis from your current model specification. Please see the vignette `robincar-survival` for details.
    Output
      Model        : Surv(time, status) ~ 1 + strata(ecog)
      Randomization: sex ~ pb(strata) (Permuted-Block)
      Stratification variables:  ecog 
      
      Contrast     : Stratified Log Hazard Ratio
      
                       Estimate Std.Err Z Value Pr(>|z|)   
      Male v.s. Female  0.54989 0.16992  3.2362 0.001211 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
      
      Test         : Stratified Log-Rank
      
                       Test Stat. Pr(>|z|)   
      Male v.s. Female     3.2776 0.001047 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

# robin_surv does give a warning if strata are not sufficiently included in the covariates

    Code
      robin_surv(Surv(time, status) ~ 1 + ph.karno, data = surv_data, treatment = sex ~
        pb(ecog))
    Condition
      Warning:
      It looks like you have not included all of the variables that were used during randomization in your analysis `formula`. You can either:
      
      a. adjust for all joint levels in your `formula` using `+ ecog` or
      b. perform a stratified test by adding to your `formula` the term `+ strata(ecog)`
      
      NOTE: (b) changes the null hypothesis from your current model specification. Please see the vignette `robincar-survival` for details.
    Output
      Model        : Surv(time, status) ~ 1 + ph.karno
      Randomization: sex ~ pb(ecog) (Permuted-Block)
      Covariates adjusted for: ph.karno (including interactions with sex)
      
      Contrast     : Covariate-adjusted Log Hazard Ratio
      
                       Estimate Std.Err Z Value Pr(>|z|)   
      Male v.s. Female  0.51335 0.16366  3.1366 0.001709 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
      
      Test         : Covariate-adjusted Log-Rank
      
                       Test Stat. Pr(>|z|)   
      Male v.s. Female      3.165 0.001551 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

# robin_surv warns if strata are insufficiently included in covariate adjusted stratified model

    Code
      robin_surv(Surv(time, status) ~ 1 + strata(ecog) + ph.karno, data = surv_data,
      treatment = sex ~ pb(strata))
    Condition
      Warning:
      It looks like you have not included all of the variables that were used during randomization in your analysis `formula`. You can either:
      
      a. adjust for all joint levels in your `formula` using `+ strata` or
      b. perform a stratified test by adding to your `formula` the term `+ strata(strata)`
      
      NOTE: (b) changes the null hypothesis from your current model specification. Please see the vignette `robincar-survival` for details.
    Output
      Model        : Surv(time, status) ~ 1 + strata(ecog) + ph.karno
      Randomization: sex ~ pb(strata) (Permuted-Block)
      Stratification variables:  ecog 
      Covariates adjusted for: ph.karno (including interactions with sex)
      
      Contrast     : Covariate-adjusted Stratified Log Hazard Ratio
      
                       Estimate Std.Err Z Value Pr(>|z|)   
      Male v.s. Female  0.53849 0.16663  3.2317 0.001231 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
      
      Test         : Covariate-adjusted Stratified Log-Rank
      
                       Test Stat. Pr(>|z|)   
      Male v.s. Female     3.2683 0.001082 **
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

# predict_counterfactual works for guassian

    Code
      predict_counterfactual(fit_glm, treatment ~ 1)
    Output
      Model        :  y ~ treatment * s1 + covar 
      Randomization:  treatment ~ 1  ( Simple )
      Variance Type:  vcovG 
      Marginal Mean: 
           Estimate  Std.Err    2.5 % 97.5 %
      pbo  0.200321 0.067690 0.067651 0.3330
      trt1 0.763971 0.075929 0.615152 0.9128
      trt2 0.971250 0.076543 0.821228 1.1213

# predict_counterfactual works for guassian with lm

    Code
      predict_counterfactual(fit_lm, treatment ~ 1, data = glm_data)
    Output
      Model        :  y ~ treatment * s1 + covar 
      Randomization:  treatment ~ 1  ( Simple )
      Variance Type:  vcovG 
      Marginal Mean: 
           Estimate  Std.Err    2.5 % 97.5 %
      pbo  0.200321 0.067690 0.067651 0.3330
      trt1 0.763971 0.075929 0.615152 0.9128
      trt2 0.971250 0.076543 0.821228 1.1213

# predict_counterfactual works for binomial

    Code
      predict_counterfactual(fit_binom, treatment ~ 1)
    Output
      Model        :  y_b ~ treatment * s1 + covar 
      Randomization:  treatment ~ 1  ( Simple )
      Variance Type:  vcovG 
      Marginal Mean: 
           Estimate  Std.Err    2.5 % 97.5 %
      pbo  0.356097 0.033599 0.290243 0.4219
      trt1 0.580696 0.034418 0.513238 0.6482
      trt2 0.621386 0.034019 0.554711 0.6881

# predict_counterfactual works if contrast are non-standard

    Code
      pc
    Output
      Model        :  y_b ~ treatment * s1 
      Randomization:  treatment ~ 1  ( Simple )
      Variance Type:  vcovG 
      Marginal Mean: 
           Estimate  Std.Err    2.5 % 97.5 %
      pbo  0.366007 0.033864 0.299634 0.4324
      trt1 0.580984 0.035027 0.512332 0.6496
      trt2 0.610154 0.034472 0.542589 0.6777

# predict_counterfactual works with provided vcov function

    Code
      predict_counterfactual(fit_lm, treatment ~ 1, data = glm_data, vcov = vcov_dummy)
    Output
      Model        :  y ~ treatment * s1 + covar 
      Randomization:  treatment ~ 1  ( Simple )
      Variance Type:  vcov_dummy  
      Marginal Mean: 
           Estimate  Std.Err    2.5 % 97.5 %
      pbo   0.20032  0.31623 -0.41947 0.8201
      trt1  0.76397  0.31623  0.14418 1.3838
      trt2  0.97125  0.31623  0.35145 1.5910

---

    Code
      predict_counterfactual(fit_lm, treatment ~ 1, data = glm_data, vcov = NULL)
    Output
      Model        :  y ~ treatment * s1 + covar 
      Randomization:  treatment ~ 1  ( Simple )
      Variance Type:  
      Marginal Mean: 
           Estimate Std.Err 2.5 % 97.5 %
      pbo   0.20032      NA    NA     NA
      trt1  0.76397      NA    NA     NA
      trt2  0.97125      NA    NA     NA

# confint method for prediction_cf works as expected

    Code
      confint(predict_counterfactual(fit_binom, treatment = treatment ~ s1,
      eff_measure = h_diff))
    Output
            Estimate     2.5 %    97.5 %
      pbo  0.3560965 0.2902435 0.4219496
      trt1 0.5806957 0.5132377 0.6481538
      trt2 0.6213865 0.5547112 0.6880618

# treatment_effect works for lm/glm object

    Code
      treatment_effect(fit_binom, treatment = treatment ~ s1, eff_measure = h_diff)
    Output
                     Estimate  Std.Err Z Value  Pr(>|z|)    
      trt1 v.s. pbo  0.224599 0.047711  4.7075 2.508e-06 ***
      trt2 v.s. pbo  0.265290 0.047534  5.5810 2.391e-08 ***
      trt2 v.s. trt1 0.040691 0.047941  0.8488     0.396    
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

---

    Code
      treatment_effect(fit_lm, treatment = treatment ~ s1, eff_measure = h_diff,
      data = glm_data)
    Output
                     Estimate Std.Err Z Value  Pr(>|z|)    
      trt1 v.s. pbo   0.56365 0.10074  5.5952 2.203e-08 ***
      trt2 v.s. pbo   0.77093 0.10133  7.6082 2.779e-14 ***
      trt2 v.s. trt1  0.20728 0.10683  1.9402   0.05235 .  
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

# treatment_effect works if variance is not used

    Code
      treatment_effect(fit_binom, treatment = treatment ~ s1, eff_measure = h_diff,
      variance = NULL)
    Output
                     Estimate  Std.Err Z Value  Pr(>|z|)    
      trt1 v.s. pbo  0.224599 0.047711  4.7075 2.508e-06 ***
      trt2 v.s. pbo  0.265290 0.047534  5.5810 2.391e-08 ***
      trt2 v.s. trt1 0.040691 0.047941  0.8488     0.396    
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

# treatment_effect works if pair is defined

    Code
      treatment_effect(fit_binom, pair = against_ref(c("pbo", "trt1", "trt2")),
      treatment = treatment ~ s1, eff_measure = h_diff)
    Output
                    Estimate  Std.Err Z Value  Pr(>|z|)    
      trt1 v.s. pbo 0.224599 0.047711  4.7075 2.508e-06 ***
      trt2 v.s. pbo 0.265290 0.047534  5.5810 2.391e-08 ***
      ---
      Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

# confint method for treatment_effect works as expected

    Code
      confint(treatment_effect(fit_binom, treatment = treatment ~ s1, eff_measure = h_diff))
    Output
                       Estimate      2.5 %    97.5 %
      trt1 v.s. pbo  0.22459921  0.1310866 0.3181118
      trt2 v.s. pbo  0.26528993  0.1721248 0.3584551
      trt2 v.s. trt1 0.04069073 -0.0532712 0.1346527

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{randomization_schema}
\alias{randomization_schema}
\title{Randomization schema}
\usage{
randomization_schema
}
\description{
Randomization schema
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variance_hc.R
\name{vcovHC}
\alias{vcovHC}
\title{Heteroskedasticity-consistent covariance matrix for predictions}
\usage{
vcovHC(x, type = "HC3", ...)
}
\arguments{
\item{x}{(\code{prediction_cf}) Counter-factual prediction.}

\item{type}{(\code{character}) Type of HC covariance matrix.}

\item{...}{Additional arguments for \code{sandwich::vcovHC}.}
}
\value{
Matrix of the heteroskedasticity-consistent covariance for the predictions.
}
\description{
The heteroskedasticity-consistent covariance matrix for predictions
is obtained with \code{sandwich::vocvHC} using sandwich method.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival.R
\name{h_test_mat}
\alias{h_test_mat}
\title{Log-Rank Test Results Matrix}
\usage{
h_test_mat(x)
}
\arguments{
\item{x}{(\code{list}) A list containing the log-rank test results.}
}
\value{
A matrix with columns for the test statistic and p-value.
}
\description{
This function creates a matrix summarizing the results of the log-rank test.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prediction_cf.R, R/surv_effect.R,
%   R/treatment_effect.R
\name{confint}
\alias{confint}
\alias{confint.prediction_cf}
\alias{confint.surv_effect}
\alias{confint.treatment_effect}
\title{Confidence Interval}
\usage{
\method{confint}{prediction_cf}(object, parm, level = 0.95, include_se = FALSE, ...)

\method{confint}{surv_effect}(object, parm, level = 0.95, transform, ...)

\method{confint}{treatment_effect}(object, parm, level = 0.95, transform, ...)
}
\arguments{
\item{object}{Object to construct confidence interval.}

\item{parm}{(\code{character} or \code{integer}) Names of the parameters to construct confidence interval.}

\item{level}{(\code{numeric}) Confidence level.}

\item{include_se}{(\code{flag}) Whether to include the standard error as a column in the result matrix.}

\item{...}{Not used.}

\item{transform}{(\code{function}) Transform function.}
}
\value{
A \code{matrix} of the confidence interval.
}
\description{
Obtain the confidence interval for the marginal mean or the contrast.
}
\examples{
robin_res <- robin_glm(
  y_b ~ treatment * s1,
  data = glm_data, treatment = treatment ~ s1, contrast = "log_risk_ratio"
)
confint(robin_res$marginal_mean, level = 0.7)
confint(robin_res$contrast, parm = 1:3, level = 0.9)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variance_anhecova.R
\name{h_adjust_pi}
\alias{h_adjust_pi}
\title{Obtain Adjustment for Proportion of Treatment Assignment}
\usage{
h_adjust_pi(pi)
}
\arguments{
\item{pi}{(\code{numeric}) vector of proportions.}
}
\value{
Numeric matrix.
}
\description{
Obtain Adjustment for Proportion of Treatment Assignment
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival.R
\name{h_lr_test_via_score}
\alias{h_lr_test_via_score}
\title{Log-Rank Test via Score Function}
\usage{
h_lr_test_via_score(score_fun, ...)
}
\arguments{
\item{score_fun}{(\code{function}) The log-rank score function to be used for testing.}

\item{...}{Additional arguments passed to \code{score_fun}.}
}
\value{
A list containing:
\itemize{
\item \code{u_l}: The log-rank score statistic.
\item \code{sigma_l2}: The variance of the log-rank statistic.
\item \code{tau_l}: The log-rank test statistic.
\item \code{pval}: The two-sided p-value of the log-rank test.
\item \code{n}: The number of observations used in the calculation.
}
}
\description{
This function performs a log-rank test using the score function.
}
\details{
This activates the ties factor correction in the score function by passing
\code{use_ties_factor = TRUE} to the \code{score_fun}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RobinCar2-package.R
\docType{package}
\name{RobinCar2-package}
\alias{RobinCar2}
\alias{RobinCar2-package}
\title{\code{RobinCar2} Package}
\description{
\code{RobinCar2} implements unbiased prediction and robust inference of variance of a fit in R.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/openpharma/RobinCar2/}
  \item Report bugs at \url{https://github.com/openpharma/RobinCar2/issues}
}

}
\author{
\strong{Maintainer}: Liming Li \email{liming.li1@astrazeneca.com} (\href{https://orcid.org/0009-0008-6870-0878}{ORCID})

Authors:
\itemize{
  \item Liming Li \email{liming.li1@astrazeneca.com} (\href{https://orcid.org/0009-0008-6870-0878}{ORCID})
  \item Marlena Bannick \email{mnorwood@uw.edu} (\href{https://orcid.org/0000-0001-6279-5978}{ORCID})
  \item Daniel Sabanes Bove \email{daniel@rconis.com} (\href{https://orcid.org/0000-0002-0176-9239}{ORCID})
  \item Dong Xi \email{Dong.Xi1@gilead.com}
  \item Ting Ye \email{tingye1@uw.edu}
  \item Yanyao Yi \email{yi_yanyao@lilly.com}
}

Other contributors:
\itemize{
  \item Gregory Chen \email{xiangyi.gregory.chen@msd.com} [contributor]
  \item Gilead Sciences, Inc. [copyright holder, funder]
  \item F. Hoffmann-La Roche AG [copyright holder, funder]
  \item Merck Sharp & Dohme, Inc. [copyright holder, funder]
  \item AstraZeneca plc [copyright holder, funder]
  \item Eli Lilly and Company [copyright holder, funder]
  \item Novartis Pharma AG [copyright holder, funder]
  \item The University of Washington [copyright holder, funder]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv_effect.R
\name{surv_effect_methods}
\alias{surv_effect_methods}
\alias{print.surv_effect}
\alias{table}
\alias{table.default}
\alias{table.surv_effect}
\title{S3 Methods for \code{surv_effect}}
\usage{
\method{print}{surv_effect}(x, ...)

table(x, ...)

\method{table}{default}(x, ...)

\method{table}{surv_effect}(x, ...)
}
\arguments{
\item{x}{(\code{surv_effect}) the obtained result from \code{\link[=robin_surv]{robin_surv()}}.}

\item{...}{ignored additional arguments (for compatibility).}
}
\description{
S3 Methods for \code{surv_effect}
}
\section{Functions}{
\itemize{
\item \code{print(surv_effect)}: prints the \code{surv_effect} object.

\item \code{table(surv_effect)}: prints and returns invisibly the events table of the \code{surv_effect} object.

}}
\examples{
x <- robin_surv(
  formula = Surv(time, status) ~ meal.cal + age + strata(strata),
  data = surv_data,
  treatment = sex ~ pb(strata)
)
print(x)
table(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{jac_mat}
\alias{jac_mat}
\title{Obtain the Jacobian matrix}
\usage{
jac_mat(jac, pair)
}
\description{
Obtain the Jacobian matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival.R
\name{h_log_hr_est_via_score}
\alias{h_log_hr_est_via_score}
\title{Estimate Log Hazard Ratio via Score Function}
\usage{
h_log_hr_est_via_score(
  score_fun,
  interval = c(-5, 5),
  control = surv_control(),
  ...
)
}
\arguments{
\item{score_fun}{(\code{function}) The log-rank score function to be used for estimation.}

\item{interval}{(\code{numeric}) A numeric vector of length 2 specifying the interval in which to search for the root.}

\item{control}{(\code{list}) Control parameters from \code{\link[=surv_control]{surv_control()}} for the root-finding algorithm.}

\item{...}{Additional arguments passed to \code{score_fun}.}
}
\value{
A list containing:
\itemize{
\item \code{theta}: The estimated log hazard ratio.
\item \code{se}: The standard error of the estimated log hazard ratio.
\item \code{sigma_L2}: The variance of the log-rank statistic.
\item \code{n}: The number of observations used in the calculation.
}
}
\description{
This function estimates the log hazard ratio by finding the root of the log-rank score function.
}
\details{
This deactivates the ties factor correction in the score function by passing
\code{use_ties_factor = FALSE} to the \code{score_fun}. The root finding is done without calculating
the variance by passing \code{calculate_variance = FALSE}, which is only calculated at the solution.
This saves computation time, and avoids spurious warnings about negative variances during the root
finding process.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{glm_data}
\alias{glm_data}
\title{Example Trial Data for GLMs with Permute-Block Randomization}
\format{
A data frame with 600 rows and 7 columns:
\describe{
\item{id}{The ID of the patients.}
\item{treatment}{The treatment assignment, "pbo", "trt1" and "trt2"}
\item{s1}{The first stratification variable, "a" and "b".}
\item{s2}{The second stratification variable, "c" and "d".}
\item{covar}{The covariate following normal distribution.}
\item{y}{The continuous response.}
\item{y_b}{The binary response.}
}
}
\source{
The data is generated by the \code{create_glm_data.R} script.
}
\usage{
glm_data
}
\description{
This dataset contains the trial example data for GLMs with permute block randomization.
}
\keyword{dataset}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival.R
\name{h_log_hr_coef_mat}
\alias{h_log_hr_coef_mat}
\title{Log Hazard Ratio Coefficient Matrix}
\usage{
h_log_hr_coef_mat(x)
}
\arguments{
\item{x}{(\code{list}) A list containing the log hazard ratio estimates and their standard errors.}
}
\value{
A matrix with columns for the log hazard ratio estimate, standard error, z-value,
and p-value.
}
\description{
This function creates a coefficient matrix for the log hazard ratio estimates.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{block_sum}
\alias{block_sum}
\title{Block Sum of a matrix}
\usage{
block_sum(x, n)
}
\description{
Block Sum of a matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{h_first_fct_nested_in_second}
\alias{h_first_fct_nested_in_second}
\title{Check Whether First Factor is Nested in Second Factor}
\usage{
h_first_fct_nested_in_second(f1, f2)
}
\arguments{
\item{f1}{A factor variable, supposed to be more fine grained than \code{f2}.}

\item{f2}{A factor variable, supposed to be more coarse grained than \code{f1}.}
}
\value{
\code{TRUE} if \code{f1} is nested in \code{f2}, \code{FALSE} otherwise.
}
\description{
This function checks whether one factor variable is nested in another in terms of their levels
and the mapping of observations to these levels:
}
\details{
\itemize{
\item Both factors must have the same length.
\item The positions of \code{NA} values must be identical in both factors.
\item After removing \code{NA} values, \code{f1} must have at least as many observed levels as \code{f2}.
\item The mapping of observations to levels must be consistent, meaning that each level in \code{f1}
corresponds to exactly one level in \code{f2}. On the other hand, multiple levels in \code{f1}
can map to the same level in \code{f2}.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival.R
\name{robin_surv}
\alias{robin_surv}
\title{Covariate Adjusted and Stratified Survival Analysis}
\usage{
robin_surv(
  formula,
  data,
  treatment,
  comparisons,
  contrast = c("hazardratio", "none"),
  test = "logrank",
  control = surv_control(),
  ...
)
}
\arguments{
\item{formula}{(\code{formula}) A formula of analysis, of the form
\code{Surv(time, status) ~ covariates + strata(x, y, z)}.
If no covariates should be adjusted for, use \code{1} instead on the right hand side. The intercept must not be removed.
If no stratification factors should be used for the analysis, do not use \code{strata()} in the formula.}

\item{data}{(\code{data.frame}) Input data frame.}

\item{treatment}{(\code{formula}) A formula of treatment assignment or assignment by stratification, of the form
\code{treatment ~ scheme(vars)}. Note that currently the randomization scheme is not used in the analysis. However,
any variables that were used in the randomization scheme must be included in the model formula,
either as covariates, or as \code{strata()}.}

\item{comparisons}{(\code{list}) An optional list of comparisons between treatment levels to be performed,
see details. By default, all pairwise comparisons are performed automatically.}

\item{contrast}{(\code{character(1)}) The contrast statistic to be used, currently only \code{"hazardratio"}
is supported. Can be disabled by specifying \code{"none"}, in which case only the log-rank test is performed.}

\item{test}{(\code{character(1)}) The test to be used, currently only \code{"logrank"} is supported.}

\item{control}{(\code{list}) Control parameters for the root-finding algorithm used to estimate the
log hazard ratio, usually created with \code{\link[=surv_control]{surv_control()}}. The arguments correspond to
\code{tol}, \code{maxiter}, and \code{trace} in \code{\link[stats:uniroot]{stats::uniroot()}}.}

\item{...}{Additional arguments passed to the survival analysis functions, in particular \code{hr_se_plugin_adjusted}
(please see \link[=survival_score_functions]{here} for details).}
}
\value{
A \code{surv_effect} object containing the results of the survival analysis.
}
\description{
Calculate log-rank test as well as hazard ratio estimates for survival data, optionally adjusted
for covariates and a stratification factor.
}
\details{
The user can optionally specify a list of comparisons between treatment levels to be performed.
The list must have two elements:
\itemize{
\item Treatment level indices of the treatment group.
\item Treatment level indices of the control group.
}

So for example if you would like to compare level 3 with level 1, and also level 3 with level 2
(but not level 2 with level 1) then you can specify:
\code{comparisons = list(c(3, 3), c(1, 2))}
}
\examples{
# Adjusted for covariates meal.cal and age and adjusted for stratification by strata:
robin_surv(
  formula = Surv(time, status) ~ meal.cal + age + strata(strata),
  data = surv_data,
  treatment = sex ~ pb(strata)
)

# Adjusted for stratification by strata and ecog but not for covariates:
robin_surv(
  formula = Surv(time, status) ~ 1 + strata(strata, ecog),
  data = surv_data,
  treatment = sex ~ sr(1)
)

# Unadjusted for covariates and stratification:
robin_surv(
  formula = Surv(time, status) ~ 1,
  data = surv_data,
  treatment = sex ~ sr(1)
)
}
\seealso{
\link{surv_effect_methods} for S3 methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/treatment_effect.R
\name{treatment_effect}
\alias{treatment_effect}
\alias{difference}
\alias{risk_ratio}
\alias{odds_ratio}
\alias{log_risk_ratio}
\alias{log_odds_ratio}
\title{Treatment Effect}
\usage{
treatment_effect(
  object,
  pair = pairwise(names(object$estimate)),
  eff_measure,
  eff_jacobian = eff_jacob(eff_measure),
  contrast_name,
  ...
)

difference(object, ...)

risk_ratio(object, ...)

odds_ratio(object, ...)

log_risk_ratio(object, ...)

log_odds_ratio(object, ...)
}
\arguments{
\item{object}{Object from which to obtain treatment effect.}

\item{pair}{(\code{contrast}) Contrast choices.}

\item{eff_measure}{(\code{function}) Treatment effect measurement function.}

\item{eff_jacobian}{(\code{function}) Treatment effect jacobian function.}

\item{contrast_name}{(\code{string}) Name of the contrast.}

\item{...}{Additional arguments for variance.}
}
\value{
A list of \code{treatment_effect} object with following elements:
\itemize{
\item \code{estimate}: estimate of the treatment effect.
\item \code{pair}: \code{contrast} object indicating the pairwise treatment effect.
\item \code{contrast}: name of the contrast function.
\item \code{euqal_val}: the value for no treatment effect given the contrast.
\item \code{variance}: the variance of the treatment effect.
\item \code{jacobian}: the Jacobian matrix.
\item \code{contrast_mat}: contrast summary matrix.
}
}
\description{
Obtain treatment effect and variance from counter-factual prediction
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival.R
\name{robin_surv_comparison}
\alias{robin_surv_comparison}
\title{Log Hazard Ratio Estimation and Log-Rank Test via Score Function}
\usage{
robin_surv_comparison(
  score_fun,
  vars,
  data,
  exp_level,
  control_level,
  contrast,
  control = surv_control(),
  unadj_score_fun = NULL,
  ...
)
}
\arguments{
\item{score_fun}{(\code{function}) The log-rank score function to be used for both estimation and testing.}

\item{vars}{(\code{list}) A list containing \code{levels}, \code{treatment}, and \code{covariates}.}

\item{data}{(\code{data.frame}) The data frame containing the survival data.}

\item{exp_level}{(\code{count}) Level of the experimental treatment arm.}

\item{control_level}{(\code{count}) Level of the control treatment arm.}

\item{unadj_score_fun}{(\code{function} or \code{NULL}) Optional unadjusted score function, see details.}

\item{...}{Additional arguments passed to \code{score_fun}.}
}
\value{
A list containing:
\itemize{
\item \code{estimate}: The estimated log hazard ratio.
\item \code{se}: The standard error of the estimated log hazard ratio.
\item \code{hr_n}: The number of observations used in the estimation.
\item \code{hr_sigma_l2}: The variance of the log-rank statistic used in the estimation.
\item \code{test_stat}: The log-rank test statistic.
\item \code{p_value}: The two-sided p-value of the log-rank test.
\item \code{test_score}: The log-rank score statistic.
\item \code{test_n}: The number of observations used in the log-rank test.
\item \code{test_sigma_l2}: The variance of the log-rank statistic used in the log-rank test.
}
}
\description{
This function combines the estimation of the log hazard ratio and the log-rank test
using a score function. Only two treatment arms are being compared and the \code{data} is subset accordingly.
}
\details{
If an unadjusted score function is provided in \code{unadj_score_fun}, then it is used to estimate the
log hazard ratio first. This unadjusted log hazard ratio estimate is then passed on to the adjusted
score function \code{score_fun} as \code{theta_hat}. This is required when the score function is adjusted for covariates.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{update_levels}
\alias{update_levels}
\title{Update levels in a contrast pair}
\usage{
update_levels(pair, levels)
}
\description{
Update levels in a contrast pair
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bias.R
\name{bias}
\alias{bias}
\title{Prediction Bias}
\usage{
bias(residual, treatment, group_idx)
}
\arguments{
\item{residual}{(\code{numeric}) residuals.}

\item{treatment}{(\code{factor}) treatment.}

\item{group_idx}{(\code{list} of \code{integer}) indices for each stratum group.}
}
\value{
Numeric matrix of bias in each stratum.
}
\description{
Obtain prediction bias within each stratum.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variance_anhecova.R
\name{vcovG}
\alias{vcovG}
\title{Generalized Covariance (ANHECOVA)}
\usage{
vcovG(x, decompose = TRUE, ...)
}
\arguments{
\item{x}{(\code{prediction_cf}) Counter-factual prediction.}

\item{decompose}{(\code{flag}) whether to use decompose method to calculate the variance.}

\item{...}{Not used.}
}
\value{
Named covariance matrix.
}
\description{
Generalized Covariance (ANHECOVA)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival_score.R
\name{survival_score_functions}
\alias{survival_score_functions}
\alias{h_lr_score_no_strata_no_cov}
\alias{h_lr_score_strat}
\alias{h_lr_score_cov}
\alias{h_lr_score_strat_cov}
\title{Log-Rank Score Functions for Survival Analysis}
\usage{
h_lr_score_no_strata_no_cov(
  theta,
  df,
  treatment,
  time,
  status,
  randomization_strata = character(),
  n = nrow(df),
  use_ties_factor = TRUE,
  calculate_variance = TRUE,
  check_rand_strat_warning = FALSE
)

h_lr_score_strat(
  theta,
  df,
  treatment,
  time,
  status,
  strata,
  randomization_strata = character(),
  use_ties_factor = TRUE,
  calculate_variance = TRUE,
  check_rand_strat_warning = FALSE
)

h_lr_score_cov(
  theta,
  df,
  treatment,
  time,
  status,
  model,
  randomization_strata = character(),
  theta_hat = theta,
  use_ties_factor = TRUE,
  hr_se_plugin_adjusted = TRUE,
  calculate_variance = TRUE,
  check_rand_strat_warning = FALSE
)

h_lr_score_strat_cov(
  theta,
  df,
  treatment,
  time,
  status,
  strata,
  model,
  randomization_strata = character(),
  theta_hat = theta,
  use_ties_factor = TRUE,
  hr_se_plugin_adjusted = TRUE,
  calculate_variance = TRUE,
  check_rand_strat_warning = FALSE
)
}
\arguments{
\item{theta}{(\code{number}) The assumed log hazard ratio of the second vs. the first level of the
treatment arm variable.}

\item{df}{(\code{data.frame}) The data frame containing the survival data.}

\item{treatment}{(\code{string}) The name of the treatment arm variable in \code{df}. It should be a factor with
two levels, where the first level is the reference group.}

\item{time}{(\code{string}) The name of the time variable in \code{df}, representing the survival time.}

\item{status}{(\code{string}) The name of the status variable in \code{df}, with 0 for censored and 1 for event.}

\item{randomization_strata}{(\code{character}) The names of the randomization strata variables in \code{df}.
These are used to check whether the means of the covariate adjustment residuals are unbiased across
these strata.}

\item{n}{(\code{count}) The number of observations. Note that this can be higher than the number of rows
when used in stratified analyses computations.}

\item{use_ties_factor}{(\code{flag}) Whether to use the ties factor in the variance calculation. This is used
when calculating the score test statistic, but not when estimating the log hazard ratio.}

\item{calculate_variance}{(\code{flag}) Whether to calculate the variance. This is useful to avoid
unnecessary computations when only the score function value is needed, e.g., during root finding.}

\item{strata}{(\code{character}) The names of the strata variables in \code{df}, which must be factors.}

\item{model}{(\code{formula}) The model formula for covariate adjustment, e.g., \code{~ cov1 + cov2}.}

\item{theta_hat}{(\code{number}) The estimated log hazard ratio when not adjusting for covariates.}

\item{hr_se_plugin_adjusted}{(\code{flag}) Defines the method for calculating the standard error of the
log hazard ratio estimate when adjusting for covariates, see details.}
}
\value{
The score function value(s), with the following attributes:
\itemize{
\item \code{sigma_l2}: The variance of the log-rank statistic.
\item \code{se_theta_l}: The corresponding standard error term for the log hazard ratio.
\item \code{n}: The number of observations used in the calculation.
}
}
\description{
These functions compute the log-rank score statistics for a survival analysis.
Depending on the function, these are stratified and/or adjusted for covariates.
}
\details{
\itemize{
\item The \code{hr_se_plugin_adjusted} flag is relevant only for the standard error of the covariate adjusted
log hazard ratio estimate: When \code{TRUE}, the adjusted hazard ratio estimate
is plugged in into the variance formula, as per the original publication.
On the other hand, when \code{FALSE}, the unadjusted estimate is used instead.  This is explained in more
detail in the vignette "Survival Analysis with RobinCar2" in Section "Covariate adjusted analysis without strata".
\item Note that for the not covariate adjusted score functions, these also work
with a \code{numeric} \code{theta} vector of length > 1.
}
}
\section{Functions}{
\itemize{
\item \code{h_lr_score_no_strata_no_cov()}: without strata or covariates.

\item \code{h_lr_score_strat()}: with strata but without covariates.

\item \code{h_lr_score_cov()}: with covariates but without strata.

\item \code{h_lr_score_strat_cov()}: with strata and covariates.

}}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival_cov_adj.R
\name{get_lm_results}
\alias{get_lm_results}
\alias{h_get_lm_results}
\alias{h_get_strat_lm_results}
\title{Calculate Coefficient Estimates and Corresponding Residuals from Linear Model Input}
\usage{
h_get_lm_results(lm_input)

h_get_strat_lm_results(strat_lm_input)
}
\arguments{
\item{lm_input}{(\code{list}) A list containing the linear model input data for each treatment arm, as returned by
\code{\link[=h_get_lm_input]{h_get_lm_input()}}.}

\item{strat_lm_input}{(\code{list}) A list containing the linear model input data
for each treatment arm and including the \code{.strata} column in the design matrix,
as returned by \code{\link[=h_get_strat_lm_input]{h_get_strat_lm_input()}}.}
}
\value{
A list with:
\itemize{
\item \code{beta_est}: the coefficient estimates for each treatment arm.
\item \code{residuals}: the corresponding residuals for each treatment arm.
}
}
\description{
Calculate the coefficient estimates for each treatment arm from the linear model input data.
Also returns the corresponding residuals.
}
\section{Functions}{
\itemize{
\item \code{h_get_lm_results()}: Calculate the coefficient estimates for the overall data set.

\item \code{h_get_strat_lm_results()}: Calculate the coefficient estimates using the stratified input.

}}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival.R
\name{surv_control}
\alias{surv_control}
\title{Control Survival Analysis Root Finding}
\usage{
surv_control(tol = .Machine$double.eps^0.25, maxiter = 1000, trace = 0)
}
\arguments{
\item{tol}{(\code{number}) The desired accuracy, passed to \code{\link[stats:uniroot]{stats::uniroot()}}.}

\item{maxiter}{(\code{count}) The maximum number of iterations, passed to \code{\link[stats:uniroot]{stats::uniroot()}}.}

\item{trace}{(\code{count}) Tracing level, passed to \code{\link[stats:uniroot]{stats::uniroot()}}.}
}
\value{
A named list with elements \code{tol}, \code{maxiter}, and \code{trace}.
}
\description{
Control parameters for the root-finding algorithm used by \code{\link[=robin_surv]{robin_surv()}} to estimate
the log hazard ratio.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{h_prep_survival_input}
\alias{h_prep_survival_input}
\title{Prepare Survival Input}
\usage{
h_prep_survival_input(formula, data, treatment)
}
\arguments{
\item{formula}{(\code{formula}) with a left hand side of the form \code{Surv(time, status)} and a right hand side
defining optional covariates or just \code{1} if there are no covariates.}

\item{data}{(\code{data.frame}) containing the variables in the formula.}

\item{treatment}{(\code{string} or \code{formula}) string name of the treatment, or a formula.}
}
\value{
A list containing the following elements:
\itemize{
\item \code{data}: The potentially updated data set.
\item \code{time}: Name of the time variable.
\item \code{status}: Name of the status variable.
\item \code{treatment}: Name of the treatment variable.
\item \code{strata}: Name of the strata variable.
\item \code{schema}: Randomization schema.
\item \code{randomization_strata}: Names of the randomization strata variables.
\item \code{covariates}: Names of the covariates in the model.
\item \code{model}: A formula only including the covariates, but not treatment or strata variables.
\item \code{n_levels}: Number of treatment levels.
\item \code{levels}: Names of the treatment levels.
}
}
\description{
Prepare Survival Input
}
\details{
Note that \code{formula} can also contain an externally defined \link[survival:Surv]{survival::Surv}
object. In this case, the \code{time} and \code{status} variables are extracted
and added to the \code{data} input. Note that it is up to the user to ensure that in this
case the column binding is correct, i.e., that the rows of the \code{data} match
with the rows of the \code{Surv} object. In addition, the same named variables must not appear
in both the \code{data} and the \code{Surv} object, to avoid ambiguity (this is a difference
vs. the behavior of \code{\link[survival:coxph]{survival::coxph()}} for better transparency).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict_couterfactual.R
\name{predict_counterfactual}
\alias{predict_counterfactual}
\title{Counterfactual Prediction}
\usage{
predict_counterfactual(fit, treatment, data, vcov, vcov_args, ...)
}
\arguments{
\item{fit}{fitted object.}

\item{treatment}{(\code{formula}) formula of form treatment ~ strata(s).}

\item{data}{(\code{data.frame}) raw dataset.}

\item{vcov}{(\code{function} or \code{character}) variance function or name.}

\item{vcov_args}{(\code{list}) additional arguments for variance function.}

\item{...}{Additional arguments for methods.}
}
\value{
List of class \code{prediction_cf} containing following elements:
\itemize{
\item \code{estimate}: predicted marginal mean.
\item \code{residual}: residual of the bias-corrected prediction.
\item \code{predictions}: all predicted values.
\item \code{predictions_liner}: linear predictions.
\item \code{schema}: randomization schema.
\item \code{response}: response value.
\item \code{fit}: fitted model.
\item \code{model_matrix}: model matrix.
\item \code{treatment_formula}: treatment assignment and randomization formula.
\item \code{treatment}: treatment value.
\item \code{group_idx}: group index based on the stratification.
\item \code{variance}: estimated variance of the marginal mean.
\item \code{variance_name}: name of the variance.
}
}
\description{
Obtain counterfactual prediction of a fit.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival.R
\name{survival_comparison_functions}
\alias{survival_comparison_functions}
\alias{robin_surv_no_strata_no_cov}
\alias{robin_surv_strata}
\alias{robin_surv_cov}
\alias{robin_surv_strata_cov}
\title{Survival Comparison Functions}
\usage{
robin_surv_no_strata_no_cov(
  vars,
  data,
  exp_level,
  control_level,
  contrast,
  control = surv_control(),
  check_rand_strat_warning = FALSE
)

robin_surv_strata(
  vars,
  data,
  exp_level,
  control_level,
  contrast,
  control = surv_control(),
  check_rand_strat_warning = FALSE
)

robin_surv_cov(
  vars,
  data,
  exp_level,
  control_level,
  contrast,
  control = surv_control(),
  ...
)

robin_surv_strata_cov(
  vars,
  data,
  exp_level,
  control_level,
  contrast,
  control = surv_control(),
  ...
)
}
\arguments{
\item{vars}{(\code{list}) A list containing \code{levels}, \code{treatment}, and \code{covariates}.}

\item{data}{(\code{data.frame}) The data frame containing the survival data.}

\item{exp_level}{(\code{count}) Level of the experimental treatment arm.}

\item{control_level}{(\code{count}) Level of the control treatment arm.}

\item{...}{Additional arguments passed to \code{score_fun}.}
}
\value{
See \code{\link[=robin_surv_comparison]{robin_surv_comparison()}}.
}
\description{
These are simple wrappers around \code{\link[=robin_surv_comparison]{robin_surv_comparison()}} called with the corresponding log-rank score
functions.
}
\section{Functions}{
\itemize{
\item \code{robin_surv_no_strata_no_cov()}: without strata and without covariates, based on
\code{\link[=h_lr_score_no_strata_no_cov]{h_lr_score_no_strata_no_cov()}}.

\item \code{robin_surv_strata()}: without strata and without covariates, based on
\code{\link[=h_lr_score_strat]{h_lr_score_strat()}}.

\item \code{robin_surv_cov()}: without strata and without covariates, based on
\code{\link[=h_lr_score_cov]{h_lr_score_cov()}} and \code{\link[=h_lr_score_no_strata_no_cov]{h_lr_score_no_strata_no_cov()}} (which is used to find the unadjusted
log hazard ratio estimate).

\item \code{robin_surv_strata_cov()}: with strata and covariates, based on
\code{\link[=h_lr_score_strat_cov]{h_lr_score_strat_cov()}} and \code{\link[=h_lr_score_strat]{h_lr_score_strat()}} (which is used to find the unadjusted
log hazard ratio estimate).

}}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival.R
\name{h_events_table}
\alias{h_events_table}
\title{Prepare Events Table}
\usage{
h_events_table(data, vars)
}
\arguments{
\item{data}{(\code{data.frame}) The data frame containing the survival data.}

\item{vars}{(\code{list}) A list containing the treatment, time, status, and strata variables.}
}
\value{
A data frame with columns for the treatment, strata, number of patients, and number of events.
}
\description{
This function creates a data frame summarizing the number of patients and events
for each treatment arm and stratification factor.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robin_lm.R
\name{h_interaction}
\alias{h_interaction}
\title{Evaluate if Interaction Exists}
\usage{
h_interaction(formula, treatment)
}
\arguments{
\item{formula}{(\code{formula}) the formula for model fitting.}

\item{treatment}{(\code{formula}) the formula for treatment assignment.}
}
\description{
Evaluate if Interaction Exists
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variance_anhecova.R
\name{h_get_erb}
\alias{h_get_erb}
\title{Obtain Adjustment for Covariance Matrix}
\usage{
h_get_erb(resi, group_idx, trt, pi, randomization)
}
\arguments{
\item{resi}{(\code{numeric}) vector of residuals.}

\item{group_idx}{(\code{list} of \code{integer}) index for each groups.}

\item{trt}{(\code{factor}) of treatment assignment.}

\item{pi}{(\code{numeric}) proportion of treatment assignment.}

\item{randomization}{(\code{string}) name of the randomization schema.}
}
\description{
Obtain Adjustment for Covariance Matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{h_get_vars}
\alias{h_get_vars}
\title{Extract Variable Names}
\usage{
h_get_vars(treatment)
}
\arguments{
\item{treatment}{(\code{string} or \code{formula}) string name of the treatment, or a formula.}
}
\value{
A list of three elements, \code{treatment}, \code{schema} and \code{strata}.
}
\description{
Extract Variable Names
}
\details{
Extract the formula elements, including \code{treatment}, \code{schema} and \code{strata}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{h_n_events_per_time}
\alias{h_n_events_per_time}
\title{Count Number of Events per Unique Event Time}
\usage{
h_n_events_per_time(df, time, status)
}
\arguments{
\item{df}{(\code{data.frame}) containing the survival data.}

\item{time}{(\code{string}) name of the time variable.}

\item{status}{(\code{string}) name of the status variable, where 1 indicates an event and 0 indicates censoring.}
}
\value{
A \code{data.frame} with two columns: \code{time} and \code{n_events}, where \code{n_events} is the
number of events at each time point.
}
\description{
This function counts the number of events at each unique event time point in a survival dataset.
}
\details{
If there are no events in the dataset, it returns an empty \code{data.frame}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{surv_data}
\alias{surv_data}
\title{Survival Example Data}
\format{
An object of class \code{data.frame} with 228 rows and 12 columns.
}
\source{
The data is generated by the \code{create_surv_data.R} script.
}
\usage{
surv_data
}
\description{
This dataset contains survival data from the \code{survival}
package's \link[survival:lung]{survival::lung} dataset, modified to include factors for
\code{sex} and \code{strata}, as well as a binary \code{status} variable which is 1
for death and 0 for censored.
}
\keyword{dataset}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival_cov_adj.R
\name{derived_outcome_vals}
\alias{derived_outcome_vals}
\alias{h_derived_outcome_vals}
\alias{h_strat_derived_outcome_vals}
\title{Derive Outcome Values Based on Log Hazard Ratio}
\usage{
h_derived_outcome_vals(
  theta,
  df,
  treatment,
  time,
  status,
  covariates,
  randomization_strata,
  n = nrow(df)
)

h_strat_derived_outcome_vals(
  theta,
  df,
  treatment,
  time,
  status,
  strata,
  covariates,
  randomization_strata
)
}
\arguments{
\item{theta}{(\code{number}) The assumed log hazard ratio of the second vs. the first level of the
treatment arm variable.}

\item{df}{(\code{data.frame}) The data frame containing the survival data.}

\item{treatment}{(\code{string}) The name of the treatment arm variable in \code{df}. It should be a factor with
two levels, where the first level is the reference group.}

\item{time}{(\code{string}) The name of the time variable in \code{df}, representing the survival time.}

\item{status}{(\code{string}) The name of the status variable in \code{df}, with 0 for censored and 1 for event.}

\item{covariates}{(\code{character}) The column names in \code{df} to be used for covariate adjustment.}

\item{randomization_strata}{(\code{character}) The names of the randomization strata variables in \code{df}.
These are used to check whether the means of the covariate adjustment residuals are unbiased across
these strata.}

\item{n}{(\code{count}) The number of observations. Note that this can be higher than the number of rows
when used in stratified analyses computations.}

\item{strata}{(\code{character}) The names of the strata variables in \code{df}, which must be factors.}
}
\value{
A data frame containing the same data as the input \code{df}, but restructured with standardized column names
\code{index}, \code{treatment}, \code{time}, \code{status}, the covariates and randomization stratification variables,
and an additional column \code{O_hat} containing the derived outcome values.
For the stratified version, the computations are done separately by stratum, and
the resulting \code{data.frame} contains an additional \code{.stratum} column indicating the stratum number.
}
\description{
Compute the derived outcome values based on a given log hazard ratio.
}
\details{
Please note that the \code{covariates} and \code{randomization_strata} must not include \code{index},
\code{treatment}, \code{time}, \code{status} to avoid naming conflicts.
}
\section{Functions}{
\itemize{
\item \code{h_derived_outcome_vals()}: calculates the derived outcome values for the overall data set.

\item \code{h_strat_derived_outcome_vals()}: calculates the derived outcome values for each stratum separately.

}}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bias.R
\name{h_unbiased_means_across_strata}
\alias{h_unbiased_means_across_strata}
\title{Check Unbiased Means of Residuals Across Randomization Strata and Treatment Groups}
\usage{
h_unbiased_means_across_strata(
  residuals_per_group,
  df,
  randomization_strata,
  eps = sqrt(.Machine$double.eps)
)
}
\arguments{
\item{residuals_per_group}{(\code{list} of \code{numeric}) A named list of numeric vectors containing residuals for each
of the two treatment groups.}

\item{df}{(\code{data.frame}) The data frame containing the \code{treatment} and randomization strata variables, produced by
\code{\link[=h_derived_outcome_vals]{h_derived_outcome_vals()}} or \code{\link[=h_strat_derived_outcome_vals]{h_strat_derived_outcome_vals()}}.}

\item{randomization_strata}{(\code{character}) A character vector of names of the randomization strata variables
in \code{df}.}

\item{eps}{(\code{numeric}) A small tolerance value to determine if means are close to zero.}
}
\value{
\code{TRUE} if the means of residuals across randomization strata are within the specified tolerance
for both treatment groups, \code{FALSE} otherwise.
}
\description{
This function checks whether the means of residuals are approximately zero
across specified randomization strata for each of the two treatment groups.
It is used in \code{\link[=robin_surv]{robin_surv()}} downstream functions to verify the correct inclusion of randomization strata
variables in the analysis survival model.
}
\seealso{
\code{\link[=bias]{bias()}} for the underlying bias (means of residuals) calculation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/treatment_effect.R, R/utils.R
\name{h_diff}
\alias{h_diff}
\alias{h_jac_diff}
\alias{h_risk_ratio}
\alias{h_jac_risk_ratio}
\alias{h_odds_ratio}
\alias{h_jac_odds_ratio}
\alias{h_log_risk_ratio}
\alias{h_jac_log_risk_ratio}
\alias{h_log_odds_ratio}
\alias{h_jac_log_odds_ratio}
\alias{eff_jacob}
\alias{pairwise}
\alias{against_ref}
\alias{custom_contrast}
\title{Contrast Functions and Jacobians}
\usage{
h_diff(x, y)

h_jac_diff(x, y)

h_risk_ratio(x, y)

h_jac_risk_ratio(x, y)

h_odds_ratio(x, y)

h_jac_odds_ratio(x, y)

h_log_risk_ratio(x, y)

h_jac_log_risk_ratio(x, y)

h_log_odds_ratio(x, y)

h_jac_log_odds_ratio(x, y)

eff_jacob(f)

pairwise(levels, x = levels)

against_ref(levels, ref = levels[1], x = tail(levels, -1))

custom_contrast(levels, x, y)
}
\arguments{
\item{x}{(\code{vector}) A vector of treatment levels.}

\item{y}{(\code{vector}) A vector of treatment levels.}

\item{f}{(\code{function}) Function with argument x and y to compute treatment effect.}

\item{levels}{(\code{character}) Levels of the treatment.}

\item{ref}{(\code{string} or \code{int}) Reference level.}
}
\value{
Vector of contrasts, or matrix of jacobians.

A list of \code{contrast} object with following elements:
\itemize{
\item Index of the treatment group.
\item Index of the reference group.
Additional attributes include \code{levels} and \code{max_levels} indicating the
names of the treatment levels and the maximum number of levels.
}
}
\description{
Contrast Functions and Jacobians

Create Contrast of Pairs
}
\examples{
h_diff(1:3, 4:6)
h_jac_risk_ratio(1:3, 4:6)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prediction_cf.R
\name{prediction_cf_methods}
\alias{prediction_cf_methods}
\alias{print.prediction_cf}
\title{S3 Methods for \code{prediction_cf}}
\usage{
\method{print}{prediction_cf}(x, level = 0.95, ...)
}
\arguments{
\item{x}{(\code{prediction_cf})\cr the obtained counter-factual prediction object.}

\item{level}{(\code{number})\cr the significance level.}
}
\value{
No return value.
}
\description{
S3 Methods for \code{prediction_cf}
}
\section{Functions}{
\itemize{
\item \code{print(prediction_cf)}: prints the prediction_cf object.

}}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_data.R
\name{find_data}
\alias{find_data}
\title{Find Data in a Fit}
\usage{
find_data(fit, ...)
}
\arguments{
\item{fit}{A fit object.}

\item{...}{Additional arguments.}
}
\value{
A data frame used in the fit.
}
\description{
Find Data in a Fit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robin_lm.R
\name{robin_lm}
\alias{robin_lm}
\title{Covariate adjusted lm model}
\usage{
robin_lm(
  formula,
  data,
  treatment,
  vcov = "vcovG",
  vcov_args = list(),
  pair,
  ...
)
}
\arguments{
\item{formula}{(\code{formula}) A formula of analysis.}

\item{data}{(\code{data.frame}) Input data frame.}

\item{treatment}{(\code{formula} or \code{character(1)}) A formula of treatment assignment or assignment by stratification,
or a string name of treatment assignment.}

\item{vcov}{(\code{function}) A function to calculate the variance-covariance matrix of the treatment effect,
including \code{vcovHC} and \code{vcovG}. The default is 'vcovG'.}

\item{vcov_args}{(\code{list}) Additional arguments passed to \code{vcov}.}

\item{pair}{Pairwise treatment comparison.}

\item{...}{Additional arguments passed to \code{lm}.}
}
\value{
A robin_output object, with \code{marginal_mean} and \code{contrast} components.
}
\description{
Covariate adjusted lm model
}
\examples{
robin_lm(
  y ~ treatment * s1,
  data = glm_data,
  treatment = treatment ~ s1
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{sum_vectors_in_list}
\alias{sum_vectors_in_list}
\title{Sum vectors in a list}
\usage{
sum_vectors_in_list(lst)
}
\description{
Sum vectors in a list
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robin_glm.R
\name{robin_glm}
\alias{robin_glm}
\title{Covariate adjusted glm model}
\usage{
robin_glm(
  formula,
  data,
  treatment,
  contrast = c("difference", "risk_ratio", "odds_ratio", "log_risk_ratio",
    "log_odds_ratio"),
  contrast_jac = NULL,
  vcov = "vcovG",
  family = gaussian(),
  vcov_args = list(),
  pair,
  ...
)
}
\arguments{
\item{formula}{(\code{formula}) A formula of analysis.}

\item{data}{(\code{data.frame}) Input data frame.}

\item{treatment}{(\code{formula} or \code{character(1)}) A formula of treatment assignment or assignment by stratification,
or a string name of treatment assignment.}

\item{contrast}{(\code{function} or \code{character(1)}) A function to calculate the treatment effect, or character of
"difference", "risk_ratio", "odds_ratio" for default contrasts.}

\item{contrast_jac}{(\code{function}) A function to calculate the Jacobian of the contrast function. Ignored if using
default contrasts.}

\item{vcov}{(\code{function}) A function to calculate the variance-covariance matrix of the treatment effect,
including \code{vcovHC} and \code{vcovG}.}

\item{family}{(\code{family}) A family object of the glm model.}

\item{vcov_args}{(\code{list}) Additional arguments passed to \code{vcov}.}

\item{pair}{Pairwise treatment comparison.}

\item{...}{Additional arguments passed to \code{glm} or \code{glm.nb}.}
}
\value{
A robin_output object, with \code{marginal_mean} and \code{contrast} components.
}
\description{
Covariate adjusted glm model
}
\details{
If family is \code{MASS::negative.binomial(NA)}, the function will use \code{MASS::glm.nb} instead of \code{glm}.
}
\examples{
robin_glm(
  y ~ treatment * s1,
  data = glm_data,
  treatment = treatment ~ s1, contrast = "difference"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{h_confint}
\alias{h_confint}
\title{Confidence interval calculations which are common across effect results.}
\usage{
h_confint(x, parm, level = 0.95, transform, include_se = FALSE, ...)
}
\description{
Confidence interval calculations which are common across effect results.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival_cov_adj.R
\name{get_lm_input}
\alias{get_lm_input}
\alias{h_get_lm_input}
\alias{h_get_strat_lm_input}
\title{Get Linear Model Input Data}
\usage{
h_get_lm_input(df, model)

h_get_strat_lm_input(df_with_stratum, model)
}
\arguments{
\item{df}{(\code{data.frame}) Including the covariates needed for the \code{model}, as well as the derived outcome \code{O_hat}
and the \code{treatment} factor.}

\item{model}{(\code{formula}) The right-hand side only model formula.}

\item{df_with_stratum}{(\code{data.frame}) A data frame with an additional column for the stratum, as returned by
\code{\link[=h_strat_derived_outcome_vals]{h_strat_derived_outcome_vals()}}.}
}
\value{
A list containing for each element of the \code{treatment} factor a list with the
corresponding model matrix \code{X} and the response vector \code{y}. For the stratified version,
the model matrix \code{X} includes the \code{.stratum} column.
}
\description{
Prepare the input data for a linear model based on the provided data frame and model formula.
}
\section{Functions}{
\itemize{
\item \code{h_get_lm_input()}: Get the linear model input data for the overall data set.

\item \code{h_get_strat_lm_input()}: Get the linear model input data with attached stratum column.

}}
\keyword{internal}
